<?php 
/**
 * Defines a simple  english language lexer for correctwriting question type.
 *
 *
 * @copyright &copy; 2011  Oleg Sychev
 * @author Oleg Sychev, Dmitriy Mamontov, Sergey Pashaev Volgograd State Technical University
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package questions
 */
require_once($CFG->dirroot.'/blocks/formal_langs/tokens_base.php');
require_once($CFG->dirroot.'/blocks/formal_langs/language_base.php');
require_once($CFG->dirroot.'/question/type/poasquestion/jlex.php');
require_once($CFG->dirroot.'/blocks/formal_langs/c_language_tokens.php');
class block_formal_langs_language_c_language extends block_formal_langs_predefined_language
{
    public function __construct() {
        parent::__construct(null,null);
    }
    public function name() {
        return 'c_language';
    }    
}
function block_formal_langs_octal_to_decimal_char($matches) {
    $code = $matches[0];
    $code = octdec($code);
    return chr(intval($code));
}
function block_formal_langs_hex_to_decimal_char($matches) {
    $code = $matches[0];
    $code = hexdec($code);
    $string = '';
    if (strlen($matches[0]) == 2) {
        $string = chr(intval($code));
    } else {
        $string = mb_convert_encoding('&#' . intval($code) . ';', 'UTF-8', 'HTML-ENTITIES');
    }
    return $string;
}


class block_formal_langs_predefined_c_language_lexer_raw extends JLexBase  {
	const YY_BUFFER_SIZE = 512;
	const YY_F = -1;
	const YY_NO_STATE = -1;
	const YY_NOT_ACCEPT = 0;
	const YY_START = 1;
	const YY_END = 2;
	const YY_NO_ANCHOR = 4;
	const YY_BOL = 256;
	var $YY_EOF = 257;

    // @var int number of  current parsed lexeme.
    private  $counter = 0;
    private  $errors  = array();
    // @var qtype_poasquestion_string  temporary string for buffer
    protected $statestring = null;
    // @var int line yyline for token
    protected $stateyyline = 0;
    // @var int column yycol for token
    protected $stateyycol = 0;
    // @var int end yyline
    protected $endyyline = 0;
    // @var int end yycolumn
    protected $endyycol  = 0;
    private function startbuffer() {
        $this->stateyyline = $this->yyline;
        $this->stateyycol = $this->yycol;
        $this->statestring = new qtype_poasquestion_string();
    }
    // Appends a symbol string to a buffer
    private function append($sym) {
        $this->advance_buffered_pos();
        $this->statestring->concatenate($sym);
    }
    // Returns buffer
    private function  buffer() {
        $result = $this->statestring;
        $this->statestring = null;
        return $result;
    }
    private function create_error($symbol) {
        $res = new block_formal_langs_lexical_error();
        $res->tokenindex = $this->counter;
        $a = new stdClass();
        $a->line = $this->yyline;
        $a->position = $this->yycol;
        $a->symbol = $symbol->string();
        $errormessage = 'clanguageunknownsymbol';
        if (mb_strlen($symbol) == 1) {
            if ($symbol[0] == '\'') {
                $errormessage = 'clanguageunmatchedsquote';
            }
            if ($symbol[0] == '"') {
                $errormessage = 'clanguageunmatchedquote';
            }
        }
        $res->errormessage = get_string($errormessage,'block_formal_langs',$a);
        $this->errors[] = $res;
    }
    private function create_buffer_error($symbol) {
        $res = new block_formal_langs_lexical_error();
        $res->tokenindex = $this->counter;
        $a = new stdClass();
        $a->line = $this->stateyyline;
        $a->position = $this->stateyycol;
        $a->symbol = $symbol->string();
        $res->errormessage = get_string('lexical_error_message','block_formal_langs',$a);
        $this->errors[] = $res;
    }
    public function get_errors() {
        return $this->errors;
    }
    private function create_token($class,$value) {
        // create token object
        $classname = 'block_formal_langs_c_token_' . $class;
        $res = new $classname(null, $class, $value, $this->return_pos(), $this->counter);
        // increase token count
        $this->counter++;
        return $res;
    }
    private function create_buffered_token($class,$value) {
            // create token object
            $classname = 'block_formal_langs_c_token_' . $class;
            $res = new $classname(null, $class, $value, $this->return_buffered_pos(), $this->counter);
            // increase token count
            $this->counter++;
            return $res;
    }
    private function is_white_space($string) {
        // Here we need to escape symbols, so double quotes are inavoidable
        $whitespace = array(' ', "\t", "\n", "\r", "f", "\v");
        return in_array($string[0], $whitespace);
    }
    // Enters state with buffered output
    private function enterbufferedstate($state) {
        $this->startbuffer();
        $this->append($this->yytext());
        $this->yybegin($state);
    }
    // Leaves state with buffered output
    private function leavebufferedstate($tokentype) {
        $this->append($this->yytext());
        $this->yybegin(self::YYINITIAL);
        return $this->create_buffered_token($tokentype,$this->buffer());
    }
    protected function check_and_create_character()
    {
        $result = $this->leavebufferedstate('character');
        $maxcharacterlength = 3;
        $value = $result->value();
        if ($value[0] == 'L')
            $maxcharacterlength = $maxcharacterlength + 1;
        if ( mb_strlen($value) > $maxcharacterlength) {
            $res = new block_formal_langs_lexical_error();
            $res->tokenindex = $this->counter - 1;
            $a = new stdClass();
            $a->line = $result->position()->linestart();
            $a->col = $result->position()->colstart();
            $a->symbol = $value;
            $res->errormessage = get_string('clanguagemulticharliteral','block_formal_langs',$a);
            $this->errors[] = $res;
        }
        return $result;
    }
    private function return_pos() {
        $begin_line = $this->yyline;
        $begin_col = $this->yycol;
        if(strpos($this->yytext(), '\n')) {
            $lines = explode("\n", $this->yytext());
            $num_lines = count($lines);
            $end_line = $begin_line + $num_lines - 1;
            $end_col = strlen($lines[$num_lines -1]) - 1;
        } else {
            $end_line = $begin_line;
            $end_col = $begin_col + strlen($this->yytext()) - 1;
        }
        $res = new block_formal_langs_node_position($begin_line, $end_line, $begin_col, $end_col);
        return $res;
    }
    private function return_buffered_pos() {
        $begin_line = $this->stateyyline;
        $begin_col = $this->stateyycol;
        $end_line =  $this->endyyline;
        $end_col =  $this->endyycol;
        $res = new block_formal_langs_node_position($begin_line, $end_line, $begin_col, $end_col);
        return $res;
    }
    private function advance_buffered_pos() {
        if(strpos($this->yytext(), '\n')) {
            $lines = explode("\n", $this->yytext());
            $num_lines = count($lines);
            $this->endyyline = $this->endyyline + $num_lines - 1;
            $this->endyycol = strlen($lines[$num_lines -1]) - 1;
        } else {
            $this->endyycol = $this->endyycol + strlen($this->yytext()) - 1;
        }
    }
	protected $yy_count_chars = true;
	protected $yy_count_lines = true;

	function __construct($stream) {
		parent::__construct($stream);
		$this->yy_lexical_state = self::YYINITIAL;
	}

	const MULTILINE_COMMENT = 3;
	const SINGLELINE_COMMENT = 4;
	const STRING = 2;
	const YYINITIAL = 0;
	const CHARACTER = 1;
	static $yy_state_dtrans = array(
		0,
		187,
		189,
		191,
		192
	);
	static $yy_acpt = array(
		/* 0 */ self::YY_NOT_ACCEPT,
		/* 1 */ self::YY_NO_ANCHOR,
		/* 2 */ self::YY_NO_ANCHOR,
		/* 3 */ self::YY_NO_ANCHOR,
		/* 4 */ self::YY_NO_ANCHOR,
		/* 5 */ self::YY_NO_ANCHOR,
		/* 6 */ self::YY_NO_ANCHOR,
		/* 7 */ self::YY_NO_ANCHOR,
		/* 8 */ self::YY_NO_ANCHOR,
		/* 9 */ self::YY_NO_ANCHOR,
		/* 10 */ self::YY_NO_ANCHOR,
		/* 11 */ self::YY_NO_ANCHOR,
		/* 12 */ self::YY_NO_ANCHOR,
		/* 13 */ self::YY_NO_ANCHOR,
		/* 14 */ self::YY_NO_ANCHOR,
		/* 15 */ self::YY_NO_ANCHOR,
		/* 16 */ self::YY_NO_ANCHOR,
		/* 17 */ self::YY_NO_ANCHOR,
		/* 18 */ self::YY_NO_ANCHOR,
		/* 19 */ self::YY_NO_ANCHOR,
		/* 20 */ self::YY_NO_ANCHOR,
		/* 21 */ self::YY_NO_ANCHOR,
		/* 22 */ self::YY_NO_ANCHOR,
		/* 23 */ self::YY_NO_ANCHOR,
		/* 24 */ self::YY_NO_ANCHOR,
		/* 25 */ self::YY_NO_ANCHOR,
		/* 26 */ self::YY_NO_ANCHOR,
		/* 27 */ self::YY_NO_ANCHOR,
		/* 28 */ self::YY_NO_ANCHOR,
		/* 29 */ self::YY_NO_ANCHOR,
		/* 30 */ self::YY_NO_ANCHOR,
		/* 31 */ self::YY_NO_ANCHOR,
		/* 32 */ self::YY_NO_ANCHOR,
		/* 33 */ self::YY_NO_ANCHOR,
		/* 34 */ self::YY_NO_ANCHOR,
		/* 35 */ self::YY_NO_ANCHOR,
		/* 36 */ self::YY_NO_ANCHOR,
		/* 37 */ self::YY_NO_ANCHOR,
		/* 38 */ self::YY_NO_ANCHOR,
		/* 39 */ self::YY_NO_ANCHOR,
		/* 40 */ self::YY_NO_ANCHOR,
		/* 41 */ self::YY_NO_ANCHOR,
		/* 42 */ self::YY_NO_ANCHOR,
		/* 43 */ self::YY_NO_ANCHOR,
		/* 44 */ self::YY_NO_ANCHOR,
		/* 45 */ self::YY_NO_ANCHOR,
		/* 46 */ self::YY_NO_ANCHOR,
		/* 47 */ self::YY_NO_ANCHOR,
		/* 48 */ self::YY_NO_ANCHOR,
		/* 49 */ self::YY_NO_ANCHOR,
		/* 50 */ self::YY_NO_ANCHOR,
		/* 51 */ self::YY_NO_ANCHOR,
		/* 52 */ self::YY_NO_ANCHOR,
		/* 53 */ self::YY_NO_ANCHOR,
		/* 54 */ self::YY_NO_ANCHOR,
		/* 55 */ self::YY_NO_ANCHOR,
		/* 56 */ self::YY_NO_ANCHOR,
		/* 57 */ self::YY_NO_ANCHOR,
		/* 58 */ self::YY_NO_ANCHOR,
		/* 59 */ self::YY_NO_ANCHOR,
		/* 60 */ self::YY_NO_ANCHOR,
		/* 61 */ self::YY_NO_ANCHOR,
		/* 62 */ self::YY_NO_ANCHOR,
		/* 63 */ self::YY_NO_ANCHOR,
		/* 64 */ self::YY_NO_ANCHOR,
		/* 65 */ self::YY_NO_ANCHOR,
		/* 66 */ self::YY_NO_ANCHOR,
		/* 67 */ self::YY_NO_ANCHOR,
		/* 68 */ self::YY_NO_ANCHOR,
		/* 69 */ self::YY_NO_ANCHOR,
		/* 70 */ self::YY_NO_ANCHOR,
		/* 71 */ self::YY_NO_ANCHOR,
		/* 72 */ self::YY_NO_ANCHOR,
		/* 73 */ self::YY_NO_ANCHOR,
		/* 74 */ self::YY_NO_ANCHOR,
		/* 75 */ self::YY_NO_ANCHOR,
		/* 76 */ self::YY_NO_ANCHOR,
		/* 77 */ self::YY_NO_ANCHOR,
		/* 78 */ self::YY_NO_ANCHOR,
		/* 79 */ self::YY_NO_ANCHOR,
		/* 80 */ self::YY_NO_ANCHOR,
		/* 81 */ self::YY_NO_ANCHOR,
		/* 82 */ self::YY_NO_ANCHOR,
		/* 83 */ self::YY_NO_ANCHOR,
		/* 84 */ self::YY_NO_ANCHOR,
		/* 85 */ self::YY_NO_ANCHOR,
		/* 86 */ self::YY_NO_ANCHOR,
		/* 87 */ self::YY_NO_ANCHOR,
		/* 88 */ self::YY_NO_ANCHOR,
		/* 89 */ self::YY_NO_ANCHOR,
		/* 90 */ self::YY_NO_ANCHOR,
		/* 91 */ self::YY_NO_ANCHOR,
		/* 92 */ self::YY_NO_ANCHOR,
		/* 93 */ self::YY_NO_ANCHOR,
		/* 94 */ self::YY_NO_ANCHOR,
		/* 95 */ self::YY_NO_ANCHOR,
		/* 96 */ self::YY_NO_ANCHOR,
		/* 97 */ self::YY_NO_ANCHOR,
		/* 98 */ self::YY_NO_ANCHOR,
		/* 99 */ self::YY_NO_ANCHOR,
		/* 100 */ self::YY_NO_ANCHOR,
		/* 101 */ self::YY_NO_ANCHOR,
		/* 102 */ self::YY_NO_ANCHOR,
		/* 103 */ self::YY_NO_ANCHOR,
		/* 104 */ self::YY_NO_ANCHOR,
		/* 105 */ self::YY_NO_ANCHOR,
		/* 106 */ self::YY_NO_ANCHOR,
		/* 107 */ self::YY_NO_ANCHOR,
		/* 108 */ self::YY_NO_ANCHOR,
		/* 109 */ self::YY_NO_ANCHOR,
		/* 110 */ self::YY_NO_ANCHOR,
		/* 111 */ self::YY_NOT_ACCEPT,
		/* 112 */ self::YY_NO_ANCHOR,
		/* 113 */ self::YY_NO_ANCHOR,
		/* 114 */ self::YY_NO_ANCHOR,
		/* 115 */ self::YY_NO_ANCHOR,
		/* 116 */ self::YY_NO_ANCHOR,
		/* 117 */ self::YY_NO_ANCHOR,
		/* 118 */ self::YY_NO_ANCHOR,
		/* 119 */ self::YY_NO_ANCHOR,
		/* 120 */ self::YY_NO_ANCHOR,
		/* 121 */ self::YY_NO_ANCHOR,
		/* 122 */ self::YY_NO_ANCHOR,
		/* 123 */ self::YY_NOT_ACCEPT,
		/* 124 */ self::YY_NO_ANCHOR,
		/* 125 */ self::YY_NO_ANCHOR,
		/* 126 */ self::YY_NO_ANCHOR,
		/* 127 */ self::YY_NO_ANCHOR,
		/* 128 */ self::YY_NO_ANCHOR,
		/* 129 */ self::YY_NOT_ACCEPT,
		/* 130 */ self::YY_NO_ANCHOR,
		/* 131 */ self::YY_NO_ANCHOR,
		/* 132 */ self::YY_NOT_ACCEPT,
		/* 133 */ self::YY_NO_ANCHOR,
		/* 134 */ self::YY_NO_ANCHOR,
		/* 135 */ self::YY_NOT_ACCEPT,
		/* 136 */ self::YY_NO_ANCHOR,
		/* 137 */ self::YY_NO_ANCHOR,
		/* 138 */ self::YY_NOT_ACCEPT,
		/* 139 */ self::YY_NO_ANCHOR,
		/* 140 */ self::YY_NO_ANCHOR,
		/* 141 */ self::YY_NOT_ACCEPT,
		/* 142 */ self::YY_NO_ANCHOR,
		/* 143 */ self::YY_NOT_ACCEPT,
		/* 144 */ self::YY_NO_ANCHOR,
		/* 145 */ self::YY_NOT_ACCEPT,
		/* 146 */ self::YY_NO_ANCHOR,
		/* 147 */ self::YY_NOT_ACCEPT,
		/* 148 */ self::YY_NO_ANCHOR,
		/* 149 */ self::YY_NOT_ACCEPT,
		/* 150 */ self::YY_NO_ANCHOR,
		/* 151 */ self::YY_NOT_ACCEPT,
		/* 152 */ self::YY_NO_ANCHOR,
		/* 153 */ self::YY_NOT_ACCEPT,
		/* 154 */ self::YY_NO_ANCHOR,
		/* 155 */ self::YY_NOT_ACCEPT,
		/* 156 */ self::YY_NO_ANCHOR,
		/* 157 */ self::YY_NOT_ACCEPT,
		/* 158 */ self::YY_NO_ANCHOR,
		/* 159 */ self::YY_NOT_ACCEPT,
		/* 160 */ self::YY_NO_ANCHOR,
		/* 161 */ self::YY_NOT_ACCEPT,
		/* 162 */ self::YY_NO_ANCHOR,
		/* 163 */ self::YY_NOT_ACCEPT,
		/* 164 */ self::YY_NO_ANCHOR,
		/* 165 */ self::YY_NOT_ACCEPT,
		/* 166 */ self::YY_NO_ANCHOR,
		/* 167 */ self::YY_NOT_ACCEPT,
		/* 168 */ self::YY_NO_ANCHOR,
		/* 169 */ self::YY_NOT_ACCEPT,
		/* 170 */ self::YY_NO_ANCHOR,
		/* 171 */ self::YY_NOT_ACCEPT,
		/* 172 */ self::YY_NO_ANCHOR,
		/* 173 */ self::YY_NOT_ACCEPT,
		/* 174 */ self::YY_NO_ANCHOR,
		/* 175 */ self::YY_NOT_ACCEPT,
		/* 176 */ self::YY_NO_ANCHOR,
		/* 177 */ self::YY_NOT_ACCEPT,
		/* 178 */ self::YY_NO_ANCHOR,
		/* 179 */ self::YY_NOT_ACCEPT,
		/* 180 */ self::YY_NO_ANCHOR,
		/* 181 */ self::YY_NOT_ACCEPT,
		/* 182 */ self::YY_NO_ANCHOR,
		/* 183 */ self::YY_NOT_ACCEPT,
		/* 184 */ self::YY_NOT_ACCEPT,
		/* 185 */ self::YY_NOT_ACCEPT,
		/* 186 */ self::YY_NOT_ACCEPT,
		/* 187 */ self::YY_NOT_ACCEPT,
		/* 188 */ self::YY_NOT_ACCEPT,
		/* 189 */ self::YY_NOT_ACCEPT,
		/* 190 */ self::YY_NOT_ACCEPT,
		/* 191 */ self::YY_NOT_ACCEPT,
		/* 192 */ self::YY_NOT_ACCEPT,
		/* 193 */ self::YY_NO_ANCHOR,
		/* 194 */ self::YY_NO_ANCHOR,
		/* 195 */ self::YY_NOT_ACCEPT,
		/* 196 */ self::YY_NOT_ACCEPT,
		/* 197 */ self::YY_NO_ANCHOR,
		/* 198 */ self::YY_NO_ANCHOR,
		/* 199 */ self::YY_NO_ANCHOR,
		/* 200 */ self::YY_NO_ANCHOR,
		/* 201 */ self::YY_NO_ANCHOR,
		/* 202 */ self::YY_NO_ANCHOR,
		/* 203 */ self::YY_NO_ANCHOR,
		/* 204 */ self::YY_NO_ANCHOR,
		/* 205 */ self::YY_NO_ANCHOR,
		/* 206 */ self::YY_NO_ANCHOR,
		/* 207 */ self::YY_NO_ANCHOR,
		/* 208 */ self::YY_NO_ANCHOR,
		/* 209 */ self::YY_NO_ANCHOR,
		/* 210 */ self::YY_NO_ANCHOR,
		/* 211 */ self::YY_NO_ANCHOR,
		/* 212 */ self::YY_NO_ANCHOR,
		/* 213 */ self::YY_NO_ANCHOR,
		/* 214 */ self::YY_NO_ANCHOR,
		/* 215 */ self::YY_NO_ANCHOR,
		/* 216 */ self::YY_NO_ANCHOR,
		/* 217 */ self::YY_NO_ANCHOR,
		/* 218 */ self::YY_NO_ANCHOR,
		/* 219 */ self::YY_NO_ANCHOR,
		/* 220 */ self::YY_NO_ANCHOR,
		/* 221 */ self::YY_NO_ANCHOR,
		/* 222 */ self::YY_NO_ANCHOR,
		/* 223 */ self::YY_NO_ANCHOR,
		/* 224 */ self::YY_NO_ANCHOR,
		/* 225 */ self::YY_NO_ANCHOR,
		/* 226 */ self::YY_NO_ANCHOR,
		/* 227 */ self::YY_NO_ANCHOR,
		/* 228 */ self::YY_NO_ANCHOR,
		/* 229 */ self::YY_NO_ANCHOR,
		/* 230 */ self::YY_NO_ANCHOR,
		/* 231 */ self::YY_NO_ANCHOR,
		/* 232 */ self::YY_NO_ANCHOR,
		/* 233 */ self::YY_NO_ANCHOR,
		/* 234 */ self::YY_NO_ANCHOR,
		/* 235 */ self::YY_NO_ANCHOR,
		/* 236 */ self::YY_NO_ANCHOR,
		/* 237 */ self::YY_NO_ANCHOR,
		/* 238 */ self::YY_NO_ANCHOR,
		/* 239 */ self::YY_NO_ANCHOR,
		/* 240 */ self::YY_NO_ANCHOR,
		/* 241 */ self::YY_NO_ANCHOR,
		/* 242 */ self::YY_NO_ANCHOR,
		/* 243 */ self::YY_NO_ANCHOR,
		/* 244 */ self::YY_NO_ANCHOR,
		/* 245 */ self::YY_NO_ANCHOR,
		/* 246 */ self::YY_NO_ANCHOR,
		/* 247 */ self::YY_NO_ANCHOR,
		/* 248 */ self::YY_NO_ANCHOR,
		/* 249 */ self::YY_NO_ANCHOR,
		/* 250 */ self::YY_NO_ANCHOR,
		/* 251 */ self::YY_NO_ANCHOR,
		/* 252 */ self::YY_NO_ANCHOR,
		/* 253 */ self::YY_NO_ANCHOR,
		/* 254 */ self::YY_NO_ANCHOR,
		/* 255 */ self::YY_NO_ANCHOR,
		/* 256 */ self::YY_NO_ANCHOR,
		/* 257 */ self::YY_NO_ANCHOR,
		/* 258 */ self::YY_NO_ANCHOR,
		/* 259 */ self::YY_NO_ANCHOR,
		/* 260 */ self::YY_NO_ANCHOR,
		/* 261 */ self::YY_NO_ANCHOR,
		/* 262 */ self::YY_NO_ANCHOR,
		/* 263 */ self::YY_NO_ANCHOR,
		/* 264 */ self::YY_NO_ANCHOR,
		/* 265 */ self::YY_NO_ANCHOR,
		/* 266 */ self::YY_NO_ANCHOR,
		/* 267 */ self::YY_NO_ANCHOR,
		/* 268 */ self::YY_NO_ANCHOR
	);
		static $yy_cmap = array(
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 66, 63, 63, 36, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 34, 50, 38, 33, 63, 46, 47, 62,
 57, 58, 45, 44, 55, 40, 42, 1, 28, 65, 65, 65, 65, 65, 65, 65, 27, 27, 56, 52,
 35, 43, 37, 61, 63, 30, 30, 30, 30, 39, 41, 26, 26, 26, 26, 26, 32, 26, 26, 26,
 26, 26, 26, 26, 26, 31, 26, 26, 29, 26, 26, 59, 64, 60, 48, 26, 63, 2, 6, 10,
 14, 8, 15, 19, 22, 13, 26, 9, 16, 17, 11, 5, 24, 26, 7, 12, 4, 3, 25, 21,
 18, 23, 20, 53, 49, 54, 51, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 0, 0,);

		static $yy_rmap = array(
 0, 1, 2, 3, 4, 5, 1, 1, 6, 7, 8, 9, 10, 11, 11, 1, 1, 1, 1, 12,
 1, 1, 1, 1, 1, 1, 1, 1, 1, 13, 14, 15, 16, 1, 1, 1, 17, 1, 18, 1,
 1, 1, 1, 19, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 13, 20, 21, 22, 1,
 1, 1, 13, 13, 13, 1, 1, 13, 13, 13, 1, 1, 1, 1, 23, 1, 1, 1, 1, 1,
 1, 1, 1, 1, 24, 1, 1, 1, 1, 25, 1, 1, 1, 1, 1, 1, 1, 1, 1, 26,
 1, 1, 1, 1, 27, 1, 1, 1, 1, 28, 1, 29, 30, 31, 32, 1, 1, 1, 1, 1,
 33, 34, 35, 36, 37, 38, 1, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51,
 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91,
 92, 93, 94, 95, 96, 97, 23, 98, 25, 99, 27, 100, 101, 102, 13, 103, 104, 105, 13, 106,
 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126,
 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146,
 147, 129, 148, 149, 150, 151, 152, 153, 154, 155, 151, 156, 157, 158, 159, 160, 161, 162, 13, 163,
 164, 165, 166, 167, 168, 169, 170, 171, 172,);

		static $yy_nxt = array(
array(
 1, 2, 3, 248, 256, 258, 260, 261, 262, 258, 263, 258, 264, 113, 125, 193, 265, 258, 258, 266,
 258, 267, 258, 258, 258, 268, 258, 4, 114, 258, 258, 258, 131, 5, 6, 112, -1, 124, 7, 258,
 130, 258, 8, 9, 133, 136, 139, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
 23, 24, 25, 6, 6, 4, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, 26, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 27, -1, 28, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 197, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, 126, -1, -1, -1, -1, 111, -1, -1, -1, -1, -1, -1, -1, 126, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 4, 4, -1, -1, 126, 126, -1, -1, -1, -1, -1, -1, 111,
 -1, -1, 31, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 4, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 129, -1, -1, -1, -1, 132, 135, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 35, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 43, 43, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, 138, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 43, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 44, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 49, -1, -1, -1, 50, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 51, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 52, -1, -1, -1, -1, -1, 53, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 54, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 23, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 223, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 143, -1, -1, -1, -1, -1, -1, 115, 115, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 43, 43, -1, -1, -1, 115, -1, -1, -1, -1, -1, -1, 143,
 -1, 115, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 43, -1,
),
array(
 -1, -1, -1, 116, -1, -1, -1, -1, 111, -1, -1, -1, -1, -1, -1, -1, 116, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 32, 32, -1, -1, 116, 116, -1, -1, -1, -1, -1, -1, 111,
 -1, -1, 31, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 32, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 59, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 60, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 153, -1, -1, -1, -1, -1, -1, 117, 117, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 43, 43, -1, -1, -1, 117, -1, -1, -1, -1, -1, -1, 153,
 -1, 117, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 43, -1,
),
array(
 -1, -1, -1, 118, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 118, 118, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 56, 56, -1, -1, 118, 118, -1, -1, -1, -1, -1, -1, -1,
 -1, 118, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 56, -1,
),
array(
 -1, -1, 57, 119, -1, -1, 57, -1, 57, -1, 57, -1, -1, -1, 57, 57, 119, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 57, 57, -1, 57, 119, 119, -1, -1, -1, -1, -1, -1, 57,
 -1, 57, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 57, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 165, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, -1, 74, 186,
 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, 84, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 84, -1,
),
array(
 -1, -1, 89, -1, -1, -1, 89, -1, 89, -1, 89, -1, -1, -1, 89, 89, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 89, 89, -1, 89, -1, -1, -1, -1, -1, -1, -1, -1, 89,
 -1, 89, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 89, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, 99, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 99, -1,
),
array(
 -1, -1, 104, -1, -1, -1, 104, -1, 104, -1, 104, -1, -1, -1, 104, 104, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 104, 104, -1, 104, -1, -1, -1, -1, -1, -1, -1, -1, 104,
 -1, 104, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 104, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 110, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 56, 56, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 141, -1, -1, -1, 141, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 56, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 36, -1, -1, -1, -1,
 -1, -1, -1, 37, -1, -1, 16, -1, -1, -1, -1, -1, -1, -1, -1, -1, 22, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 134, 258, 258, 258, 29, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, 126, -1, -1, -1, -1, 111, -1, -1, -1, -1, -1, -1, -1, 126, -1, 123, -1,
 -1, -1, -1, -1, -1, -1, -1, 32, 32, 123, -1, 126, 126, -1, -1, -1, -1, -1, -1, 111,
 -1, -1, 31, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 32, -1,
),
array(
 -1, -1, 77, -1, 78, -1, 79, 80, -1, -1, -1, 81, -1, -1, -1, 82, -1, -1, 188, -1,
 -1, -1, -1, -1, -1, 83, -1, -1, 84, 188, -1, -1, -1, -1, -1, -1, -1, -1, 85, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, 86, 87, -1, 88, 84, -1,
),
array(
 -1, -1, 92, -1, 93, -1, 94, 95, -1, -1, -1, 96, -1, -1, -1, 97, -1, -1, 190, -1,
 -1, -1, -1, -1, -1, 98, -1, -1, 99, 190, -1, -1, -1, -1, -1, -1, -1, -1, 100, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, 101, 102, -1, 103, 99, -1,
),
array(
 -1, 106, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 57, -1, -1, -1, 57, -1, 57, -1, 57, -1, -1, -1, 57, 57, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 57, 57, -1, 57, -1, -1, -1, -1, -1, -1, -1, -1, 57,
 -1, 57, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 57, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 38, -1, -1,
 -1, -1, -1, 39, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 30, 258, 258, 211, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 115, 115, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 127, 127, -1, -1, -1, 115, -1, -1, -1, -1, -1, -1, -1,
 -1, 115, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 127, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 117, 117, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 128, 128, -1, -1, -1, 117, -1, -1, -1, -1, -1, -1, -1,
 -1, 117, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 128, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 145, -1, -1, -1, -1, 147, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 40, -1, -1,
 41, -1, -1, 42, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, 33, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, 34, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 149, -1, -1, -1, 58, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 45, 46, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 55, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 151, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, 47, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 29, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, 61, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 17, -1, -1,
 -1, -1, -1, 48, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 62, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 56, 56, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 56, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 194, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 127, 127, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 155, -1, -1, -1, 155, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 127, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 194, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 157, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 55, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 159, 161, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 63,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 163, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 29, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 195, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 63, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 128, 128, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 167, -1, -1, -1, 167, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 128, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 64, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 127, 127, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 127, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 62, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 169, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 62, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 65, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 64, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 66, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 67, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 171, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 29, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 173, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 67, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, 128, 128, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 128, -1,
),
array(
 -1, -1, 258, 258, 68, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 70, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 67, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, 177, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 69, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 71, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 55, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 179, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 69, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 181, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 194, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 72, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 68, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, 183, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 62, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 183, 184, -1, -1, 196, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, -1, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185,
),
array(
 -1, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 73, 185, 185,
 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185, 185,
 185, 185, 185, 185, 185, 185, 185,
),
array(
 1, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, -1, 75, 75, 75,
 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75, 75,
 75, 75, 76, 75, 120, 75, -1,
),
array(
 1, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, -1, 90, 91, 90,
 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90,
 90, 90, 90, 90, 121, 90, -1,
),
array(
 1, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, -1, 105, 105, 105,
 105, 105, 105, 105, 105, 122, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
 105, 105, 105, 105, 105, 105, -1,
),
array(
 1, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 108, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
 107, 107, 107, 107, 107, 107, 109,
),
array(
 -1, -1, 258, 258, 258, 137, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 212, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 175, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1,
),
array(
 -1, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, -1, 186, 186,
 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186, 186,
 186, 186, 186, 186, 186, 186, 186,
),
array(
 -1, -1, 258, 258, 140, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 257, 216, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 249, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 217, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 218, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 259,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 142, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 144, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 219, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 220, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 146, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 221, 258, 258, 258, 258, 251, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 222,
 252, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 253, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 224, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 225, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 148, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 150, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 152, 258, 258, 254, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 154, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 156, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 229, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 231, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 232, 258, 258, 258, 258, 258, 258, 258, 158, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 233, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 235, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 238, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 239, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 134, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 160, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 241,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 242, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 162, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 255, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 164, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 243, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 166, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 168, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 152, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 170, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 172, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 174, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 244, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 245, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 176, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 247, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 178, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 226, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 180, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 182, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 199, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 228, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 234, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 236, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 237, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 240, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 246, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 200, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 227, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 230, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 201, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 202, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 203, 258, 258, 258, 258, 204, 258, 205, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 204, 258, 258, 206, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 207, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 208, 258, 258, 258, 258, 258, 258, 258, 258, 209, 258, 258, 258, 258, 258, 258,
 258, 210, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 213, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 214, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 250, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
array(
 -1, -1, 258, 258, 258, 215, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258, 258,
 258, 258, 258, 258, 258, 258, 258, 198, 198, 258, 258, 258, 258, -1, -1, -1, -1, -1, -1, 258,
 -1, 258, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, 198, -1,
),
);

	public function /*Yytoken*/ next_token ()
 {
		$yy_anchor = self::YY_NO_ANCHOR;
		$yy_state = self::$yy_state_dtrans[$this->yy_lexical_state];
		$yy_next_state = self::YY_NO_STATE;
		$yy_last_accept_state = self::YY_NO_STATE;
		$yy_initial = true;

		$this->yy_mark_start();
		$yy_this_accept = self::$yy_acpt[$yy_state];
		if (self::YY_NOT_ACCEPT != $yy_this_accept) {
			$yy_last_accept_state = $yy_state;
			$this->yy_mark_end();
		}
		while (true) {
			if ($yy_initial && $this->yy_at_bol) $yy_lookahead = self::YY_BOL;
			else $yy_lookahead = $this->yy_advance();
			$yy_next_state = self::$yy_nxt[self::$yy_rmap[$yy_state]][self::$yy_cmap[$yy_lookahead]];
			if ($this->YY_EOF == $yy_lookahead && true == $yy_initial) {

    if ($this->yy_lexical_state == self::SINGLELINE_COMMENT) {
        $this->yybegin(self::YYINITIAL);
        return $this->create_token('singleline_comment', $this->buffer());
    } else if ($this->yy_lexical_state == self::MULTILINE_COMMENT)  {
        $this->yybegin(self::YYINITIAL);
        $this->create_buffer_error($this->statestring);
        return $this->create_token('unknown', $this->buffer());
    } else if ($this->yy_lexical_state == self::STRING)  {
        $this->yybegin(self::YYINITIAL);
        $this->create_buffer_error($this->statestring);
        return $this->create_token('unknown', $this->statestring);
    } else if ($this->yy_lexical_state == self::CHARACTER)  {
        $this->yybegin(self::YYINITIAL);
        $this->create_buffer_error($this->statestring);
        return $this->create_token('unknown', $this->buffer());
    } else {
        return null;
    }
			}
			if (self::YY_F != $yy_next_state) {
				$yy_state = $yy_next_state;
				$yy_initial = false;
				$yy_this_accept = self::$yy_acpt[$yy_state];
				if (self::YY_NOT_ACCEPT != $yy_this_accept) {
					$yy_last_accept_state = $yy_state;
					$this->yy_mark_end();
				}
			}
			else {
				if (self::YY_NO_STATE == $yy_last_accept_state) {
					throw new Exception("Lexical Error: Unmatched Input.");
				}
				else {
					$yy_anchor = self::$yy_acpt[$yy_last_accept_state];
					if (0 != (self::YY_END & $yy_anchor)) {
						$this->yy_move_end();
					}
					$this->yy_to_mark();
					switch ($yy_last_accept_state) {
						case 1:
							
						case -2:
							break;
						case 2:
							{ return $this->create_token('operators',$this->yytext()); }
						case -3:
							break;
						case 3:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -4:
							break;
						case 4:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -5:
							break;
						case 5:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -6:
							break;
						case 6:
							{ if (!$this->is_white_space($this->yytext())) { $this->create_error($this->yytext()); return $this->create_token('unknown',$this->yytext()); } break; }
						case -7:
							break;
						case 7:
							{ $this->enterbufferedstate(self::STRING); break; }
						case -8:
							break;
						case 8:
							{ return $this->create_token('operators',$this->yytext()); }
						case -9:
							break;
						case 9:
							{ return $this->create_token('operators',$this->yytext()); }
						case -10:
							break;
						case 10:
							{ return $this->create_token('operators',$this->yytext()); }
						case -11:
							break;
						case 11:
							{ return $this->create_token('operators',$this->yytext()); }
						case -12:
							break;
						case 12:
							{ return $this->create_token('operators',$this->yytext()); }
						case -13:
							break;
						case 13:
							{ return $this->create_token('operators',$this->yytext()); }
						case -14:
							break;
						case 14:
							{ return $this->create_token('operators',$this->yytext()); }
						case -15:
							break;
						case 15:
							{ return $this->create_token('semicolon',$this->yytext()); }
						case -16:
							break;
						case 16:
							{ return $this->create_token('bracket','{'); }
						case -17:
							break;
						case 17:
							{ return $this->create_token('bracket','}'); }
						case -18:
							break;
						case 18:
							{ return $this->create_token('comma',$this->yytext()); }
						case -19:
							break;
						case 19:
							{ return $this->create_token('colon',$this->yytext()); }
						case -20:
							break;
						case 20:
							{ return $this->create_token('bracket',$this->yytext()); }
						case -21:
							break;
						case 21:
							{ return $this->create_token('bracket',$this->yytext()); }
						case -22:
							break;
						case 22:
							{ return $this->create_token('bracket','['); }
						case -23:
							break;
						case 23:
							{ return $this->create_token('bracket',']'); }
						case -24:
							break;
						case 24:
							{ return $this->create_token('question_mark',$this->yytext()); }
						case -25:
							break;
						case 25:
							{ $this->enterbufferedstate(self::CHARACTER); break; }
						case -26:
							break;
						case 26:
							{ $this->enterbufferedstate(self::SINGLELINE_COMMENT); break; }
						case -27:
							break;
						case 27:
							{ return $this->create_token('operators',$this->yytext()); }
						case -28:
							break;
						case 28:
							{ $this->enterbufferedstate(self::MULTILINE_COMMENT); break; }
						case -29:
							break;
						case 29:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -30:
							break;
						case 30:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -31:
							break;
						case 31:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -32:
							break;
						case 32:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -33:
							break;
						case 33:
							{ $this->enterbufferedstate(self::STRING); break; }
						case -34:
							break;
						case 34:
							{ $this->enterbufferedstate(self::CHARACTER); break; }
						case -35:
							break;
						case 35:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -36:
							break;
						case 36:
							{ return $this->create_token('operators',$this->yytext()); }
						case -37:
							break;
						case 37:
							{ return $this->create_token('operators',$this->yytext()); }
						case -38:
							break;
						case 38:
							{ return $this->create_token('operators',$this->yytext()); }
						case -39:
							break;
						case 39:
							{ return $this->create_token('operators',$this->yytext()); }
						case -40:
							break;
						case 40:
							{ return $this->create_token('operators',$this->yytext()); }
						case -41:
							break;
						case 41:
							{ return $this->create_token('operators',$this->yytext()); }
						case -42:
							break;
						case 42:
							{ return $this->create_token('operators',$this->yytext()); }
						case -43:
							break;
						case 43:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -44:
							break;
						case 44:
							{ return $this->create_token('operators',$this->yytext()); }
						case -45:
							break;
						case 45:
							{ return $this->create_token('operators',$this->yytext()); }
						case -46:
							break;
						case 46:
							{ return $this->create_token('operators',$this->yytext()); }
						case -47:
							break;
						case 47:
							{ return $this->create_token('operators',$this->yytext()); }
						case -48:
							break;
						case 48:
							{ return $this->create_token('operators',$this->yytext()); }
						case -49:
							break;
						case 49:
							{ return $this->create_token('operators',$this->yytext()); }
						case -50:
							break;
						case 50:
							{ return $this->create_token('operators',$this->yytext()); }
						case -51:
							break;
						case 51:
							{ return $this->create_token('operators',$this->yytext()); }
						case -52:
							break;
						case 52:
							{ return $this->create_token('operators',$this->yytext()); }
						case -53:
							break;
						case 53:
							{ return $this->create_token('operators',$this->yytext()); }
						case -54:
							break;
						case 54:
							{ return $this->create_token('operators',$this->yytext()); }
						case -55:
							break;
						case 55:
							{ return $this->create_token('typename',$this->yytext()); }
						case -56:
							break;
						case 56:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -57:
							break;
						case 57:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -58:
							break;
						case 58:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -59:
							break;
						case 59:
							{ return $this->create_token('operators',$this->yytext()); }
						case -60:
							break;
						case 60:
							{ return $this->create_token('operators',$this->yytext()); }
						case -61:
							break;
						case 61:
							{ return $this->create_token('ellipsis',$this->yytext()); }
						case -62:
							break;
						case 62:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -63:
							break;
						case 63:
							{ return $this->create_token('typename',$this->yytext()); }
						case -64:
							break;
						case 64:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -65:
							break;
						case 65:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -66:
							break;
						case 66:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -67:
							break;
						case 67:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -68:
							break;
						case 68:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -69:
							break;
						case 69:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -70:
							break;
						case 70:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -71:
							break;
						case 71:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -72:
							break;
						case 72:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -73:
							break;
						case 73:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -74:
							break;
						case 74:
							{ return $this->create_token('preprocessor',$this->yytext()); }
						case -75:
							break;
						case 75:
							{ $this->append($this->yytext()); break;}
						case -76:
							break;
						case 76:
							{ return $this->check_and_create_character();  }
						case -77:
							break;
						case 77:
							{ $this->append("\a"); break;}
						case -78:
							break;
						case 78:
							{ $this->append("\t"); break;}
						case -79:
							break;
						case 79:
							{ $this->append("\b"); break;}
						case -80:
							break;
						case 80:
							{ $this->append("\r"); break;}
						case -81:
							break;
						case 81:
							{ $this->append("\n"); break;}
						case -82:
							break;
						case 82:
							{ $this->append("\f"); break;}
						case -83:
							break;
						case 83:
							{ $this->append("\v"); break; }
						case -84:
							break;
						case 84:
							{ $this->append(block_formal_langs_octal_to_decimal_char(substr($this->yytext(),1))); break;}
						case -85:
							break;
						case 85:
							{ $this->append('"'); break;}
						case -86:
							break;
						case 86:
							{ $this->append('?'); break;}
						case -87:
							break;
						case 87:
							{ $this->append('\''); break;}
						case -88:
							break;
						case 88:
							{ $this->append('\\'); break;}
						case -89:
							break;
						case 89:
							{ $this->append(block_formal_langs_hex_to_decimal_char(substr($this->yytext(),2))); break;}
						case -90:
							break;
						case 90:
							{ $this->append($this->yytext()); break;}
						case -91:
							break;
						case 91:
							{ return $this->leavebufferedstate('string');  }
						case -92:
							break;
						case 92:
							{ $this->append("\a"); break;}
						case -93:
							break;
						case 93:
							{ $this->append("\t"); break;}
						case -94:
							break;
						case 94:
							{ $this->append("\b"); break;}
						case -95:
							break;
						case 95:
							{ $this->append("\r"); break;}
						case -96:
							break;
						case 96:
							{ $this->append("\n"); break;}
						case -97:
							break;
						case 97:
							{ $this->append("\f"); break;}
						case -98:
							break;
						case 98:
							{ $this->append("\v"); break; }
						case -99:
							break;
						case 99:
							{ $this->append(block_formal_langs_octal_to_decimal_char(substr($this->yytext(),1))); break;}
						case -100:
							break;
						case 100:
							{ $this->append('"'); break;}
						case -101:
							break;
						case 101:
							{ $this->append('?'); break;}
						case -102:
							break;
						case 102:
							{ $this->append('\''); break;}
						case -103:
							break;
						case 103:
							{ $this->append('\\'); break;}
						case -104:
							break;
						case 104:
							{ $this->append(block_formal_langs_hex_to_decimal_char(substr($this->yytext(),2))); break;}
						case -105:
							break;
						case 105:
							{ $this->append($this->yytext()); break;  }
						case -106:
							break;
						case 106:
							{ return $this->leavebufferedstate('multiline_comment');  }
						case -107:
							break;
						case 107:
							{ $this->append($this->yytext()); break;}
						case -108:
							break;
						case 108:
							{ return $this->leavebufferedstate('singleline_comment'); break; }
						case -109:
							break;
						case 109:
							{ return $this->leavebufferedstate('singleline_comment'); break; }
						case -110:
							break;
						case 110:
							{ return $this->leavebufferedstate('singleline_comment'); break; }
						case -111:
							break;
						case 112:
							{ return $this->create_token('operators',$this->yytext()); }
						case -112:
							break;
						case 113:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -113:
							break;
						case 114:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -114:
							break;
						case 115:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -115:
							break;
						case 116:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -116:
							break;
						case 117:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -117:
							break;
						case 118:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -118:
							break;
						case 119:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -119:
							break;
						case 120:
							{ $this->append($this->yytext()); break;}
						case -120:
							break;
						case 121:
							{ $this->append($this->yytext()); break;}
						case -121:
							break;
						case 122:
							{ $this->append($this->yytext()); break;  }
						case -122:
							break;
						case 124:
							{ return $this->create_token('operators',$this->yytext()); }
						case -123:
							break;
						case 125:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -124:
							break;
						case 126:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -125:
							break;
						case 127:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -126:
							break;
						case 128:
							{ return $this->create_token('numeric',$this->yytext()); }
						case -127:
							break;
						case 130:
							{ return $this->create_token('operators',$this->yytext()); }
						case -128:
							break;
						case 131:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -129:
							break;
						case 133:
							{ return $this->create_token('operators',$this->yytext()); }
						case -130:
							break;
						case 134:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -131:
							break;
						case 136:
							{ return $this->create_token('operators',$this->yytext()); }
						case -132:
							break;
						case 137:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -133:
							break;
						case 139:
							{ return $this->create_token('operators',$this->yytext()); }
						case -134:
							break;
						case 140:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -135:
							break;
						case 142:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -136:
							break;
						case 144:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -137:
							break;
						case 146:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -138:
							break;
						case 148:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -139:
							break;
						case 150:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -140:
							break;
						case 152:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -141:
							break;
						case 154:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -142:
							break;
						case 156:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -143:
							break;
						case 158:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -144:
							break;
						case 160:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -145:
							break;
						case 162:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -146:
							break;
						case 164:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -147:
							break;
						case 166:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -148:
							break;
						case 168:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -149:
							break;
						case 170:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -150:
							break;
						case 172:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -151:
							break;
						case 174:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -152:
							break;
						case 176:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -153:
							break;
						case 178:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -154:
							break;
						case 180:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -155:
							break;
						case 182:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -156:
							break;
						case 193:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -157:
							break;
						case 194:
							{ return $this->create_token('keyword',$this->yytext()); }
						case -158:
							break;
						case 197:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -159:
							break;
						case 198:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -160:
							break;
						case 199:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -161:
							break;
						case 200:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -162:
							break;
						case 201:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -163:
							break;
						case 202:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -164:
							break;
						case 203:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -165:
							break;
						case 204:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -166:
							break;
						case 205:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -167:
							break;
						case 206:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -168:
							break;
						case 207:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -169:
							break;
						case 208:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -170:
							break;
						case 209:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -171:
							break;
						case 210:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -172:
							break;
						case 211:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -173:
							break;
						case 212:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -174:
							break;
						case 213:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -175:
							break;
						case 214:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -176:
							break;
						case 215:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -177:
							break;
						case 216:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -178:
							break;
						case 217:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -179:
							break;
						case 218:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -180:
							break;
						case 219:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -181:
							break;
						case 220:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -182:
							break;
						case 221:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -183:
							break;
						case 222:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -184:
							break;
						case 223:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -185:
							break;
						case 224:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -186:
							break;
						case 225:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -187:
							break;
						case 226:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -188:
							break;
						case 227:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -189:
							break;
						case 228:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -190:
							break;
						case 229:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -191:
							break;
						case 230:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -192:
							break;
						case 231:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -193:
							break;
						case 232:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -194:
							break;
						case 233:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -195:
							break;
						case 234:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -196:
							break;
						case 235:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -197:
							break;
						case 236:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -198:
							break;
						case 237:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -199:
							break;
						case 238:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -200:
							break;
						case 239:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -201:
							break;
						case 240:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -202:
							break;
						case 241:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -203:
							break;
						case 242:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -204:
							break;
						case 243:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -205:
							break;
						case 244:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -206:
							break;
						case 245:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -207:
							break;
						case 246:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -208:
							break;
						case 247:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -209:
							break;
						case 248:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -210:
							break;
						case 249:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -211:
							break;
						case 250:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -212:
							break;
						case 251:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -213:
							break;
						case 252:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -214:
							break;
						case 253:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -215:
							break;
						case 254:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -216:
							break;
						case 255:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -217:
							break;
						case 256:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -218:
							break;
						case 257:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -219:
							break;
						case 258:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -220:
							break;
						case 259:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -221:
							break;
						case 260:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -222:
							break;
						case 261:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -223:
							break;
						case 262:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -224:
							break;
						case 263:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -225:
							break;
						case 264:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -226:
							break;
						case 265:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -227:
							break;
						case 266:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -228:
							break;
						case 267:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -229:
							break;
						case 268:
							{ return $this->create_token('identifier',$this->yytext()); }
						case -230:
							break;
						default:
						$this->yy_error('INTERNAL',false);
					case -1:
					}
					$yy_initial = true;
					$yy_state = self::$yy_state_dtrans[$this->yy_lexical_state];
					$yy_next_state = self::YY_NO_STATE;
					$yy_last_accept_state = self::YY_NO_STATE;
					$this->yy_mark_start();
					$yy_this_accept = self::$yy_acpt[$yy_state];
					if (self::YY_NOT_ACCEPT != $yy_this_accept) {
						$yy_last_accept_state = $yy_state;
						$this->yy_mark_end();
					}
				}
			}
		}
	}
}
