﻿<?php
/**
 * Unit tests for matchers
 *
 * @copyright &copy; 2012 Sychev Oleg, Volgograd State Technical University
 * @author Vitaliy Bondarenko, Volgograd State Technical University
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package questions
 */

defined('MOODLE_INTERNAL') || die();

global $CFG;
require_once($CFG->dirroot . '/question/type/preg/preg_matcher.php');

class qtype_preg_cross_tests_from_pcre {

    function data_for_test_simple_regexp() {
        $test1 = array( 'str'=>'the quick brown fox',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'The quick brown FOX',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(19),
                        'next'=>'t',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'What do you know about the quick brown fox?',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>23),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'What do you know about THE QUICK BROWN FOX?',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'left'=>array(18),
                        'next'=>'h',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'the quick brown fox',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_modifier_i() {
        $test1 = array( 'str'=>'the quick brown fox',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'The quick brown FOX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'What do you know about the quick brown fox?',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>23),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'What do you know about THE QUICK BROWN FOX?',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>23),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'The quick brown fox',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_escape_sequences() { //escape-последовательность \a не поддерживается
        $test1 = array( 'str'=>"abcd\t\n\r\f\a\e9;\$\\?caxyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>20),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abcd\t\n\r\f\a\e\071\x3b\$\\\?caxyz',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_quantifiers() {
        $test1 = array( 'str'=>'abxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>23),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aabxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>24),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaabxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaabxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abcxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>24),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aabcxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqqqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>28),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqqqqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>29),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqqqqqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>30),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqqqqqqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>31),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test14 = array( 'str'=>'aaaabcxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test15 = array( 'str'=>'abxyzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>24),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test16 = array( 'str'=>'aabxyzzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test17 = array( 'str'=>'aaabxyzzzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>28),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test18 = array( 'str'=>'aaaabxyzzzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>29),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test19 = array( 'str'=>'abcxyzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test20 = array( 'str'=>'aabcxyzzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test21 = array( 'str'=>'aaabcxyzzzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>29),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test22 = array( 'str'=>'aaaabcxyzzzzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>30),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test23 = array( 'str'=>'aaaabcxyzzzzpqrrrabbbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>31),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test24 = array( 'str'=>'aaaabcxyzzzzpqrrrabbbxyyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>32),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test25 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypABzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test26 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypABBzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test27 = array( 'str'=>'>>>aaabxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test28 = array( 'str'=>'>aaaabxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test29 = array( 'str'=>'>>>>abcxyzpqrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>24),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test30 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(21),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test31 = array( 'str'=>'abxyzpqrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>9),
                        'left'=>array(13),
                        'next'=>'r',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test32 = array( 'str'=>'abxyzpqrrrrabbxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'left'=>array(12),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test33 = array( 'str'=>'abxyzpqrrrabxyyyypqAzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'left'=>array(10),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test34 = array( 'str'=>'aaaabcxyzzzzpqrrrabbbxyyyyyypqAzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'left'=>array(4),
                        'next'=>'p',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test35 = array( 'str'=>'aaaabcxyzzzzpqrrrabbbxyyypqAzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'left'=>array(5),
                        'next'=>'y',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test36 = array( 'str'=>'aaabcxyzpqrrrabbxyyyypqqqqqqqAzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>28),
                        'left'=>array(3),
                        'next'=>'A',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*abc?xyz+pqr{3}ab{2,}xy{4,5}pq{0,6}AB{0,}zz',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13, $test14, $test15, $test16, $test17, $test18, $test19, $test20, $test21, $test22, $test23, $test24, $test25, $test26, $test27, $test28, $test29, $test30, $test31, $test32, $test33, $test34, $test35, $test36));
    }

    function data_for_test_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'abczz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abcabczz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>8,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'zz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abcabcabczz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>6,1=>3),
                        'left'=>array(2),
                        'next'=>'z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'>>abczz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(abc){1,2}zz',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

/*  function data_for_test_subpattern_and_alternative_and_lazy_quantifiers() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'abbbbbbbbbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>13,1=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'bbbbbbbbbbbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>11),
                        'length'=>array(0=>13,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'aaac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'abbbbbbbbbbbac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(b+?|a){1,2}?c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11));
    }
*/
    function data_for_test_subpattern_and_alternative_and_greedy_quantifiers() {
        $test1 = array( 'str'=>'bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'abbbbbbbbbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>13,1=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'bbbbbbbbbbbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>11),
                        'length'=>array(0=>13,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'aaac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'abbbbbbbbbbbac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>12,1=>11),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(b+|a){1,2}c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11));
    }

/*  function data_for_test_subpattern_and_alternative_and_greedy_and_lazy_quantifiers() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'bbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(b+|a){1,2}?bc',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_and_alternative_and_greedy_and_lazy_quantifiers_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'babc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>5,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bababc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>6,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bababbc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'babababc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(b*|ba){1,2}?bc',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_subpattern_and_alternative_and_greedy_and_lazy_quantifiers_3() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'babc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>5,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bababc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>6,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bababbc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'babababc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(ba|b*){1,2}?bc',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_escape_sequences_control() { //control escape-последовательности не поддерживаются
        $test1 = array( 'str'=>"\x01\x01\e;z",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\ca\cA\c[\c{\c:',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_square_closing_bracket_in_a_character_class() {
        $test1 = array( 'str'=>'athing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>']thing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'cthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'dthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'ething',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'fthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'[thing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'\\thing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[ab\]cde]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10));
    }

    function data_for_test_square_closing_bracket_without_backslash_at_beginning_of_the_character_class() {
        $test1 = array( 'str'=>']thing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'cthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'dthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ething',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'athing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'fthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[]cde]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_square_closing_bracket_in_negative_character_class() {
        $test1 = array( 'str'=>'fthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'[thing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'\\thing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'athing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'bthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>']thing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'cthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'dthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'ething',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^ab\]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[^ab\]cde]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10));
    }

    function data_for_test_square_closing_bracket_without_backslash_at_beginning_of_the_negative_character_class() {
        $test1 = array( 'str'=>'athing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'fthing',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>']thing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'cthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'dthing',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'ething',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[^]cde]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[^]cde]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_unknown_character() {
        $test1 = array( 'str'=>'Ѓ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\Ѓ',
                     'tests'=>array($test1));
    }

    function data_for_test_russian_character() {
        $test1 = array( 'str'=>'я',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^я',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier() {
        $test1 = array( 'str'=>'0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'2',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'3',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'4',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'5',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'6',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'7',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'8',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'9',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'10',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>'100',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test14 = array( 'str'=>'abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[0-9]+$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13, $test14));
    }

    function data_for_test_any_character_in_quantifier() {
        $test1 = array( 'str'=>'enter',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'inter',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'uponter',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*nter',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_in_quantifier_2() {
        $test1 = array( 'str'=>'xxx0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xxx1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xxx',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^xxx[0-9]+$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_any_character_in_quantifier_with_character_classes() {
        $test1 = array( 'str'=>'x123',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xx123',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'123456',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'left'=>array(3),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'123',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'x1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.+[0-9][0-9][0-9]$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

/*  function data_for_test_any_character_in_lazy_quantifier_with_character_classes() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'x123',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xx123',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'123456',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'123',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'[0-9]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'x1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.+?[0-9][0-9][0-9]$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
    function data_for_test_character_class_and_quantifiers_and_subpatterns() {
        $test1 = array( 'str'=>'abc!pqr=apquxz.ixr.zzz.ac.uk',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>28,1=>3,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(23),
                        'next'=>'!',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'!pqr=apquxz.ixr.zzz.ac.uk',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(24),
                        'next'=>'[^!]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abc!=apquxz.ixr.zzz.ac.uk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>25,1=>3,2=>21),
                        'left'=>array(21),
                        'next'=>'=',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abc!pqr=apquxz:ixr.zzz.ac.uk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>14,1=>3,2=>3),
                        'left'=>array(14),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'abc!pqr=apquxz.ixr.zzz.ac.ukk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>28,1=>3,2=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^([^!]+)!(.+)=apquxz\.ixr\.zzz\.ac\.uk$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_character() {
        $test1 = array( 'str'=>'Well, we need a colon: somewhere',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>21),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Fail if we don\'t',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>':',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>':',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_with_modifier_i() {
        $test1 = array( 'str'=>'0abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'fed',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'E',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'::',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'5f03:12C0::932e',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>15,1=>15),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'fed def',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'Any old stuff',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>11,1=>11),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'0zzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'gzzz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\da-f:]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>"fed\x20",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'Any old rubbish',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>10,1=>10),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([\da-f:]+)$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13));
    }

    function data_for_test_quantifiers_and_subpatterns() {
        $test1 = array( 'str'=>'.1.2.3',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>3,3=>5),
                        'length'=>array(0=>6,1=>1,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'A.12.123.0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2,2=>5,3=>9),
                        'length'=>array(0=>10,1=>2,2=>3,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'left'=>array(6),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'.1.2.3333',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1,2=>3,3=>5),
                        'length'=>array(0=>8,1=>1,2=>1,3=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'1.2.3',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'left'=>array(2),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'1234.2.3',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>5,2=>7),
                        'length'=>array(0=>8,1=>1,2=>1),
                        'left'=>array(2),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_space_and_not_space_characters_in_quantifiers_and_subpatterns() {
        $test1 = array( 'str'=>'1 IN SOA non-sp1 non-sp2(',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>9,3=>17),
                        'length'=>array(0=>25,1=>1,2=>7,3=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1    IN    SOA    non-sp1    non-sp2   (',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>18,3=>29),
                        'length'=>array(0=>40,1=>1,2=>7,3=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(13),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'1IN SOA non-sp1 non-sp2(',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(12),
                        'next'=>'\s',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\d+)\s+IN\s+SOA\s+(\S+)\s+(\S+)\s*\(\s*$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_character_classes_in_quantifiers_and_subpattern() {
        $test1 = array( 'str'=>'a.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Z.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'2.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ab-c.pq-r.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>10,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'sxk.zzz.ac.uk.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>10),
                        'length'=>array(0=>14,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'x-.y-.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'[a-zA-Z\d]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'-abc.peq.',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'[a-zA-Z\d]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a-zA-Z\d][a-zA-Z\d\-]*(\.[a-zA-Z\d][a-zA-z\d\-]*)*\.$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

    function data_for_test_character_classes_in_quantifiers_and_subpatterns() {
        $test1 = array( 'str'=>'*.a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*.b0-a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*.c3-b.c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>6),
                        'length'=>array(0=>8,1=>3,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*.c-a.b-c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>5,3=>7),
                        'length'=>array(0=>9,1=>2,2=>4,3=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'*.0',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[a-z]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'*.a-',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'*.a-b.c-',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3,2=>5),
                        'length'=>array(0=>7,1=>2,2=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'*.c-a.0-c',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\*\.[a-z]([a-z\-\d]*[a-z\d]+)?(\.[a-z]([a-z\-\d]*[a-z\d]+)?)*$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }

    function data_for_test_positive_lookahead_assertion_and_subpatterns() {
        $test1 = array( 'str'=>'abde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2,2=>0,3=>3),
                        'length'=>array(0=>4,1=>2,2=>3,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=ab(de))(abd)(e)',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_lookahead_assertion_and_subpatterns() {
        $test1 = array( 'str'=>'abdf',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0,3=>3),
                        'length'=>array(0=>4,2=>3,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?!(ab)de|x)(abd)(f)',
                     'tests'=>array($test1));
    }

    function data_for_test_positive_lookahead_assertion_and_subpatterns_2() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>2,3=>0),
                        'length'=>array(0=>2,1=>4,2=>2,3=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=(ab(cd)))(ab)',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'a.b.c.d',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'A.B.C.D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a.b.c.1.2.3.C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>11),
                        'length'=>array(0=>13,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\da-f](\.[\da-f])*$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_any_character_and_space_character_in_quantifiers_and_subpattern() {
        $test1 = array( 'str'=>'"1234"',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'"abcd" ;',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'"" ; rhubarb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>12,1=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'"',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'"1234" : things',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\".*\"\s*(;.*)?$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_empty_string() {
        $test1 = array( 'str'=>'',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^$',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'ab c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'\s',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ab cde',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'   ^    a     b\sc  $ ',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_modifier_x_in_regexp() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'ab c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'\s',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ab cde',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?x)   ^    a     b\sc  $ ',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_modifier_x_with_escaped_space_and_space_in_character_class() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'a bcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a b d',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(4),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ab d',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(4),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^   a\ b[c ]d       $',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
    function data_for_test_many_subpatterns() {
        $test1 = array( 'str'=>'abcdefhijklm',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5,7=>6,8=>7,9=>8,10=>9,11=>10,12=>11),
                        'length'=>array(0=>12,1=>3,2=>2,3=>1,4=>3,5=>2,6=>1,7=>3,8=>2,9=>1,10=>3,11=>2,12=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a(b(c)))(d(e(f)))(h(i(j)))(k(l(m)))$',
                     'tests'=>array($test1));
    }

    function data_for_test_many_subpatterns_and_unnumbered_subpatterns() {
        $test1 = array( 'str'=>'abcdefhijklm',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>2,3=>4,4=>5,5=>7,6=>8,7=>10,8=>11),
                        'length'=>array(0=>12,1=>2,2=>1,3=>2,4=>1,5=>2,6=>1,7=>2,8=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a(b(c)))(?:d(e(f)))(?:h(i(j)))(?:k(l(m)))$',
                     'tests'=>array($test1));
    }

/*  function data_for_test_metacharacters_in_character_classes() { //символ \b не поддерживается
        $test1 = array( 'str'=>"a+ Z0+\x08\n\x1d\x12",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\w][\W][\s][\S][\d][\D][\b][\n][\c]][\022]',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_metacharacters_in_character_class_in_quantifier() {
        $test1 = array( 'str'=>".^\$(*+)|{?,?}",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>13),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[.^$|()*+?{,}]+',
                     'tests'=>array($test1));
    }

    function data_for_test_metacharacter_and_greedy_quantifier() {
        $test1 = array( 'str'=>'z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'az',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'a+',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aa+',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a*\w',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

/*  function data_for_test_metacharacter_and_lazy_quantifier() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'az',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'a+',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aa+',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a*?\w',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }
*/
    function data_for_test_metacharacter_and_greedy_quantifier_2() {
        $test1 = array( 'str'=>'az',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aa+',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a+\w',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_metacharacter_and_lazy_quantifier_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'az',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aa+',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a+?\w',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
    function data_for_test_digits_and_word_characters_in_quantifiers() {
        $test1 = array( 'str'=>'1234567890',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12345678ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12345678__',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(10),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'1234567',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(3),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\d{8}\w{2,}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_character_class_in_greedy_quantifier() {
        $test1 = array( 'str'=>'uoie',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12345',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'[aeiou\d]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'123456',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[aeiou\d]{4,5}$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

/*  function data_for_test_character_class_in_lazy_quantifier() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'uoie',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12345',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'123456',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[aeiou\d]{4,5}?',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
    function data_for_test_back_reference() {
        $test1 = array( 'str'=>'abc=abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7),
                        'length'=>array(0=>10,1=>3,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'def=defdefdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>10),
                        'length'=>array(0=>13,1=>3,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(10),
                        'next'=>'[ad]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abc=defdef',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'left'=>array(6),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A(abc|def)=(\1){2,3}\Z',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_many_back_references() {
        $test1 = array( 'str'=>'abcdefghijkcda2',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5,7=>6,8=>7,9=>8,10=>9,11=>10,12=>11),
                        'length'=>array(0=>15,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1,11=>1,12=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abcdefghijkkkkcda2',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5,7=>6,8=>7,9=>8,10=>9,11=>10,12=>14),
                        'length'=>array(0=>18,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1,11=>1,12=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)(b)(c)(d)(e)(f)(g)(h)(i)(j)(k)\11*(\3\4)\1(?#)2$',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_alternatives_and_subpatterns_and_back_reference() {
        $test1 = array( 'str'=>'cataract cataract23',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3,3=>4,4=>17,5=>18),
                        'length'=>array(0=>19,1=>8,2=>5,3=>4,4=>0,5=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'catatonic catatonic23',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3,3=>4,4=>19,5=>20),
                        'length'=>array(0=>21,1=>9,2=>6,3=>5,4=>0,5=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'caterpillar caterpillar23',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3,4=>23,5=>24),
                        'length'=>array(0=>25,1=>11,2=>8,4=>0,5=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(cat(a(ract|tonic)|erpillar)) \1()2(3)',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_many_character_classes_and_spaces_in_quantifiers() {
        $test1 = array( 'str'=>'From abcd  Mon Sep 01 12:33:02 1997',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>27,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^From +([^ ]+) +[a-zA-Z][a-zA-Z][a-zA-Z] +[a-zA-Z][a-zA-Z][a-zA-Z] +[0-9]?[0-9] +[0-9][0-9]:[0-9][0-9]',
                     'tests'=>array($test1));
    }

    function data_for_test_many_metacharacters_in_quantifiers() {
        $test1 = array( 'str'=>'From abcd  Mon Sep 01 12:33:02 1997',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>15),
                        'length'=>array(0=>27,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'From abcd  Mon Sep  1 12:33:02 1997',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>15),
                        'length'=>array(0=>27,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'F',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'From abcd  Sep 01 12:33:02 1997',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>11),
                        'length'=>array(0=>15,1=>4),
                        'left'=>array(11),
                        'next'=>'[a-zA-Z]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^From\s+\S+\s+([a-zA-Z]{3}\s+){2}\d{1,2}\s+\d\d:\d\d',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"12\n34",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"12\r34",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^12.34',
                     'modifiers'=>'s',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_positive_lookahead_assertion_and_metacharacters() {
        $test1 = array( 'str'=>"the quick brown\t fox",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>10),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\w+(?=\t)',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_lookahead_assertion_and_quantifier() {
        $test1 = array( 'str'=>'foobar is foolish see?',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>10,1=>13),
                        'length'=>array(0=>12,1=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'foo(?!bar)(.*)',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_lookahead_assertion_and_unnumbered_subpattern_and_quantifiers() {
        $test1 = array( 'str'=>'foobar crowbar etc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>8,1=>14),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'barrel',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'2barrel',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>7,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'A barrel',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>8,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?!foo)...|^.{0,2})bar(.*)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_positive_and_negative_lookahead_assertions_and_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'abc456',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc123',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\D*)(?=\d)(?!123)',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_newline_in_comment() {
        $test1 = array( 'str'=>'1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^1234(?# test newlines\ninside)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_oneline_comment_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'1234',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^1234 #comment in extended re\n',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_oneline_comment_modifier_x_2() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'#rhubarb\nabcd',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_oneline_comment_modifier_x_3() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abcd#rhubarb',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_back_reference_in_quantifier() {
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3),
                        'length'=>array(0=>4,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)\1{2,3}(.)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_begin_of_line_in_negative_lookahead_assertion() {
        $test1 = array( 'str'=>'the abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?!^)abc',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_begin_of_line_in_positive_lookahead_assertion() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'the abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=^)abc',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_in_greedy_quantifier_and_greedy_quantifier_in_alternative_in_subpattern() {
        $test1 = array( 'str'=>'aabbbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>7,1=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[ab]{1,3}(ab*|b)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_character_class_in_lazy_quantifier_and_greedy_quantifier_in_alternative_in_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aabbbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>7,1=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[ab]{1,3}?(ab*|b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_character_class_in_lazy_quantifier_and_lazy_quantifier_in_alternative_in_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aabbbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[ab]{1,3}?(ab*?|b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_character_class_in_greedy_quantifier_and_lazy_quantifier_in_alternative_in_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aabbbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[ab]{1,3}(ab*?|b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_O_O_WTF() { //наркоманство какое-то и модификатор x не поддерживается
        $test1 = array( 'str'=>'Alan Other <user@dom.ain>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'<user@dom.ain>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'user@dom.ain',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'"A. Other" <user.1234@dom.ain> (a comment)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>42),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'A. Other <user.1234@dom.ain> (a comment)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>38),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'"/s=user/ou=host/o=place/prmd=uu.yy/admd= /c=gb/"@x400-re.lay',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>61),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'A missing angle <user@some.where',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>17),
                        'length'=>array(0=>15),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>17),
                        'length'=>array(0=>15),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'The quick brown fox',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>17),
                        'length'=>array(0=>15),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*                          # optional leading comment
(?:    (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
" (?:                      # opening quote...
[^\\\x80-\xff\n\015"]                #   Anything except backslash and quote
|                     #    or
\\ [^\x80-\xff]           #   Escaped something (something != CR)
)* "  # closing quote
)                    # initial word
(?:  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  \.  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*   (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
" (?:                      # opening quote...
[^\\\x80-\xff\n\015"]                #   Anything except backslash and quote
|                     #    or
\\ [^\x80-\xff]           #   Escaped something (something != CR)
)* "  # closing quote
)  )* # further okay, if led by a period
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  @  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*    (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                           # initial subdomain
(?:                                  #
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  \.                        # if led by a period...
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*   (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                     #   ...further okay
)*
# address
|                     #  or
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
" (?:                      # opening quote...
[^\\\x80-\xff\n\015"]                #   Anything except backslash and quote
|                     #    or
\\ [^\x80-\xff]           #   Escaped something (something != CR)
)* "  # closing quote
)             # one word, optionally followed by....
(?:
[^()<>@,;:".\\\[\]\x80-\xff\000-\010\012-\037]  |  # atom and space parts, or...
\(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)       |  # comments, or...

" (?:                      # opening quote...
[^\\\x80-\xff\n\015"]                #   Anything except backslash and quote
|                     #    or
\\ [^\x80-\xff]           #   Escaped something (something != CR)
)* "  # closing quote
# quoted strings
)*
<  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*                     # leading <
(?:  @  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*    (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                           # initial subdomain
(?:                                  #
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  \.                        # if led by a period...
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*   (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                     #   ...further okay
)*

(?:  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  ,  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  @  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*    (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                           # initial subdomain
(?:                                  #
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  \.                        # if led by a period...
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*   (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                     #   ...further okay
)*
)* # further okay, if led by comma
:                                # closing colon
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  )? #       optional route
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
" (?:                      # opening quote...
[^\\\x80-\xff\n\015"]                #   Anything except backslash and quote
|                     #    or
\\ [^\x80-\xff]           #   Escaped something (something != CR)
)* "  # closing quote
)                    # initial word
(?:  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  \.  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*   (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
" (?:                      # opening quote...
[^\\\x80-\xff\n\015"]                #   Anything except backslash and quote
|                     #    or
\\ [^\x80-\xff]           #   Escaped something (something != CR)
)* "  # closing quote
)  )* # further okay, if led by a period
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  @  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*    (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                           # initial subdomain
(?:                                  #
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  \.                        # if led by a period...
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*   (?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|   \[                         # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*    #    stuff
\]                        #           ]
)                     #   ...further okay
)*
#       address spec
(?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*  > #                  trailing >
# name and address
)  (?: [\040\t] |  \(
(?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  |  \( (?:  [^\\\x80-\xff\n\015()]  |  \\ [^\x80-\xff]  )* \)  )*
\)  )*                       # optional trailing comment
',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }
*/
/*  function data_for_test_O_O_WTF_2() { //наркоманство какое-то v2.0 и модификатор x не поддерживается
        $test1 = array( 'str'=>'Alan Other <user@dom.ain>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>25),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'<user@dom.ain>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'user@dom.ain',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'"A. Other" <user.1234@dom.ain> (a comment)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>30),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'A. Other <user.1234@dom.ain> (a comment)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'"/s=user/ou=host/o=place/prmd=uu.yy/admd= /c=gb/"@x400-re.lay',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>61),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'A missing angle <user@some.where',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>17),
                        'length'=>array(0=>15),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>17),
                        'length'=>array(0=>15),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'The quick brown fox',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>17),
                        'length'=>array(0=>15),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional leading comment
(?:
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
# Atom
|                       #  or
"                                     # "
[^\\\x80-\xff\n\015"] *                            #   normal
(?:  \\ [^\x80-\xff]  [^\\\x80-\xff\n\015"] * )*        #   ( special normal* )*
"                                     #        "
# Quoted string
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
\.
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
# Atom
|                       #  or
"                                     # "
[^\\\x80-\xff\n\015"] *                            #   normal
(?:  \\ [^\x80-\xff]  [^\\\x80-\xff\n\015"] * )*        #   ( special normal* )*
"                                     #        "
# Quoted string
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# additional words
)*
@
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
(?:
\.
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
)*
# address
|                             #  or
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
# Atom
|                       #  or
"                                     # "
[^\\\x80-\xff\n\015"] *                            #   normal
(?:  \\ [^\x80-\xff]  [^\\\x80-\xff\n\015"] * )*        #   ( special normal* )*
"                                     #        "
# Quoted string
)
# leading word
[^()<>@,;:".\\\[\]\x80-\xff\000-\010\012-\037] *               # "normal" atoms and or spaces
(?:
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
|
"                                     # "
[^\\\x80-\xff\n\015"] *                            #   normal
(?:  \\ [^\x80-\xff]  [^\\\x80-\xff\n\015"] * )*        #   ( special normal* )*
"                                     #        "
) # "special" comment or quoted string
[^()<>@,;:".\\\[\]\x80-\xff\000-\010\012-\037] *            #  more "normal"
)*
<
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# <
(?:
@
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
(?:
\.
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
)*
(?: ,
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
@
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
(?:
\.
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
)*
)*  # additional domains
:
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
)?     #       optional route
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
# Atom
|                       #  or
"                                     # "
[^\\\x80-\xff\n\015"] *                            #   normal
(?:  \\ [^\x80-\xff]  [^\\\x80-\xff\n\015"] * )*        #   ( special normal* )*
"                                     #        "
# Quoted string
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
\.
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
# Atom
|                       #  or
"                                     # "
[^\\\x80-\xff\n\015"] *                            #   normal
(?:  \\ [^\x80-\xff]  [^\\\x80-\xff\n\015"] * )*        #   ( special normal* )*
"                                     #        "
# Quoted string
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# additional words
)*
@
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
(?:
\.
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
(?:
[^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]+    # some number of atom characters...
(?![^(\040)<>@,;:".\\\[\]\000-\037\x80-\xff]) # ..not followed by something that could be part of an atom
|
\[                            # [
(?: [^\\\x80-\xff\n\015\[\]] |  \\ [^\x80-\xff]  )*     #    stuff
\]                           #           ]
)
[\040\t]*                    # Nab whitespace.
(?:
\(                              #  (
[^\\\x80-\xff\n\015()] *                             #     normal*
(?:                                 #       (
(?:  \\ [^\x80-\xff]  |
\(                            #  (
[^\\\x80-\xff\n\015()] *                            #     normal*
(?:  \\ [^\x80-\xff]   [^\\\x80-\xff\n\015()] * )*        #     (special normal*)*
\)                           #                       )
)    #         special
[^\\\x80-\xff\n\015()] *                         #         normal*
)*                                  #            )*
\)                             #                )
[\040\t]* )*    # If comment found, allow more spaces.
# optional trailing comments
)*
#       address spec
>                    #                 >
# name and address
)
',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }
*/
    function data_for_test_oct_code_of_character_one_two_three_and_more_digits() {
        $test1 = array( 'str'=>"abc\0def\00pqr\000xyz\0000AB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc456 abc\0def\00pqr\000xyz\0000ABCDE",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\0def\00pqr\000xyz\0000AB',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_hex_code_of_character_two_and_more_digits() {
        $test1 = array( 'str'=>"abc\x0def\x00pqr\x000xyz\x0000AB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>20),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc456 abc\x0def\x00pqr\x000xyz\x0000ABCDE",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>20),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\x0def\x00pqr\x000xyz\x0000AB',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_oct_code_of_characters_in_character_class() {
        $test1 = array( 'str'=>"\0A",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"\01B",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"\037C",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\000-\037]',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_oct_code_of_character_in_quantifier() {
        $test1 = array( 'str'=>"\0\0\0\0",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\0*',
                     'tests'=>array($test1));
    }

    function data_for_test_hex_code_of_character_one_digit_in_quantifier() {
        $test1 = array( 'str'=>"The A\x0\x0Z",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"An A\0\x0\0Z",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'A',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"A\0Z",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'\x00',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"A\0\x0\0\x0Z",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'Z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A\x0{2,3}Z',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_alternative_without_second_operand_in_subpattern_and_back_reference() {
        $test1 = array( 'str'=>'cowcowbell',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>6),
                        'length'=>array(0=>10,1=>3,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bell',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>4,1=>0,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'cowbell',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(4),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(cow|)\1(bell)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_space_character_in_begin_of_line() {
        $test1 = array( 'str'=>"\040abc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"\x0cabc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"\nabc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"\rabc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"\tabc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\s',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\s',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\s',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

/*  function data_for_test_modifier_x_with_unknown_space_character() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a	b\n      c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_alternative_without_second_operand_in_subpattern_and_back_reference_in_quantifier() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'acb',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a|)\1*b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_alternative_without_second_operand_in_subpattern_and_back_reference_in_quantifier_2() {
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a|)\1+b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_alternative_without_second_operand_in_subpattern_and_back_reference_in_quantifier_3() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'acb',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a|)\1?b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_alternative_without_second_operand_in_subpattern_and_back_reference_in_quantifier_4() {
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a|)\1{2}b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_alternative_without_second_operand_in_subpattern_and_back_reference_in_quantifier_5() {
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'aaaaab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a|)\1{2,3}b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_character_in_quantifier_and_after_it() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'abbbbbc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{1,3}bc',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_negative_character_classes_in_quantifiers_in_subpatterns() {
        $test1 = array( 'str'=>'track1.title:TBlah blah blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7,3=>14),
                        'length'=>array(0=>28,1=>6,2=>5,3=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^.]*)\.([^:]*):[T ]+(.*)',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_classes_in_quantifiers_in_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'track1.title:TBlah blah blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7,3=>14),
                        'length'=>array(0=>28,1=>6,2=>5,3=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^.]*)\.([^:]*):[T ]+(.*)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_classes_in_quantifiers_in_subpatterns_modifier_i_2() {
        $test1 = array( 'str'=>'track1.title:TBlah blah blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7,3=>14),
                        'length'=>array(0=>28,1=>6,2=>5,3=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^.]*)\.([^:]*):[t ]+(.*)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_characters_different_case_in_character_class_in_quantifier() {
        $test1 = array( 'str'=>'WXY_^abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[W-c]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'wxy',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[W-c]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[W-c]+$',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_characters_different_case_in_character_class_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'WXY_^abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'wxy_^ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[W-c]+$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_hex_codes_of_characters_different_case_in_character_class_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'WXY_^abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'wxy_^ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\x3f-\x5F]+$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"qqq\nabc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"abc\nzzz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"qqq\nabc\nzzz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc$',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_tests_for_modifier_m_without_it() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"qqq\nabc",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abc\nzzz",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"qqq\nabc\nzzz",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_begin_and_end_of_string_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\n ", //- странный тест, совпадения быть не должно, но в исходном списке тестов у этого теста есть совпадение
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"qqq\nabc",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"abc\nzzz",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>"qqq\nabc\nzzz",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Aabc\Z',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_begin_and_end_of_string_and_any_character_in_quantifier_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"abc\ndef",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A(.)*\Z',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_and_end_of_string_and_any_character_in_quantifier_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>10),
                        'length'=>array(0=>11,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\ndef",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A(.)*\Z',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_unnumbered_subpatterns_in_alternative() {
        $test1 = array( 'str'=>'b::c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'c::b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:b)|(?::+)',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_with_hyphen_in_quantifier() {
        $test1 = array( 'str'=>'az-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[-az]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[-az]+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_with_hyphen_in_quantifier_2() {
        $test1 = array( 'str'=>'za-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[az-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[az-]+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_with_hyphen_in_quantifier_3() {
        $test1 = array( 'str'=>'a-z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'b',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a\-z]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[a\-z]+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_with_hyphen_in_quantifier_4() {
        $test1 = array( 'str'=>'abcdxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[a-z]+',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_with_hyphen_and_digit_in_quantifier() {
        $test1 = array( 'str'=>'12-34',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\d-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\d-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\d-]+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_with_hyphen_and_digit_in_quantifier_2() {
        $test1 = array( 'str'=>'12-34z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\d-z]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\d-z]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\d-z]+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_hex_code_of_backslash() {
        $test1 = array( 'str'=>'\\',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\x5c',
                     'tests'=>array($test1));
    }

    function data_for_test_hex_code_of_space() {
        $test1 = array( 'str'=>'the Zoo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'Z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Zulu',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\x20',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\x20Z',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_back_reference_modifier_i() {
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABCabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ABCABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\1',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_invalid_quantifier() {
        $test1 = array( 'str'=>'ab{3cd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{3cd',
                     'tests'=>array($test1));
    }

    function data_for_test_invalid_quantifier_2() {
        $test1 = array( 'str'=>'ab{3,cd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{3,cd',
                     'tests'=>array($test1));
    }

    function data_for_test_invalid_quantifier_3() {
        $test1 = array( 'str'=>'ab{3,4a}cd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{3,4a}cd',
                     'tests'=>array($test1));
    }

    function data_for_test_invalid_quantifier_4() {
        $test1 = array( 'str'=>'{4,5a}bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'{4,5a}bc',
                     'tests'=>array($test1));
    }

    function data_for_test_end_of_line() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abc\ndef",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_subpattern_and_oct_code_of_character() {
        $test1 = array( 'str'=>"abc\x53",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\123',
                     'tests'=>array($test1));
    }
/* на следующих 2 тестовых функциях выбрасывается исключение: Uninitialized string offset: 1
    function data_for_test_subpattern_and_oct_code_of_character_2() {
        $test1 = array( 'str'=>"abc\x93",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\223',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_and_oct_code_of_character_3() {
        $test1 = array( 'str'=>"abc\xd3",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\323',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_subpattern_and_oct_code_of_character_4() {
        $test1 = array( 'str'=>"abc\x40",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\100",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\100',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_subpattern_and_oct_code_of_character_5() {
        $test1 = array( 'str'=>"abc\x400",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\x40\x30",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"abc\1000",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abc\100\x30",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"abc\100\060",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>"abc\100\60",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\1000',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_oct_code_of_character() {
        $test1 = array( 'str'=>"abc\081",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\0\x38\x31",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\81',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_oct_code_of_character_2() {
        $test1 = array( 'str'=>"abc\091",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\0\x39\x31",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\91',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_many_subpatterns_and_back_reference_and_oct_code_of_character() {
        $test1 = array( 'str'=>'abcdefghijkllS',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5,7=>6,8=>7,9=>8,10=>9,11=>10,12=>11),
                        'length'=>array(0=>14,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1,11=>1,12=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)(b)(c)(d)(e)(f)(g)(h)(i)(j)(k)(l)\12\123',
                     'tests'=>array($test1));
    }

    function data_for_test_many_subpatterns_and_back_reference_and_oct_code_of_character_2() {
        $test1 = array( 'str'=>"abcdefghijk\12S",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5,7=>6,8=>7,9=>8,10=>9,11=>10),
                        'length'=>array(0=>13,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1,11=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)(b)(c)(d)(e)(f)(g)(h)(i)(j)(k)\12\123',
                     'tests'=>array($test1));
    }

    function data_for_test_backslash_i() {
        $test1 = array( 'str'=>'abidef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab\idef',
                     'tests'=>array($test1));
    }

    function data_for_test_null_quantifier() {
        $test1 = array( 'str'=>'bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a{0}bc',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_null_quantifier() {
        $test1 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|(bc)){0,0}?xyz',
                     'tests'=>array($test1));
    }

    function data_for_test_oct_code_of_character_in_character_class() {
        $test1 = array( 'str'=>"abc\010de",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc[\10]de',
                     'tests'=>array($test1));
    }

    function data_for_test_oct_code_of_character_in_character_class_2() {
        $test1 = array( 'str'=>"abc\1de",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc[\1]de',
                     'tests'=>array($test1));
    }

    function data_for_test_oct_code_of_character_in_character_class_3() {
        $test1 = array( 'str'=>"abc\1de",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)[\1]de',
                     'tests'=>array($test1));
    }

/*  function data_for_test_modifier_s_2() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?s)a.b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_negative_character_classes() { //символ \b не поддерживается
        $test1 = array( 'str'=>'baNOTccccd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>5),
                        'length'=>array(0=>9,1=>1,2=>1,3=>3,4=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'baNOTcccd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>5),
                        'length'=>array(0=>8,1=>1,2=>1,3=>3,4=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'baNOTccd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>4),
                        'length'=>array(0=>7,1=>1,2=>1,3=>2,4=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bacccd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>2),
                        'length'=>array(0=>5,1=>1,2=>1,3=>0,4=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>8),
                        'length'=>array(0=>11,1=>1,2=>1,3=>6,4=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'anything',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'[^a]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>"b\bc",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(4),
                        'next'=>'[^\b]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'baccd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>2),
                        'length'=>array(0=>4,1=>1,2=>1,3=>0,4=>2),
                        'left'=>array(1),
                        'next'=>'[^d]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^([^a])([^\b])([^c]*)([^d]{3,4})',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }
*/
    function data_for_test_negative_character_class() {
        $test1 = array( 'str'=>'Abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaAabcd ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_modifier_i() {
        $test1 = array( 'str'=>'Abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaAabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_quantifier() {
        $test1 = array( 'str'=>'AAAaAbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"bbb\nccc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]+',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_negative_character_class_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'AAAaAbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]+',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_before_end_of_line() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>10),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^k]$',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_quantifier_before_end_of_line() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'kbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'kabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>8),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'akb',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[^k]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'akk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[^k]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^k]{2,3}$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_quantifier_without_right_limit_and_negative_character_class_before_end_of_line() {
        $test1 = array( 'str'=>'12345678@a.b.c.d',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>16),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'123456789@x.y.z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>15),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(11),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'12345678@x.y.uk',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>14),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'1234567@a.b.c.d',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(4),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\d{8,}\@.+[^k]$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_back_reference_in_quantifier_without_right_limit() {
        $test1 = array( 'str'=>'aaaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>10,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(8),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>1),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)\1{8,}',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_negative_character_class_3() {
        $test1 = array( 'str'=>'aaaabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaAabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^az]',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_negative_character_class_modifier_i_2() {
        $test1 = array( 'str'=>'aaaabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaAabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^az]',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_oct_codes_of_all_ascii_characters() { //судя по сообщение об ошибке - наличие больше 100 функций (например, восьмеричных кодов символов) не поддерживается
        $test1 = array( 'str'=>"\000\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\057\060\061\062\063\064\065\066\067\070\071\072\073\074\075\076\077\100\101\102\103\104\105\106\107\110\111\112\113\114\115\116\117\120\121\122\123\124\125\126\127\130\131\132\133\134\135\136\137\140\141\142\143\144\145\146\147\150\151\152\153\154\155\156\157\160\161\162\163\164\165\166\167\170\171\172\173\174\175\176\177\200\201\202\203\204\205\206\207\210\211\212\213\214\215\216\217\220\221\222\223\224\225\226\227\230\231\232\233\234\235\236\237\240\241\242\243\244\245\246\247\250\251\252\253\254\255\256\257\260\261\262\263\264\265\266\267\270\271\272\273\274\275\276\277\300\301\302\303\304\305\306\307\310\311\312\313\314\315\316\317\320\321\322\323\324\325\326\327\330\331\332\333\334\335\336\337\340\341\342\343\344\345\346\347\350\351\352\353\354\355\356\357\360\361\362\363\364\365\366\367\370\371\372\373\374\375\376\377",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>256),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\000\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037\040\041\042\043\044\045\046\047\050\051\052\053\054\055\056\057\060\061\062\063\064\065\066\067\070\071\072\073\074\075\076\077\100\101\102\103\104\105\106\107\110\111\112\113\114\115\116\117\120\121\122\123\124\125\126\127\130\131\132\133\134\135\136\137\140\141\142\143\144\145\146\147\150\151\152\153\154\155\156\157\160\161\162\163\164\165\166\167\170\171\172\173\174\175\176\177\200\201\202\203\204\205\206\207\210\211\212\213\214\215\216\217\220\221\222\223\224\225\226\227\230\231\232\233\234\235\236\237\240\241\242\243\244\245\246\247\250\251\252\253\254\255\256\257\260\261\262\263\264\265\266\267\270\271\272\273\274\275\276\277\300\301\302\303\304\305\306\307\310\311\312\313\314\315\316\317\320\321\322\323\324\325\326\327\330\331\332\333\334\335\336\337\340\341\342\343\344\345\346\347\350\351\352\353\354\355\356\357\360\361\362\363\364\365\366\367\370\371\372\373\374\375\376\377',
                     'tests'=>array($test1));
    }
*//* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_negative_character_classes_and_lazy_quantifier() {
        $test1 = array( 'str'=>'xxxxxxxxxxxPSTAIREISLLxxxxxxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>11),
                        'length'=>array(0=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'P[^*]TAIRE[^*]{1,6}?LL',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_negative_character_classes_and_lazy_quantifier_without_right_limit() {
        $test1 = array( 'str'=>'xxxxxxxxxxxPSTAIREISLLxxxxxxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>11),
                        'length'=>array(0=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'P[^*]TAIRE[^*]{1,}?LL',
                     'tests'=>array($test1));
    }

    function data_for_test_digits_in_quantifiers() {
        $test1 = array( 'str'=>'1.230003938',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>10,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1.875000282',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'1.235',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\.\d\d[1-9]?)\d+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_positive_lookahead_assertion_in_alternative() {
        $test1 = array( 'str'=>'1.230003938',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1,2=>4),
                        'length'=>array(0=>3,1=>3,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1.875000282',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1,2=>4),
                        'length'=>array(0=>4,1=>4,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'1.235',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1,1=>1,2=>4),
                        'length'=>array(0=>4,1=>4,2=>1),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\.\d\d((?=0)|\d(?=\d)))',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_unknown() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?)b',
                     'tests'=>array($test1));
    }

    function data_for_test_word_boundary_modifier_i() {
        $test1 = array( 'str'=>'Food is on the foo table',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>15,1=>15,2=>19),
                        'length'=>array(0=>9,1=>3,2=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\b(foo)\s+(\w+)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_in_greedy_quantifier() {
        $test1 = array( 'str'=>'The food is under the bar in the barn.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>7),
                        'length'=>array(0=>32,1=>26),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'foo(.*)bar',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_lazy_quantifier() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'The food is under the bar in the barn.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>7),
                        'length'=>array(0=>21,1=>15),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'foo(.*?)bar',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_and_digit_in_greedy_quantifiers_in_subpatterns() {
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>23),
                        'length'=>array(0=>23,1=>23,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)(\d*)',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_and_digit_in_greedy_quantifiers_in_subpatterns_2() {
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>22),
                        'length'=>array(0=>23,1=>22,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)(\d+)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_lazy_quantifier_and_digit_in_greedy_quantifier_in_subpatterns() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>0,1=>0,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*?)(\d*)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_and_digit_in_greedy_quantifier_in_subpatterns_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7),
                        'length'=>array(0=>8,1=>7,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*?)(\d+)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_and_digit_in_greedy_quantifiers_in_subpatterns_before_end_of_line() {
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>22),
                        'length'=>array(0=>23,1=>22,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)(\d+)$',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_lazy_quantifier_and_digit_in_greedy_quantifier_in_subpatterns_before_end_of_line() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>18),
                        'length'=>array(0=>23,1=>18,2=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*?)(\d+)$',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_and_digit_in_greedy_quantifiers_in_subpatterns_and_word_boundary_before_end_of_line() {
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>18),
                        'length'=>array(0=>23,1=>18,2=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)\b(\d+)$',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_and_digit_in_greedy_quantifiers_and_not_digit_in_subpatterns_before_end_of_line() {
        $test1 = array( 'str'=>'I have 2 numbers: 53147',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>18),
                        'length'=>array(0=>23,1=>18,2=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*\D)(\d+)$',
                     'tests'=>array($test1));
    }

    function data_for_test_not_digit_in_quantifier_and_negative_lookahead_assertion() {
        $test1 = array( 'str'=>'ABC123',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\D*(?!123)',
                     'tests'=>array($test1));
    }

    function data_for_test_not_digit_in_quantifier_and_positive_and_negative_lookahead_assertions() {
        $test1 = array( 'str'=>'ABC445',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABC123',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\D*)(?=\d)(?!123)',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_hyphen_in_character_class_and_square_closing_bracket_outside_character_class() {
        $test1 = array( 'str'=>'W46]789 ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'-46]789',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'[W-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Wall',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'4',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'Zebra',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'[W-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'42',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'[W-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'[abcd] ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'[W-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>']abcd[',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'[W-]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[W-]46]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

    function data_for_test_hyphen_and_square_closing_bracket_in_character_class() {
        $test1 = array( 'str'=>'W46]789 ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Wall',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Zebra',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Xylophone  ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'42',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'[abcd] ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>']abcd[',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'\\backslash ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[W-\]46]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'-46]789',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[W-\]46]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'well',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[W-\]46]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[W-\]46]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11));
    }

    function data_for_test_digits_separated_by_slashes() {
        $test1 = array( 'str'=>'01/01/2000',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\d\d\/\d\d\/\d\d\d\d',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_unnumbered_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'word cat dog elephant mussel cow horse canary baboon snake shark otherword',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>74),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'word cat dog elephant mussel cow horse canary baboon snake shark',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>59),
                        'left'=>array(9),
                        'next'=>'o',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'word (?:[a-zA-Z0-9]+ ){0,10}otherword',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_quantifier_in_unnumbered_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'word cat dog elephant mussel cow horse canary baboon snake shark the quick brown fox and the lazy dog and several other words getting close to thirty by now I hope',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>159),
                        'left'=>array(9),
                        'next'=>'o',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'word (?:[a-zA-Z0-9]+ ){0,300}otherword',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_null_quantifier() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){0,0}',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){0,1}',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_subpattern_in_quantifier_3() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){0,2}',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_subpattern_in_quantifier_4() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){0,3}',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_subpattern_in_quantifier_5() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){0,}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_subpattern_in_quantifier_6() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab  ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){1,1}',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_subpattern_in_quantifier_7() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){1,2}',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_subpattern_in_quantifier_8() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){1,3}',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_subpattern_in_quantifier_9() {
        $test1 = array( 'str'=>'bcd',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){1,}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_any_character_in_quantifier_and_dot_gif() {
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*\.gif',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_in_quantifier_and_dot_gif_2() {
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.{0,}\.gif',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_quantifier_and_dot_gif_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*\.gif',
                     'modifiers'=>'m',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_and_dot_gif_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*\.gif',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_and_dot_gif_modifiers_ms() { //модификаторы m и s не поддерживаются
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*\.gif',
                     'modifiers'=>'ms',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_in_quantifier_before_end_of_line() {
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>15),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_quantifier_before_end_of_line_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'modifiers'=>'m',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_before_end_of_line_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_before_end_of_line_modifiers_ms() { //модификаторы m и s не поддерживаются
        $test1 = array( 'str'=>"borfle\nbib.gif\nno",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'modifiers'=>'ms',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_in_quantifier_before_end_of_line_2() {
        $test1 = array( 'str'=>"borfle\nbib.gif\nno\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>15),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_quantifier_before_end_of_line_modifier_m_2() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"borfle\nbib.gif\nno\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'modifiers'=>'m',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_before_end_of_line_modifier_s_2() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"borfle\nbib.gif\nno\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>18),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_before_end_of_line_modifiers_ms_2() { //модификаторы m и s не поддерживаются
        $test1 = array( 'str'=>"borfle\nbib.gif\nno\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>18),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*$',
                     'modifiers'=>'ms',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_in_quantifier_or_begin_of_line_in_alternative() {
        $test1 = array( 'str'=>"abcde\n1234Xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6,1=>6),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BarFoo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abcde\nBar",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*X|^B)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_any_character_in_quantifier_or_begin_of_line_in_alternative_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"abcde\n1234Xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6,1=>6),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BarFoo ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"abcde\nBar  ",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6,1=>6),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*X|^B)',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_any_character_in_quantifier_or_begin_of_line_in_alternative_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"abcde\n1234Xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BarFoo ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abcde\nBar  ",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*X|^B)',
                     'modifiers'=>'s',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_any_character_in_quantifier_or_begin_of_line_in_alternative_modifiers_ms() { //модификаторы m и s не поддерживаются
        $test1 = array( 'str'=>"abcde\n1234Xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BarFoo ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"abcde\nBar  ",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6,1=>6),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*X|^B)',
                     'modifiers'=>'ms',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_any_character_in_quantifier_or_begin_of_line_in_alternative_modifier_s_in_regexp() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"abcde\n1234Xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BarFoo ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abcde\nBar  ",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>11,1=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?s)(.*X|^B)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_any_character_in_quantifier_or_begin_of_line_in_alternative_modifier_s_in_unnumbered_subpattern() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"abcde\n1234Xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BarFoo ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"abcde\nBar  ",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?s:.*X|^B)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_any_character_in_quantifier_after_begin_of_line() {
        $test1 = array( 'str'=>'**** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'left'=>array(1),
                        'next'=>'B',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\nB",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'B',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*B',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_any_character_in_quantifier_after_begin_of_line_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"abc\nB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?s)^.*B',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_after_begin_of_line_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"abc\nB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?m)^.*B',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_after_begin_of_line_modifiers_ms() { //модификаторы m и s не поддерживаются
        $test1 = array( 'str'=>"abc\nB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?ms)^.*B',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_line_modifiers_ms() { //модификаторы m и s не поддерживаются
        $test1 = array( 'str'=>"abc\nB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?ms)^B',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_end_of_line_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"B\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?s)B$',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_many_digits() {
        $test1 = array( 'str'=>'123456654321',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]',
                     'tests'=>array($test1));
    }

    function data_for_test_many_digits_2() {
        $test1 = array( 'str'=>'123456654321 ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\d\d\d\d\d\d\d\d\d\d\d\d',
                     'tests'=>array($test1));
    }

    function data_for_test_many_digits_3() {
        $test1 = array( 'str'=>'123456654321',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\d][\d][\d][\d][\d][\d][\d][\d][\d][\d][\d][\d]',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_3() {
        $test1 = array( 'str'=>'abcabcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[abc]{12}',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_4() {
        $test1 = array( 'str'=>'abcabcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a-c]{12}',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_quantifier_10() {
        $test1 = array( 'str'=>'abcabcabcabc ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>11),
                        'length'=>array(0=>12,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a|b|c){12}',
                     'tests'=>array($test1));
    }

    function data_for_test_big_character_class() {
        $test1 = array( 'str'=>'n',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[abcdefghijklmnopqrstuvwxy0123456789]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'z ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[abcdefghijklmnopqrstuvwxy0123456789]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[abcdefghijklmnopqrstuvwxy0123456789]',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_null_quantifier_2() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abce  ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abcde{0,0}',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_in_null_quantifier() {
        $test1 = array( 'str'=>'abe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcde ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'e',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab[cd]{0,0}e',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_subpattern_in_null_quantifier_2() {
        $test1 = array( 'str'=>'abd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab(c){0,0}d',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>5,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5,1=>6),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bbbbb',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(b*)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_digit_in_null_quantifier() {
        $test1 = array( 'str'=>'abe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ab1e',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'e',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab\d{0}e',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_quantifier_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'the "quick" brown fox',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>5),
                        'length'=>array(0=>7,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"\"the \"quick\" brown fox\"",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>14),
                        'length'=>array(0=>25,1=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'"([^\\"]+|\\.)*"',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_html_tags_modifiers_is() { //модификаторы i и s не поддерживаются
        $test1 = array( 'str'=>'<TR BGCOLOR=\'#DBE9E9\'><TD align=left valign=top>43.<a href=\'joblist.cfm?JobID=94 6735&Keyword=\'>Word Processor<BR>(N-1286)</a></TD><TD align=left valign=top>Lega lstaff.com</TD><TD align=left valign=top>CA - Statewide</TD></TR>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>25,3=>48,4=>51,5=>122,6=>122,8=>134,9=>157,10=>180,11=>203),
                        'length'=>array(0=>227,1=>18,2=>22,3=>3,4=>71,5=>0,6=>0,8=>22,9=>15,10=>22,11=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'<tr([\w\W\s\d][^<>]{0,})><TD([\w\W\s\d][^<>]{0,})>([\d]{0,}\.)(.*)((<BR>([\w\W\s\d][^<>]{0,})|[\s]{0,}))<\/a><\/TD><TD([\w\W\s\d][^<>]{0,})>([\w\W\s\d][^<>]{0,})<\/TD><TD([\w\W\s\d][^<>]{0,})>([\w\W\s\d][^<>]{0,})<\/TD><\/TR>',
                     'modifiers'=>'is',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_negative_character_class_4() {
        $test1 = array( 'str'=>'acb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^a]b',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_character() {
        $test1 = array( 'str'=>'acb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.b',
                     'tests'=>array($test1, $test2, $test3));
    }

/*  function data_for_test_negative_character_class_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>'acb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^a]b',
                     'modifiers'=>'s',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>'acb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.b',
                     'modifiers'=>'s',
                     'tests'=>array($test1, $test2));
    }
*//* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_subpattern_and_alternative_and_lazy_quantifiers_2() {
        $test1 = array( 'str'=>'bac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bbbbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bbbbbac ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(b+?|a){1,2}?c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_subpattern_and_alternative_and_greedy_and_lazy_quantifiers_2() {
        $test1 = array( 'str'=>'bac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bbbbac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bbbbbac ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(b+|a){1,2}?c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_begin_of_string_in_negative_lookahead_assertion_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"x\nb\n",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"a\bx\n  ",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?!\A)x',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_hex_character_code() {
        $test1 = array( 'str'=>"\0{ab}",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\x0{ab}',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_alternative_in_subpattern_in_lazy_quantifier() {
        $test1 = array( 'str'=>'CD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(A|B)*?CD',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_in_greedy_quantifier() {
        $test1 = array( 'str'=>'CD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(A|B)*CD',
                     'tests'=>array($test1));
    }

/*  function data_for_test_subpattern_in_lazy_quantifier_and_back_reference() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ABABAB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(AB)*?\1',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_subpattern_in_greedy_quantifier_and_back_reference() {
        $test1 = array( 'str'=>'ABABAB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>6,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(AB)*\1',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_lookbehind_assertion() {
        $test1 = array( 'str'=>'foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'catfood',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'arfootle',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'rfoosh',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'barfoo',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'towbarfoo',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<!bar)foo',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_negative_lookbehind_assertion_2() {
        $test1 = array( 'str'=>'catfood',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'foo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'barfoo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(4),
                        'next'=>'[^r]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'towbarfoo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\w{3}(?<!bar)foo',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_subpattern_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'fooabar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(7),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bar',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(7),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'foobbar',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>3),
                        'left'=>array(4),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(foo)a)bar',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_begin_and_end_of_string_modifier_m_2() { //модификатор m не поддерживается
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"abc\n",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"qqq\nabc",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"abc\nzzz",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>"qqq\nabc\nzzz",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Aabc\z',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'/this/is/a/very/long/line/in/deed/with/very/many/slashes/in/it/you/see/',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>71),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'/this/is/a/very/long/line/in/deed/with/very/many/slashes/in/and/foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>67),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*\/)foo',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_digits_in_subpattern_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'1.230003938',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>10,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'1.875000282',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'\.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'1.235',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>4,1=>4),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(\.\d\d[1-9]?))\d+',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
 /*  function data_for_test_escape_sequences_in_quantifiers_in_atomic_groups_in_alternative_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'now is the time for all good men to come to the aid of the party',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>59),
                        'length'=>array(0=>64,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'this is not a line with only words and spaces!',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>45,1=>45),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^((?>\w+)|(?>\s+))*$',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_digits_in_quantifier_and_word_character_in_subpatterns() {
        $test1 = array( 'str'=>'12345a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>5),
                        'length'=>array(0=>6,1=>5,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12345+ ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>4),
                        'length'=>array(0=>5,1=>4,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\d+)(\w)',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_digits_in_quantifier_in_atomic_group_and_word_character_in_subpatterns() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'12345a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>5),
                        'length'=>array(0=>6,1=>5,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12345+ ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>5),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>\d+))(\w)',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_quantifier_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a+)b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_atomic_group_in_subpattern() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>a+)b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_atomic_group_in_subpattern_2() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(a+))b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_atomic_group_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaabbbccc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>b)+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifiers_in_alternatives_in_atomic_group_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaabbbbccccd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a+|b+|c+)*c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_character_class_in_quantifier_in_atomic_group_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'((abc(ade)ufh()()x',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>17),
                        'length'=>array(0=>16,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>[^()]+)|\([^()]*\))+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_character_class_in_quantifier_in_atomic_group_in_subpattern_in_quantifier_2() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'(abc)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'(abc(def)xyz)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>9),
                        'length'=>array(0=>13,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'\(',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'((()aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa   ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(2),
                        'next'=>'[^()]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\(((?>[^()]+)|\([^()]+\))+\)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_modifier_i_and_not_modifier_i() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?-i)b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_modifier_x_in_subpattern() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'a bcd e',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(6),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a b cd e',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(4),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcd e   ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(6),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'a bcde ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>5),
                        'left'=>array(2),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a (?x)b c)d e',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_modifier_x_and_not_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'a bcde f',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>8,1=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(7),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcdef  ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(7),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a b(?x)c d (?-x)e f)',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_modifier_i_in_subpattern() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'Abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'ABc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'ABC',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'AbC',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a(?i)b)c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }

    function data_for_test_modifier_i_in_subpattern_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ABC',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?i:b)c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_modifier_i_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aBc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBBc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aBBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?i:b)*c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_modifier_i_in_positive_lookahead_assertion() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abCd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aBCd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abcD     ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?=b(?i)c)\w\wd',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_modifiers_s_not_i() { //модификатор s не поддерживается
        $test1 = array( 'str'=>'more than million',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'more than MILLION',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"more \n than Million",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(15),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'MORE THAN MILLION',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(15),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>"more \n than \n million",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'left'=>array(7),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?s-i:more.*than).*million',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_modifiers_s_not_i_2() { //модификатор s не поддерживается
        $test1 = array( 'str'=>'more than million',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'more than MILLION',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"more \n than Million",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(15),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'MORE THAN MILLION',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(15),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>"more \n than \n million",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'left'=>array(7),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?s-i)more.*than).*million',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_modifier_i_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aBBc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'Abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'abAb',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'abbC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a(?i)b+)+c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }
*/
    function data_for_test_modifier_i_in_positive_lookahead_assertion_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Ab',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(?i)b)\w\wc',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_modifier_i_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'abxxc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBxxc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Abxxc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ABxxc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(5),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'abxxC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a(?i)b)(\w\w)c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

/*  function data_for_test_unnumbered_and_conditional_subpatterns() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aA',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'A',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'A',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bA',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'B',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(a)|b)(?(1)A|B)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_conditional_subpattern_in_quantifier() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)?(?(1)a|b)+$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_positive_lookahead_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abc:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'123',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'xyz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?(?=abc)\w{3}:|\d\d)$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_negative_lookahead_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abc:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'123',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'xyz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?(?!abc)\d\d|\w{3}:)$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_positive_lookbehind_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'cat',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'fcat',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'focat',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'foocat',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?<=foo)bar|cat)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_negative_lookbehind_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'cat',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'fcat',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'focat',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'foocat',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?<!foo)cat|bar)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_conditional_subpattern_modifier_x() { //модификатор x и условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'(abcd)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'the quick (abcd) fox',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'(abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'( \( )? [^()]+ (?(1) \) |) ',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_conditional_subpattern_modifier_x_2() { //модификатор x и условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'(abcd)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'the quick (abcd) fox',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'(abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'( \( )? [^()]+ (?(1) \) ) ',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_subpatterns_and_back_reference_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'12',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>3,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>4,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'1',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'1234',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?(2)a|(1)(2))+$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
    function data_for_test_modifier_i_in_subpattern_and_back_reference() {
        $test1 = array( 'str'=>'blah blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BLAH BLAH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Blah Blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'blaH blaH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(9),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'blah BLAH',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>4),
                        'left'=>array(4),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'Blah blah',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>4),
                        'left'=>array(4),
                        'next'=>'B',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'blaH blah',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>8,1=>4),
                        'left'=>array(1),
                        'next'=>'H',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?i)blah)\s+\1',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

    function data_for_test_back_reference_and_modifier_i_in_subpattern() {
        $test1 = array( 'str'=>'blah blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BLAH BLAH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Blah Blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'blaH blaH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'blah BLAH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'Blah blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'blaH blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?i)blah)\s+(?i:\1)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

/*  function data_for_test_quantifier_in_atomic_group_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a*)*',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>6,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>9),
                        'length'=>array(0=>9,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc|)+',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_character_class_in_quantifier_in_subpattern_in_quantifier() { //дальше идут какие-то странные, пустые тесты, хотя, вроде бы, не должны быть такими
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([a]*)*',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ababab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaabcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([ab]*)*',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_negative_character_class_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^a]*)*',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'cccc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^ab]*)*',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_lazy_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([a]*?)*',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_lazy_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'baba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([ab]*?)*',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_negative_character_class_in_lazy_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^a]*?)*',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_lazy_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'cccc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'baba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^ab]*?)*',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_quantifier_in_atomic_group_in_quantifier_2() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaabcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a*)*',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_greedy_quantifier_in_atomic_group_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>5,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aabbaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>6),
                        'length'=>array(0=>2,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>a*))*',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_lazy_quantifier_in_atomic_group_in_subpattern_in_quantifier() { //ленивые квантификаторы не поддерживаются
        $test1 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aabbaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>a*?))*',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_digits_in_conditional_subpattern_modifier_x() { //модификатор x и условные подмаски не поддерживаются
        $test1 = array( 'str'=>'12-sep-98',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12-09-98',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(8),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'sep-12-98',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>5),
                        'left'=>array(3),
                        'next'=>'-',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=[^a-z]+[a-z])  \d{2}-[a-z]{3}-\d{2}  |  \d{2}-\d{2}-\d{2} ) ',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_subpattern_in_positive_lookahead_assertion_and_back_reference() {
        $test1 = array( 'str'=>'foobarfoo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'foobarfootling',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(9),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'barfoo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6,1=>3),
                        'length'=>array(0=>0,1=>3),
                        'left'=>array(6),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(foo))bar\1',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_alternative_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'saturday',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'sunday',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Saturday',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Sunday',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'SATURDAY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'SUNDAY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'SunDay',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?i:saturday|sunday)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_alternative_in_subpattern_modifier_i_2() {
        $test1 = array( 'str'=>'abcx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBCx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'BBx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'abcX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(1),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'aBCX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(1),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'bbX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'BBX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a(?i)bc|BB)x',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }

    function data_for_test_character_class_in_alternative_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'elephant',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'Europe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'frog',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'France',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[efEF]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'Africa',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[efEF]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^([ab](?i)[cd]|[ef])',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }

/*  function data_for_test_alternatives_in_subpattern_modifier_i_and_m_not_i_and_i() { //модификатор m не поддерживается
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aBd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'zebra',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'Zambesi',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aCD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'XY',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(ab|a(?i)[b-c](?m-i)d|x(?i)y|z)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9));
    }
*/
/*  function data_for_test_positive_lookbehind_assertion_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"foo\nbar",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(7),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bar',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(7),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"baz\nbar",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(7),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=foo\n)^bar',
                     'modifiers'=>'m',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_negative_lookbehind_assertion_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'barbaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'barbarbaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'koobarbaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'baz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'r',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'foobarbaz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(?<!foo)bar)baz',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
/*The cases of aaaa and aaaaaa are missed out below because Perl does things differently. We know that odd, and maybe incorrect, things happen with recursive references in Perl, as far as 5.11.3 - see some stuff in test #2.*/
    function data_for_test_back_reference_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'aaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'aaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'aaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'aaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>'aaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'aaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test14 = array( 'str'=>'aaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a\1?){4}$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13, $test14));
    }

    function data_for_test_back_references_in_quantifiers_in_subpatterns() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>1,1=>1,2=>1),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2),
                        'length'=>array(0=>1,1=>1,2=>1,3=>1),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3),
                        'length'=>array(0=>4,1=>1,2=>1,3=>1,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>4),
                        'length'=>array(0=>5,1=>1,2=>2,3=>1,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>4),
                        'length'=>array(0=>6,1=>1,2=>2,3=>1,4=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'aaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>7,1=>1,2=>2,3=>3,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>7,1=>1,2=>2,3=>3,4=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'aaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>7,1=>1,2=>2,3=>3,4=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'aaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'aaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>'aaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'aaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test14 = array( 'str'=>'aaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test15 = array( 'str'=>'aaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test16 = array( 'str'=>'aaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3,4=>6),
                        'length'=>array(0=>10,1=>1,2=>2,3=>3,4=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a\1?)(a\1?)(a\2?)(a\3?)$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13, $test14, $test15, $test16));
    }
/*The following tests are taken from the Perl 5.005 test suite; some of them are compatible with 5.004, but I'd rather not have to sort them out.*/
    function data_for_test_characters() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xabcy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ababc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'xbc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'axc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'abx',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_quantifier() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*c',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*bc',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_any_character_in_const_quantifier() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.{1}',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_in_quantifier_2() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.{3,4}',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_3() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{0,}bc',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_4() {
        $test1 = array( 'str'=>'abbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab+bc',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_quantifier_5() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{1,}bc',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_6() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{1,3}bc',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_7() {
        $test1 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{3,4}bc',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_8() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(6),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(5),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{4,5}bc',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifier_9() {
        $test1 = array( 'str'=>'abbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab?bc',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_quantifier_10() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{0,1}bc',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_11() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab?c',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_12() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{0,1}c',
                     'tests'=>array($test1));
    }

    function data_for_test_begin_and_end_of_line() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abbbbc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_begin_of_line() {
        $test1 = array( 'str'=>'abcc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc',
                     'tests'=>array($test1));
    }

    function data_for_test_end_of_line_2() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aabcd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_begin_of_line_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^',
                     'tests'=>array($test1));
    }

    function data_for_test_end_of_line_3() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'$',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'axc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.c',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_character_in_quantifier_3() {
        $test1 = array( 'str'=>'axyzc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.*c',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class() {
        $test1 = array( 'str'=>'abd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bc]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'axyzd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bc]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[bc]d',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_character_class_2() {
        $test1 = array( 'str'=>'ace',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[b-d]e',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_3() {
        $test1 = array( 'str'=>'aac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[b-d]',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_4() {
        $test1 = array( 'str'=>'a-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[-b]',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_5() {
        $test1 = array( 'str'=>'a-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[b-]',
                     'tests'=>array($test1));
    }

    function data_for_test_square_closing_bracket_without_character_class() {
        $test1 = array( 'str'=>'a]',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a]',
                     'tests'=>array($test1));
    }

    function data_for_test_square_closing_bracket_in_character_class() {
        $test1 = array( 'str'=>'a]b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[]]b',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_2() {
        $test1 = array( 'str'=>'aed',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[^bc]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^bc]d',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_hyphen_in_negative_character_class() {
        $test1 = array( 'str'=>'adc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^-b]c',
                     'tests'=>array($test1));
    }

    function data_for_test_square_closing_bracket_without_backslash_at_beginning_of_the_negative_character_class_2() {
        $test1 = array( 'str'=>'adc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a-c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a]c',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[^]b]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^]b]c',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_word() {
        $test1 = array( 'str'=>'a-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'-a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-a-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\ba\b',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_word_2() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'y',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xy',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'y',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'yz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xyz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'y',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\by\b',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_not_word() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a-',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-a',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'-a-',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Ba\B',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_end_of_word() {
        $test1 = array( 'str'=>'xy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\By\b',
                     'tests'=>array($test1));
    }

    function data_for_test_begin_of_word() {
        $test1 = array( 'str'=>'yz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\by\B',
                     'tests'=>array($test1));
    }

    function data_for_test_not_word_2() {
        $test1 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\By\B',
                     'tests'=>array($test1));
    }

    function data_for_test_word_character() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\w',
                     'tests'=>array($test1));
    }

    function data_for_test_not_word_character() {
        $test1 = array( 'str'=>'-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\W',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\W',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_space_character() {
        $test1 = array( 'str'=>'a b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\sb',
                     'tests'=>array($test1));
    }

    function data_for_test_not_space_character() {
        $test1 = array( 'str'=>'a-b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a b',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'\S',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\Sb',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_digit() {
        $test1 = array( 'str'=>'1',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\d',
                     'tests'=>array($test1));
    }

    function data_for_test_not_digit() {
        $test1 = array( 'str'=>'-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'1',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\D',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\D',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_word_character_in_character_class() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\w]',
                     'tests'=>array($test1));
    }

    function data_for_test_not_word_character_in_character_class() {
        $test1 = array( 'str'=>'-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\W]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\W]',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_space_character_in_character_class() {
        $test1 = array( 'str'=>'a b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[\s]b',
                     'tests'=>array($test1));
    }

    function data_for_test_not_space_character_in_character_class() {
        $test1 = array( 'str'=>'a-b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a b',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[\S]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[\S]b',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_digit_in_character_class() {
        $test1 = array( 'str'=>'1',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\d]',
                     'tests'=>array($test1));
    }

    function data_for_test_not_digit_in_character_class() {
        $test1 = array( 'str'=>'-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'1',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\D]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\D]',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_alternative() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab|cd',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_empty_subpattern() {
        $test1 = array( 'str'=>'def',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>2,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'()ef',
                     'tests'=>array($test1));
    }

    function data_for_test_round_opening_bracket() {
        $test1 = array( 'str'=>'a(b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\(b',
                     'tests'=>array($test1));
    }

    function data_for_test_round_opening_bracket_in_quantifier() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a((b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\(*b',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_backslash() { //по неизвестной мне причине вызывает PHP Fatal Error: Вызов неопределенного метода qtype_preg_node_error::match()
        $test1 = array( 'str'=>'a\\b',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'\\',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\\\\b',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_nested_subpatterns() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>1,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((a))',
                     'tests'=>array($test1));
    }

    function data_for_test_subpatterns() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>2),
                        'length'=>array(0=>3,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)b(c)',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_2() {
        $test1 = array( 'str'=>'aabbabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b+c',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_3() {
        $test1 = array( 'str'=>'aabbabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a{1,}b{1,}c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_lazy_quantifier_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.+?c',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b)*',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){0,}',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_3() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b)+',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_4() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){1,}',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_5() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b)?',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_6() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){0,1}',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_in_quantifier_2() {
        $test1 = array( 'str'=>'cde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^ab]*',
                     'tests'=>array($test1));
    }

    function data_for_test_simple_regexp_2() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'b',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_quantifier_13() {
        $test1 = array( 'str'=>'',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'abbbcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([abc])*d',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([abc])*bcd',
                     'tests'=>array($test1));
    }

    function data_for_test_many_alternatives() {
        $test1 = array( 'str'=>'e',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a|b|c|d|e',
                     'tests'=>array($test1));
    }

    function data_for_test_many_alternatives_in_subpattern() {
        $test1 = array( 'str'=>'ef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|b|c|d|e)f',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_14() {
        $test1 = array( 'str'=>'abcdefg',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abcd*efg',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_15() {
        $test1 = array( 'str'=>'xabyabbbz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xayabbbz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_alternative_in_subpattern() {
        $test1 = array( 'str'=>'abcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab|cd)e',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_6() {
        $test1 = array( 'str'=>'hij',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[abhgefdc]ij',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_without_second_branch_in_subpattern() {
        $test1 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>2,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc|)ef',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_and_quantifier() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|b)c*d',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab|ab*)bc',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]*)c*',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>3),
                        'length'=>array(0=>4,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]*)(c*d)',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier_in_subpattern_2() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>3),
                        'length'=>array(0=>4,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]+)(c*d)',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier_in_subpattern_3() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>2),
                        'length'=>array(0=>4,1=>1,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]*)(c+d)',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_5() {
        $test1 = array( 'str'=>'adcdcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[bcd]*dcdcde',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_6() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(7),
                        'next'=>'[bcd]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abcde',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(5),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'adcdcde',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'left'=>array(3),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[bcd]+dcdcde',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_alternative_in_subpattern_and_quantifier_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab|a)b*c',
                     'tests'=>array($test1));
    }

    function data_for_test_many_subpatterns_2() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>1,4=>3),
                        'length'=>array(0=>4,1=>3,2=>1,3=>1,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((a)(b)c)(d)',
                     'tests'=>array($test1));
    }

    function data_for_test_character_classes() {
        $test1 = array( 'str'=>'alpha',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[a-zA-Z_][a-zA-Z0-9_]*',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_and_character_class_in_alternative_in_subpattern_in_alternative() {
        $test1 = array( 'str'=>'abh',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a(bc+|b[eh])g|.h$',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_and_end_of_line_and_subpattern_in_alternatives_in_subpattern() {
        $test1 = array( 'str'=>'effgz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ij',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'reffgz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[jk]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'effg',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'bcdd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(bc+d$|ef*g.|h?i(j|k))',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_many_nested_subpatterns() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>0,4=>0,5=>0,6=>0,7=>0,8=>0,9=>0,10=>0),
                        'length'=>array(0=>1,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((((((((((a))))))))))',
                     'tests'=>array($test1));
    }

    function data_for_test_many_nested_subpatterns_and_back_reference() {
        $test1 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>0,4=>0,5=>0,6=>0,7=>0,8=>0,9=>0,10=>0),
                        'length'=>array(0=>2,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((((((((((a))))))))))\10',
                     'tests'=>array($test1));
    }

    function data_for_test_many_nested_subpatterns_2() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>0,4=>0,5=>0,6=>0,7=>0,8=>0,9=>0),
                        'length'=>array(0=>1,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(((((((((a)))))))))',
                     'tests'=>array($test1));
    }

    function data_for_test_simple_regexp_3() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aa',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'uh-uh',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'multiple words of text',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_simple_regexp_4() {
        $test1 = array( 'str'=>'multiple words, yeah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'multiple words',
                     'tests'=>array($test1));
    }

    function data_for_test_any_characters_in_quantifiers_in_subpatterns() {
        $test1 = array( 'str'=>'abcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3),
                        'length'=>array(0=>5,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)c(.*)',
                     'tests'=>array($test1));
    }

    function data_for_test_any_characters_in_quantifiers_in_subpatterns_2() {
        $test1 = array( 'str'=>'(a, b)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>4),
                        'length'=>array(0=>6,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\((.*), (.*)\)',
                     'tests'=>array($test1));
    }

    function data_for_test_simple_regexp_5() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abcd',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(bc)d',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_7() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[-]?c',
                     'tests'=>array($test1));
    }

    function data_for_test_back_reference_2() {
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\1',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_back_reference() {
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([a-c]*)\1',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_and_back_reference_in_alternative() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'x',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)|\1',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_character_class_in_subpattern_and_lazy_quantifier_and_back_reference_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'ababbbcbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>3),
                        'length'=>array(0=>5,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(([a-c])b*?\2)*',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_subpattern_and_lazy_quantifier_and_back_reference_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'ababbbcbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6,2=>6),
                        'length'=>array(0=>9,1=>3,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(([a-c])b*?\2){3}',
                     'tests'=>array($test1));
    }

    function data_for_test_back_reference_in_alternative_in_subpattern_and_back_reference_and_subpattern_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aaaxabaxbaaxbbax',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>12,1=>12,2=>12,3=>14),
                        'length'=>array(0=>4,1=>4,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((\3|b)\2(a)x)+',
                     'tests'=>array($test1));
    }

    function data_for_test_back_reference_in_alternative_in_subpattern_and_back_reference_and_subpattern_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'bbaababbabaaaaabbaaaabba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>15,1=>21,2=>21,3=>23),
                        'length'=>array(0=>9,1=>3,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((\3|b)\2(a)){2,}',
                     'tests'=>array($test1));
    }

    function data_for_test_modifier_i_2() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'XABCY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aaxabxbaxbbx',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[cC]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'XBC',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'[aA]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'AXC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'ABX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[cC]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

    function data_for_test_quantifier_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i() {
        $test1 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{0,}?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_3() {
        $test1 = array( 'str'=>'ABBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab+?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_modifier_i_3() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABQ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab+bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_4() {
        $test1 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{1,}?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_5() {
        $test1 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{1,3}?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_6() {
        $test1 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{3,4}?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_7() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(6),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABQ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(5),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{4,5}?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_8() {
        $test1 = array( 'str'=>'ABBC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab??bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_9() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{0,1}?bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_10() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab??c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_lazy_quantifier_modifier_i_11() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab{0,1}?c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_begin_and_end_of_line_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'[aA]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABBBBC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[cC]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ABCC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_begin_of_line_modifier_i() {
        $test1 = array( 'str'=>'ABCC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_end_of_line_modifier_i() {
        $test1 = array( 'str'=>'AABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc$',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_begin_of_line_modifier_i_2() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_end_of_line_modifier_i_2() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'$',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'AXC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.c',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_any_character_in_lazy_quantifier_modifier_i() {
        $test1 = array( 'str'=>'AXYZC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.*?c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_in_greedy_quantifier_modifier_i() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>6),
                        'left'=>array(1),
                        'next'=>'[cC]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'AABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'AXYZD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>'[cC]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.*c',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_modifier_i() {
        $test1 = array( 'str'=>'ABD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[bc]d',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_modifier_i_2() {
        $test1 = array( 'str'=>'ACE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[b-dB-D]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[eE]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ABD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[eE]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[b-d]e',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_character_class_modifier_i_3() {
        $test1 = array( 'str'=>'AAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[b-d]',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_modifier_i_4() {
        $test1 = array( 'str'=>'A-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[-b]',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_modifier_i_5() {
        $test1 = array( 'str'=>'A-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[b-]',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_square_closing_bracket_outside_character_class_modifier_i() {
        $test1 = array( 'str'=>'A]',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a]',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_square_closing_bracket_without_backslash_at_beginning_of_the_character_class_modifier_i() {
        $test1 = array( 'str'=>'A]B',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[]]b',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_modifier_i_3() {
        $test1 = array( 'str'=>'AED',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^bc]d',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_modifier_i_4() {
        $test1 = array( 'str'=>'ADC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'[cC]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ABD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[^-bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'A-C',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[^-bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^-b]c',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_square_closing_bracket_without_backslash_at_beginning_of_the_negative_character_class_modifier_i() {
        $test1 = array( 'str'=>'ADC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[^]b]c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab|cd',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_empty_subpattern_modifier_i() {
        $test1 = array( 'str'=>'DEF',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>2,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'()ef',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_after_end_of_line_modifier_i() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'A]C',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'B',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'$b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_round_opening_bracket_modifier_i() {
        $test1 = array( 'str'=>'A(B',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\(b',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_round_opening_bracket_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'A((B',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\(*b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_backslash_modifier_i() { //по неизвестной мне причине вызывает PHP Fatal Error: Вызов неопределенного метода qtype_preg_node_error::match()
        $test1 = array( 'str'=>'A\\B',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'\\',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\\\\b',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_nested_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>1,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((a))',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>2),
                        'length'=>array(0=>3,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)b(c)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_modifier_i() {
        $test1 = array( 'str'=>'AABBABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b+c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_modifier_i_2() {
        $test1 = array( 'str'=>'AABBABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a{1,}b{1,}c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_lazy_quantifier_modifier_i_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ABCABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.+?c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_modifier_i_3() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ABCABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.*?c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_modifier_i_4() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ABCABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a.{0,5}?c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b)*',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){0,}',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_modifier_i_3() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b)+',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_modifier_i_4() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){1,}',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_modifier_i_5() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b)?',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_in_quantifier_modifier_i_6() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){0,1}',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

/*  function data_for_test_quantifier_in_alternative_in_subpattern_in_lazy_quantifier_modifier_i() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+|b){0,1}?',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_negative_character_class_in_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'CDE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^ab]*',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_subpattern_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'ABBBCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([abc])*d',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_subpattern_in_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([abc])*bcd',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_modifier_i() {
        $test1 = array( 'str'=>'E',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a|b|c|d|e',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'EF',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|b|c|d|e)f',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_modifier_i_4() {
        $test1 = array( 'str'=>'ABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abcd*efg',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_modifier_i_5() {
        $test1 = array( 'str'=>'XABYABBBZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'XAYABBBZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab*',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_alternative_in_subpattern_modifier_i_3() {
        $test1 = array( 'str'=>'ABCDE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab|cd)e',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_modifier_i_6() {
        $test1 = array( 'str'=>'HIJ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[abhgefdc]ij',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_after_begin_of_line_modifier_i() {
        $test1 = array( 'str'=>'ABCDE',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'[eE]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(ab|cd)e',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_without_second_branch_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ABCDEF',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>2,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc|)ef',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_and_quantifier_modifier_i() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|b)c*d',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_alternative_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab|ab*)bc',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_character_class_in_subpattern_and_quantifier_modifier_i() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]*)c*',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>3),
                        'length'=>array(0=>4,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]*)(c*d)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier_in_subpattern_modifier_i_2() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>3),
                        'length'=>array(0=>4,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]+)(c*d)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_quantifier_in_subpattern_modifier_i_3() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>2),
                        'length'=>array(0=>4,1=>1,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a([bc]*)(c+d)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'ADCDCDE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[bcd]*dcdcde',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_and_quantifier_modifier_i_2() {
        $test1 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab|a)b*c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_subpatterns_modifier_i_2() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>1,4=>3),
                        'length'=>array(0=>4,1=>3,2=>1,3=>1,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((a)(b)c)(d)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_classes_modifier_i() {
        $test1 = array( 'str'=>'ALPHA',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[a-zA-Z_][a-zA-Z0-9_]*',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_and_character_class_in_alternative_in_subpattern_and_alternative_modifier_i() {
        $test1 = array( 'str'=>'ABH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a(bc+|b[eh])g|.h$',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_and_end_of_line_and_subpattern_in_alternatives_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'EFFGZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'IJ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'REFFGZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[jkJK]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ADCDCDE',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[gG]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'EFFG',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'BCDD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(bc+d$|ef*g.|h?i(j|k))',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_many_nested_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'A',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>0,4=>0,5=>0,6=>0,7=>0,8=>0,9=>0,10=>0),
                        'length'=>array(0=>1,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((((((((((a))))))))))',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_many_nested_subpatterns_and_back_reference_modifier_i() {
        $test1 = array( 'str'=>'AA',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>0,4=>0,5=>0,6=>0,7=>0,8=>0,9=>0,10=>0),
                        'length'=>array(0=>2,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1,10=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((((((((((a))))))))))\10',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_many_nested_subpatterns_modifier_i_2() {
        $test1 = array( 'str'=>'A',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0,3=>0,4=>0,5=>0,6=>0,7=>0,8=>0,9=>0),
                        'length'=>array(0=>1,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,9=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(((((((((a)))))))))',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_many_unnumbered_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'A',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?:(?:(?:(?:(?:(?:(?:(?:(a))))))))))',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_subpattern_in_many_unnumbered_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?:(?:(?:(?:(?:(?:(?:(?:(a|b|c))))))))))',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_modifier_i_3() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'AA',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'UH-UH',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(22),
                        'next'=>'m',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'multiple words of text',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_modifier_i_4() {
        $test1 = array( 'str'=>'MULTIPLE WORDS, YEAH',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>14),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'multiple words',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_any_characters_in_quantifiers_modifier_i() {
        $test1 = array( 'str'=>'ABCDE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3),
                        'length'=>array(0=>5,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)c(.*)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_any_characters_in_quantifiers_modifier_i_2() {
        $test1 = array( 'str'=>'(A, B)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>4),
                        'length'=>array(0=>6,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\((.*), (.*)\)',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_modifier_i_5() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abcd',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(bc)d',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_hyphen_in_character_class_in_quantifier_modifier_i() {
        $test1 = array( 'str'=>'AC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[-]?c',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_in_quantifier_in_subpattern_and_back_reference_modifier_i() {
        $test1 = array( 'str'=>'ABCABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([a-c]*)\1',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_lookahead_assertion_and_any_character() {
        $test1 = array( 'str'=>'abad',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?!b).',
                     'tests'=>array($test1));
    }

    function data_for_test_positive_lookahead_assertion_and_any_character() {
        $test1 = array( 'str'=>'abad',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?=d).',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_positive_lookahead_assertion_and_any_character() {
        $test1 = array( 'str'=>'abad',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?=c|d).',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_unnumbered_subpattern_and_any_character_in_subpattern() {
        $test1 = array( 'str'=>'ace',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d)(.)',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern() {
        $test1 = array( 'str'=>'ace',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d)*(.)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_alternatives_in_unnumbered_subpattern_in_lazy_quantifier_and_any_character_in_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ace',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d)+?(.)',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern_2() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d)+(.)',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern_3() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){2}(.)',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern_4() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){4,5}(.)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_alternatives_in_unnumbered_subpattern_in_lazy_quantifier_and_any_character_in_subpattern_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>6,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){4,5}?(.)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_subpatterns_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>0,3=>3),
                        'length'=>array(0=>6,1=>3,2=>3,3=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((foo)|(bar))*',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern_5() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){6,7}(.)',
                     'tests'=>array($test1));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_alternatives_in_unnumbered_subpattern_in_lazy_quantifier_and_any_character_in_subpattern_3() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){6,7}?(.)',
                     'tests'=>array($test1));
    }

    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern_6() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){5,6}(.)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_alternatives_in_unnumbered_subpattern_in_lazy_quantifier_and_any_character_in_subpattern_4() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){5,6}?(.)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_alternatives_in_unnumbered_subpattern_in_greedy_quantifier_and_any_character_in_subpattern_7() {
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){5,7}(.)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_alternatives_in_unnumbered_subpattern_in_lazy_quantifier_and_any_character_in_subpattern_5() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'acdbcdbe',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|c|d){5,7}?(.)',
                     'tests'=>array($test1));
    }
*//* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_alternative_in_subpattern_in_lazy_quantifier_in_alternatives_in_subpattern_in_lazy_quantifier_and_any_character_in_subpattern() {
        $test1 = array( 'str'=>'ace',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>2),
                        'length'=>array(0=>3,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?:b|(c|e){1,2}?|d)+?(.)',
                     'tests'=>array($test1));
    }

    function data_for_test_any_character_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(.+)?B',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_in_subpattern_in_alternative() {
        $test1 = array( 'str'=>'.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^([^a-z])|(\^)$',
                     'tests'=>array($test1));
    }

    function data_for_test_begin_of_line_and_character_class() {
        $test1 = array( 'str'=>'<&OUT',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[<>]&',
                     'tests'=>array($test1));
    }

    function data_for_test_back_reference_in_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'aaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'AB',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>7,1=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a\1?){4}$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_back_reference_in_conditional_subpattern_in_quantifier() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(9),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>9,1=>3),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a(?(1)\1)){4}$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_subpatterns_in_alternative_in_unnumbered_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5),
                        'length'=>array(0=>6,1=>1,2=>1,3=>1,4=>1,5=>1,6=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(f)(o)(o)|(b)(a)(r))*',
                     'tests'=>array($test1));
    }

    function data_for_test_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'cb',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'b',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a)b',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_negative_lookbehind_assertion_3() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<!c)b',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_characters_in_unnumbered_subpattern_in_greedy_quantifier() {
        $test1 = array( 'str'=>'aba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:..)*a',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_characters_in_unnumbered_subpattern_in_lazy_quantifier() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:..)*?a',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_any_character_in_subpattern_in_positive_lookahead_assertion_in_alternative_in_unnumbered_subpattern_in_quantifier_and_back_reference() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:b|a(?=(.)))*\1',
                     'tests'=>array($test1));
    }

    function data_for_test_empty_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(){3,5}',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aax',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a+)*ax',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aax',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>3,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^((a|b)+)*ax',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_in_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'aax',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>3,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^((a|bc)+)*ax',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'cab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|x)*ab',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_quantifier_11() {
        $test1 = array( 'str'=>'cab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)*ab',
                     'tests'=>array($test1));
    }

    function data_for_test_unnumbered_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'cb',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'[aA]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?i)a)b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_subpattern_modifier_i_2() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?i)a)b',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_unnumbered_subpattern_modifier_i_2() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?i:a)b',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_unnumbered_subpattern_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?i:a))b',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_unnumbered_subpattern_modifier_not_i_modifier_i() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Ab',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'AB',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?-i)a)b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_subpattern_modifier_not_i_modifier_i() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?-i)a)b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_unnumbered_subpattern_modifier_not_i_modifier_i_2() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'AB',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'Ab',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?-i:a)b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_unnumbered_subpattern_modifier_not_i_in_subpattern_modifier_i() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?-i:a))b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_unnumbered_subpattern_modifier_not_i_in_subpattern_modifier_i_2() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>2,1=>2),
                        'left'=>array(1),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'AB',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"a\nB",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'.',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?-i:a.))b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

/*  function data_for_test_any_character_in_unnumbered_subpattern_modifier_s_not_i_in_subpattern_modifier_i() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"a\nB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?s-i:a.))b',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_many_unnumbered_subpatterns_and_empty_unnumbered_subpatterns() {
        $test1 = array( 'str'=>'cabbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:c|d)(?:)(?:a(?:)(?:b)(?:b(?:))(?:b(?:)(?:b)))',
                     'tests'=>array($test1));
    }

    function data_for_test_many_unnumbered_subpatterns_and_empty_unnumbered_subpatterns_2() {
        $test1 = array( 'str'=>'caaaaaaaabbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>41),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:c|d)(?:)(?:aaaaaaaa(?:)(?:bbbbbbbb)(?:bbbbbbbb(?:))(?:bbbbbbbb(?:)(?:bbbbbbbb)))',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_and_digit_and_back_reference_modifier_i() {
        $test1 = array( 'str'=>'Ab4ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab4Ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(ab)\d\1',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_word_character_and_digit_in_quantifiers() {
        $test1 = array( 'str'=>'foobar1234baz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>13),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'foo\w*\d{4}baz',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_quantifier_and_nested_unnumbered_subpatterns_in_quantifiers() {
        $test1 = array( 'str'=>'x~~',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'x(~~)*(?:(?:F)?)?',
                     'tests'=>array($test1));
    }

    function data_for_test_comment_and_quantifier() {
        $test1 = array( 'str'=>'aaac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a(?#xxx){3}c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_comments_and_quantifier_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'aaac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a (?#xxx) (?#yyy) {3}c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_character_class_in_negative_lookbehind_assertion() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"B\nB",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'dbcb',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<![cd])b',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_in_negative_lookbehind_assertion_and_character_class() {
        $test1 = array( 'str'=>'dbaacb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<![cd])[ab]',
                     'tests'=>array($test1));
    }

    function data_for_test_alternative_in_subpattern_in_negative_lookbehind_assertion_and_character_class() {
        $test1 = array( 'str'=>'dbaacb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<!(c|d))[ab]',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_lookbehind_assertion_and_character_class() {
        $test1 = array( 'str'=>'cdaccb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<!cd)[ab]',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifiers_in_unnumbered_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'dbcb',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'a--',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'aa--',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a?b?)*$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

/*  function data_for_test_any_characters_in_nested_subpatterns_modifier_s_and_subpattern_modifier_m() { //модификаторы s и m не поддерживаются
        $test1 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2),
                        'length'=>array(0=>3,1=>2,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?s)^a(.))((?m)^b$)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?m)^b$)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_modifier_m_2() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nb\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?m)^b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_modifier_m_2() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nb\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?m)^(b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_modifier_m_3() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nb\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?m)^b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_modifier_m_4() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nb\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>2),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\n((?m)^b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_characters_in_subpattern_modifier_s_and_negative_lookahead_assertion() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>3),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?s).)c(?!.)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_characters_in_subpattern_modifier_s_and_negative_lookahead_assertion_2() { //модификатор s не поддерживается
        $test1 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?s)b.)c(?!.)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_begin_of_line_after_empty_subpattern() {
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'()^b',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_subpattern_modifier_m_5() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>2),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?m)^b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_quantifier_and_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(x)?(?(1)a|b)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_subpattern_in_quantifier_and_conditional_subpattern_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(x)?(?(1)b|a)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_empty_subpattern_in_quantifier_and_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'()?(?(1)b|a)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_empty_subpattern_in_quantifier_and_conditional_subpattern_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'()?(?(1)a|b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_round_brackets_in_subpatterns_in_quantifier_and_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'(blah)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>5),
                        'length'=>array(0=>6,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'blah)',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'(blah',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>')',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\()?blah(?(1)(\)))$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_round_brackets_in_subpatterns_in_quantifier_and_conditional_subpattern_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'(blah)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>5),
                        'length'=>array(0=>6,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'blah',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'blah)',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'(blah',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>')',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\(+)?blah(?(1)(\)))$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_negative_lookahead_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?!a)b|a)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_positive_lookahead_assertion_in_conditional_subpattern_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=a)b|a)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_positive_lookahead_assertion_in_conditional_subpattern_3() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=a)a|b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_lazy_quantifier_in_subpattern_in_positive_lookahead_assertion_and_back_reference_in_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1,2=>1),
                        'length'=>array(0=>3,1=>1,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=(a+?))(\1ab)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_word_character_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'one:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\w+:)+',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_positive_lookbehind_assertion_after_end_of_line() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'$(?<=^(a))',
                     'tests'=>array($test1));
    }

/*  function data_for_test_lazy_quantifier_in_subpattern_in_positive_lookahead_assertion_and_back_reference() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=(a+?))\1ab',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_negative_character_class_in_quantifier_and_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'aexycd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[^bcd]*(c+)',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_subpattern_and_quantifier() {
        $test1 = array( 'str'=>'caab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>3,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a*)b+',
                     'tests'=>array($test1));
    }

    function data_for_test_word_character_in_character_class_in_quantifier_in_subpattern_in_quantifier_and_word_character_in_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>4,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xy:z:::abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7),
                        'length'=>array(0=>11,1=>7,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,2=>4),
                        'length'=>array(0=>7,2=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcd:',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>4,2=>4),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([\w:]+::)?(\w+)$',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_quantifier_in_atomic_group_2() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a+)b',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_square_opening_bracket_in_character_class_in_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'a:[b]:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([[:]+)',
                     'tests'=>array($test1));
    }

    function data_for_test_square_opening_bracket_in_character_class_in_quantifier_in_subpattern_2() {
        $test1 = array( 'str'=>'a=[b]=',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([[=]+)',
                     'tests'=>array($test1));
    }

    function data_for_test_square_opening_bracket_in_character_class_in_quantifier_in_subpattern_3() {
        $test1 = array( 'str'=>'a.[b].',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([[.]+)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_quantifier_in_atomic_group_in_subpattern_3() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>a+)b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_subpattern_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(a+))b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_negative_character_classes_in_quantifiers_in_atomic_group_in_alternative_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'((abc(ade)ufh()()x',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>17),
                        'length'=>array(0=>16,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>[^()]+)|\([^()]*\))+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_end_of_string() { //\Z не поддерживается
        $test1 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"a\nb\n",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\Z',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_end_of_string_2() { //\Z не поддерживается
        $test1 = array( 'str'=>"a\nb\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'b\Z',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_end_of_string_3() {
        $test1 = array( 'str'=>"a\nb",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'b\z',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_empty_subpattern_in_conditional_subpattern_and_character_classes_in_quantifiers_in_atomic_groups_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a-b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'0-9',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'a.b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'5.6.7',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'the.quick.brown.fox',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>19,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'a100.b200.300c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>14,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'12-ab.1245',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>10,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'[^\W_]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'[^\W_]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>'.a',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'[^\W_]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'-a',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'left'=>array(1),
                        'next'=>'[^\W_]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test14 = array( 'str'=>'a-',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test15 = array( 'str'=>'a.',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test16 = array( 'str'=>'a_b',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test17 = array( 'str'=>'a.-',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test18 = array( 'str'=>'a..',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test19 = array( 'str'=>'ab..bc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test20 = array( 'str'=>'the.quick.brown.fox-',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>19,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test21 = array( 'str'=>'the.quick.brown.fox.',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>19,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test22 = array( 'str'=>'the.quick.brown.fox_',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>19,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test23 = array( 'str'=>'the.quick.brown.fox+',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>19,1=>0),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?>(?(1)\.|())[^\W_](?>[a-z0-9-]*[^\W_])?)+$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13, $test14, $test15, $test16, $test17, $test18, $test19, $test20, $test21, $test22, $test23));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_atomic_group_and_alternative_in_subpattern_in_positive_lookbehind_assertion() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'alphabetabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>8),
                        'length'=>array(0=>12,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'endingwxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>10,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(11),
                        'next'=>'l',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a rather long string that doesn\'t end with one of them',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(11),
                        'next'=>'l',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*)(?<=(abcd|wxyz))',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_negative_lookahead_assertion_and_character_class_in_quantifier_in_unnumbered_subpattern_in_quantifier_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'word cat dog elephant mussel cow horse canary baboon snake shark otherword',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>74),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'word cat dog elephant mussel cow horse canary baboon snake shark',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>64),
                        'left'=>array(10),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'word (?>(?:(?!otherword)[a-zA-Z0-9]+ ){0,30})otherword',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_character_class_in_quantifier_in_atomic_group_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'word cat dog elephant mussel cow horse canary baboon snake shark the quick brown fox and the lazy dog and several other words getting close to thirty by now I hope',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>163),
                        'left'=>array(10),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'word (?>[a-zA-Z0-9]+ ){0,30}otherword',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_digit_in_quantifier_and_negative_lookahead_assertion_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'999foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'123999foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'123abcfoo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=\d{3}(?!999))foo',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_any_characters_in_negative_lookahead_assertion_and_digit_in_quantifier_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'999foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'123999foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'123abcfoo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(?!...999)\d{3})foo',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_digit_in_quantifier_and_negative_lookahead_assertion_and_any_characters_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'123abcfoo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'123456foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(9),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'123999foo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>0),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=\d{3}(?!999)...)foo',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_digit_in_quantifier_and_any_characters_in_positive_lookbehind_assertion_and_negative_lookbehind_assertion() {
        $test1 = array( 'str'=>'123abcfoo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'123456foo',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(9),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'123999foo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>7),
                        'length'=>array(0=>0),
                        'left'=>array(3),
                        'next'=>'f',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=\d{3}...)(?<!999)foo',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

/*  function data_for_test_html_reference_modifiers_isx() { //модификаторы x и s не поддерживаются
        $test1 = array( 'str'=>'<a href=abcd xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>8),
                        'length'=>array(0=>12,3=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"<a href=\"abcd xyz pqr\" cats",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>21,2=>9),
                        'length'=>array(0=>22,1=>1,2=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'<a href=\'abcd xyz pqr\' cats',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>21,2=>9),
                        'length'=>array(0=>22,1=>1,2=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'<a[\s]+href[\s]*=[\s]*          # find <a href=\n([\"\'])?                       # find single or double quote\n(?(1) (.*?)\1 | ([^\s]+))       # if quote found, match up to next matching\n                                 # quote, otherwise match up to next space\n',
                     'modifiers'=>'isx',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_html_reference_modifiers_isx_2() { //модификаторы x и s не поддерживаются
        $test1 = array( 'str'=>'<a href=abcd xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>8),
                        'length'=>array(0=>12,3=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"<a href=\"abcd xyz pqr\" cats",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>21,2=>9),
                        'length'=>array(0=>22,1=>1,2=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'<a href       =       \'abcd xyz pqr\' cats',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>35,2=>23),
                        'length'=>array(0=>36,1=>1,2=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'<a\s+href\s*=\s*                # find <a href=\n (["\'])?                         # find single or double quote\n (?(1) (.*?)\1 | (\S+))          # if quote found, match up to next matching\n                                 # quote, otherwise match up to next space\n',
                     'modifiers'=>'isx',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_html_reference_modifiers_isx_3() { //модификаторы x и s не поддерживаются
        $test1 = array( 'str'=>'<a href=abcd xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>8),
                        'length'=>array(0=>12,3=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"<a href=\"abcd xyz pqr\" cats",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>21,2=>9),
                        'length'=>array(0=>22,1=>1,2=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'<a href       =       \'abcd xyz pqr\' cats',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>35,2=>23),
                        'length'=>array(0=>36,1=>1,2=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'<a\s+href(?>\s*)=(?>\s*)        # find <a href=\n(["\'])?                         # find single or double quote\n(?(1) (.*?)\1 | (\S+))          # if quote found, match up to next matching\n                                 # quote, otherwise match up to next space\n',
                     'modifiers'=>'isx',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_subpattern_in_quantifier_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'ZABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>0),
                        'length'=>array(0=>2,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((Z)+|A)*',
                     'tests'=>array($test1));
    }

    function data_for_test_empty_subpattern_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'ZABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>1),
                        'length'=>array(0=>2,1=>1,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(Z()|A)*',
                     'tests'=>array($test1));
    }

    function data_for_test_empty_subpattern_in_subpattern_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'ZABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>1,3=>1),
                        'length'=>array(0=>2,1=>1,2=>0,3=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(Z(())|A)*',
                     'tests'=>array($test1));
    }

/*  function data_for_test_atomic_group_in_quantifier_in_alternative_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ZABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>Z)+|A)*',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_empty_atomic_group_in_quantifier_in_alternative_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ZABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>)+|A)*',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_hypen_in_character_class_without_backslash() {
        $test1 = array( 'str'=>'abcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'-things',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'0digit',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a-\d]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bcdef',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a-\d]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a-\d]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_hypen_in_character_class_without_backslash_2() {
        $test1 = array( 'str'=>'abcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'-things',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'0digit',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\d-a]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'bcdef',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\d-a]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\d-a]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_() {
        $test1 = array( 'str'=>"> \x09\x0a\x0c\x0d\x0b<",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[[:space:]]+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"> \x09\x0a\x0c\x0d\x0b<",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[[:blank:]]+',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_space_character_in_character_class_in_quantifier() {
        $test1 = array( 'str'=>"> \x09\x0a\x0c\x0d\x0b<",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\s]+',
                     'tests'=>array($test1));
    }

    function data_for_test_space_character_in_quantifier() {
        $test1 = array( 'str'=>"> \x09\x0a\x0c\x0d\x0b<",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\s+',
                     'tests'=>array($test1));
    }

/*  function data_for_test_modifier_x_2() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'\n',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a\nb',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_string_in_negative_lookahead_assertion_modifier_m_2() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nxb\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?!\A)x',
                     'modifiers'=>'m',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_line_in_negative_lookahead_assertion_modifier_m() { //модификатор m не поддерживается
        $test1 = array( 'str'=>"a\nxb\n",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?!^)x',
                     'modifiers'=>'m',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\Qabc\Eabc',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc(*+|abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\Q(*+|\Eabc',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(9),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcabcabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(7),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'   abc\Q abc\Eabc',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc#not comment\n    literal",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc#comment\n\Q#not comment\nliteral\E',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc#not comment\n    literal",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc#comment\n\Q#not comment\nliteral',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc#not comment\n    literal",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc#comment\n\Q#not comment\nliteral\E #more comment\n    ',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc#not comment\n    literal",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>27),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc#comment\n\Q#not comment\nliteral\E #more comment',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc\\\$xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Qabc\$xyz\E',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc\$xyz",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Qabc\E\$\Qxyz\E',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xyzabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Gabc',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_unnumbered_subpattern_modiier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'XabcdY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Xa b c d Y',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(?x: b c )d',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_alternative_in_subpattern_modiier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'XabcY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'AxyzB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?x)x y z | a b c)',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_modiier_i_and_not_i() {
        $test1 = array( 'str'=>'XabCY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'XabcY',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'left'=>array(1),
                        'next'=>'C',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?i)AB(?-i)C',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_modiier_i_and_not_i_in_alternative_in_subpattern() {
        $test1 = array( 'str'=>'abCE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'DE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'left'=>array(3),
                        'next'=>'[Bb]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcE',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(2),
                        'next'=>'C',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abCe',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(1),
                        'next'=>'E',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'dE',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'D',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'De',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'E',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?i)AB(?-i)C|D)E',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

    function data_for_test_any_character_in_quantifier_in_subpattern_and_digit_in_quantifier_and_back_reference() {
        $test1 = array( 'str'=>'abc123abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc123bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)\d+\1',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_character_in_quantifier_in_subpattern_and_digit_in_quantifier_and_back_reference_modifier_s() {
        $test1 = array( 'str'=>'abc123abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc123bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*)\d+\1',
                     'modifiers'=>'s',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_character_in_quantifier_in_nested_subpatterns_and_digit_in_quantifier_and_back_reference() {
        $test1 = array( 'str'=>'abc123abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>9,1=>3,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc123bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1,2=>1),
                        'length'=>array(0=>7,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((.*))\d+\1',
                     'tests'=>array($test1, $test2));
    }
/* This tests for an IPv6 address in the form where it can have up to eight components, one and only one of which is empty. This must be an internal component. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a123::a123',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>9),
                        'length'=>array(0=>10,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a123:b342::abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>14),
                        'length'=>array(0=>15,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a123:b342::324e:abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>19),
                        'length'=>array(0=>20,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a123:ddde:b342::324e:abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>24),
                        'length'=>array(0=>25,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'a123:ddde:b342::324e:dcba:abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>29),
                        'length'=>array(0=>30,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'a123:ddde:9999:b342::324e:dcba:abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'1:2:3:4:5:6:7:8',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test9 = array( 'str'=>'a123:bce:ddde:9999:b342::324e:dcba:abcd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test10 = array( 'str'=>'a123::9999:b342::324e:dcba:abcd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test11 = array( 'str'=>'abcde:2:3:4:5:6:7:8',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test12 = array( 'str'=>'::1',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test13 = array( 'str'=>'abcd:fee0:123::   ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test14 = array( 'str'=>':1',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test15 = array( 'str'=>'1:  ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>34),
                        'length'=>array(0=>35,1=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?!:)                       # colon disallowed at start\n(?:                         # start of item\n(?: [0-9a-f]{1,4} |       # 1-4 hex digits or\n(?(1)0 | () ) )           # if null previously matched, fail; else null\n:                         # followed by colon\n){1,7}                      # end item; 1-7 of them required\n[0-9a-f]{1,4} $             # final hex number at end of string\n(?(1)|.)                    # check that there was an empty component\n',
                     'modifiers'=>'xi',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8, $test9, $test10, $test11, $test12, $test13, $test14, $test15));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'d',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>']',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'b',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[za\-d\]]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[z\Qa-d]\E]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }
*/
    function data_for_test_escape_sequences_in_character_class() {
        $test1 = array( 'str'=>'z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\z\C]',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_unknown_escape_sequence() {
        $test1 = array( 'str'=>'M',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\M',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>70,1=>70),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a+)*b',
                     'tests'=>array($test1));
    }

    function data_for_test_character_class_and_alternatives_and_unnumbered_subpatterns_modifier_i() {
        $test1 = array( 'str'=>'REGular',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'regulaer',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'Regex',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'regulдr',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?i)reg(?:ul(?:[aд]|ae)r|ex)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_character_class_in_quantifier_ru() {
        $test1 = array( 'str'=>'Ежеда',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Ежедя',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ЕжедА',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ЕжедЯ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'Ежед[а-яА-Я]+',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_positive_lookbehind_assertion_and_any_character() {
        $test1 = array( 'str'=>"\x84XAZXB",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=Z)X.',
                     'tests'=>array($test1));
    }

/*  function data_for_test_modifier_x_3() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'ab cd defg',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab cd (?x) de fg',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_modifier_x_4() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'ab cddefg',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'left'=>array(8),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abcddefg',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(7),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab cd(?x) de fg',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_negative_character_class_in_negative_lookbehind_assertion_and_subpattern() {
        $test1 = array( 'str'=>'foobarX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>3),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'boobarX',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<![^f]oo)(bar)',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_negative_lookbehind_assertion() {
        $test1 = array( 'str'=>'offX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'onyX',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<![^f])X',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_negative_character_class_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'onyX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'offX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=[^f])X',
                     'tests'=>array($test1, $test2, $test3));
    }

/*  function data_for_test_modifier_x_space_characters_in_quantifiers_in_unnumbered_subpattern_modifier_not_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'A # B',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'#',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'#',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?x)(?-x: \s*#\s*)',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_modifiers_x_not_is_and_space_characters_in_quantifiers_in_unnumbered_subpattern_modifiers_not_ixs() { //модификаторы i, x и s не поддерживаются
        $test1 = array( 'str'=>'A #include',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'left'=>array(8),
                        'next'=>'#',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'A#include',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(9),
                        'next'=>' ',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'A #Include',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'left'=>array(7),
                        'next'=>'i',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?x-is)(?:(?-ixs) \s*#\s*) include',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_quantifiers_and_word_character() {
        $test1 = array( 'str'=>'aaabbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*b*\w',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifiers_and_word_character_2() {
        $test1 = array( 'str'=>'aaabbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*b?\w',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifiers_and_word_character_3() {
        $test1 = array( 'str'=>'aaabbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*b{0,4}\w',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifiers_and_word_character_4() {
        $test1 = array( 'str'=>'aaabbbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*b{0,}\w',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifiers_and_word_character_5() {
        $test1 = array( 'str'=>'0a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*\d*\w',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_quantifiers_and_word_character_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*b *\w',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifiers_and_word_character_comment_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*b#comment\n*\w',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifiers_and_word_character_modifier_x_2() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a* b *\w',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_word_and_any_characters_in_quantifiers_and_backslash_and_end_of_string_character_and_any_character_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>"abc=xyz\\\npqr",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\w+=.*(\\\n.*)*',
                     'tests'=>array($test1));
    }

    function data_for_test_word_character_in_quantifier_in_subpattern_in_positive_lookahead_assertion_and_back_reference() {
        $test1 = array( 'str'=>'abcd:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=(\w+))\1:',
                     'tests'=>array($test1));
    }

    function data_for_test_word_character_in_quantifier_in_subpattern_in_positive_lookahead_assertion_and_back_reference_2() {
        $test1 = array( 'str'=>'abcd:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=(\w+))\1:',
                     'tests'=>array($test1));
    }

/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\Eabc',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\Eabc]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'E',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\Eabc]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\Eabc]',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a-\Ec]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a-\Ec]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'E',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a-\Ec]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a-\Ec]',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a\E\E-\Ec]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a\E\E-\Ec]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'E',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[a\E\E-\Ec]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a\E\E-\Ec]',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\E\Qa\E-\Qz\E]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\E\Qa\E-\Qz\E]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\E\Qa\E-\Qz\E]+',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>']',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a\Q]bc\E]',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'-',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[a-\Q\E]',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_empty_subpattern_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>3),
                        'length'=>array(0=>4,1=>1,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a()*)*',
                     'tests'=>array($test1));
    }

    function data_for_test_nested_unnumbered_subpatterns_in_quantifier_in_unnumbered_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a(?:(?:))*)*',
                     'tests'=>array($test1));
    }

    function data_for_test_empty_subpattern_in_quantifier_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>4),
                        'length'=>array(0=>4,1=>1,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a()+)+',
                     'tests'=>array($test1));
    }

    function data_for_test_nested_unnumbered_subpatterns_in_quantifier_in_unnumbered_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a(?:(?:))+)+',
                     'tests'=>array($test1));
    }

/*  function data_for_test_subpattern_in_quantifier_and_alternative_without_second_branch_in_subpattern_in_conditional_subpattern_in_quantifier() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abbD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ccccD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>4),
                        'length'=>array(0=>5,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>1,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a){0,3}(?(1)b|(c|))*D',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_alternative_without_second_branch_in_subpattern_in_quantifier_and_digit() {
        $test1 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>59),
                        'length'=>array(0=>60,1=>1),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa4',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>60),
                        'length'=>array(0=>61,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|)*\d',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_alternative_without_second_branch_in_atomic_group_in_quantifier_and_digit() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>60),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa4',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>61),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a|)*\d',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_alternative_without_second_branch_in_unnumbered_subpattern_in_quantifier_and_digit() {
        $test1 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>60),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa4',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>61),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:a|)*\d',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_any_character_in_quantifier_in_atomic_group_and_end_of_line_character_in_negative_lookbehind_assertion_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\n",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?s)(?>.*)(?<!\n)',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_end_of_line_character_in_negative_character_class_in_quantifier_and_end_of_line_character_and_end_of_string_in_negative_lookahead_assertion() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\n",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?![^\n]*\n\z)',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_end_of_string_and_end_of_line_in_negative_lookbehind_assertion() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"abc\n",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\z(?<!\n)',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_character_in_subpattern_in_quantifier_and_any_character_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>4,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(.*(.)?)*',
                     'tests'=>array($test1));
    }

/*  function data_for_test_conditional_subpattern_in_alternative_in_subpattern_in_quantifier_in_subpattern_modifier_x() { //модификатор x и условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>0,1=>0,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'( (A | (?(1)0|) )*   )',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_conditional_subpattern_in_subpattern_in_quantifier_in_subpattern_modifier_x() { //модификатор x и условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>0,1=>0,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'( ( (?(1)0|) )*   )',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_conditional_subpattern_in_quantifier_in_subpattern_modifier_x() { //модификатор x и условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>0,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(  (?(1)0|)*   )',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_square_opening_bracket_in_character_class() {
        $test1 = array( 'str'=>'a]',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>':]',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[[:abcd:xyz]]',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_square_opening_and_closing_brackets_in_character_class() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'[',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>':',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>']',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'p',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[abc[:x\]pqr]',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_any_character_in_quantifier_and_character_classes() {
        $test1 = array( 'str'=>'fooabcfoo',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>9),
                        'left'=>array(1),
                        'next'=>'[xyz]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*[op][xyz]',
                     'tests'=>array($test1));
    }

/*  function data_for_test_any_character_in_quantifier_in_positive_lookahead_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'adc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=.*b)b|^)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_positive_lookahead_assertion_in_conditional_subpattern_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'adc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=^.*b)b|^)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_positive_lookahead_assertion_in_conditional_subpattern_in_quantifier() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'adc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=.*b)b|^)*',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_positive_lookahead_assertion_in_conditional_subpattern_in_quantifier_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'adc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=.*b)b|^)+',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_positive_lookahead_assertion_and_any_character_in_quantifier_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=b).*b|^d)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_positive_lookahead_assertion_and_any_character_in_quantifier_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=.*b).*b|^d)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_character_class_in_positive_lookahead_assertion_and_negative_character_class_in_conditional_subpattern_in_alternative_in_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'%ab%',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>4,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^%((?(?=[a])[^%])|b)*%$',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_modifiers_i_and_not_i_in_alternative() {
        $test1 = array( 'str'=>'XabX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'XAbX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'CcC ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'XABX   ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?i)a(?-i)b|c',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_hex_codes_of_characters_and_space_character_in_character_class_in_quantifier() {
        $test1 = array( 'str'=>"\x0a\x0b\x0c\x0d",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[\x00-\xff\s]+',
                     'tests'=>array($test1));
    }

    function data_for_test_control_escape_sequence() {
        $test1 = array( 'str'=>'?',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\c',
                     'tests'=>array($test1));
    }

    function data_for_test_back_reference_modifier_i_2() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'[aA]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(abc)\1',
                     'modifiers'=>'i',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_in_quantifier_modifier_i_3() {
        $test1 = array( 'str'=>'12abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]*',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый захватывающий квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_negative_character_class_in_possessive_quantifier_modifier_i() {
        $test1 = array( 'str'=>'12abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12ABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]*+',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_negative_character_class_in_lazy_quantifier_modifier_i() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'[xX]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'[xX]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12ABC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'[xX]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]*?X',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_negative_character_class_in_lazy_quantifier_modifier_i_2() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[xX]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[xX]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12ABC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'[xX]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]+?X',
                     'modifiers'=>'i   ',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_negative_character_class_in_quantifier_modifier_i_4() {
        $test1 = array( 'str'=>'12aXbcX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12AXBCX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'BCX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]?X',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

/*  function data_for_test_negative_character_class_in_lazy_quantifier_modifier_i_3() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'12aXbcX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12AXBCX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'BCX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]??X',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_negative_character_class_in_possessive_quantifier_modifier_i_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'12aXbcX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'12AXBCX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'BCX ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]?+X',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_negative_character_class_in_quantifier_modifier_i_5() {
        $test1 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABCDEF',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]{2,3}',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый ленивый квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_negative_character_class_in_lazy_quantifier_modifier_i_4() {
        $test1 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABCDEF',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]{2,3}?',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }
/* следующая функция не закомментирована, несмотря на неподдерживаемый захватывающий квантификатор, т.к. на жадных такой же результат (а ленивые распознаются, но матчатся как жадные) */
    function data_for_test_negative_character_class_in_possessive_quantifier_modifier_i_3() {
        $test1 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABCDEF',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[^a]{2,3}+',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_alternative_without_second_branch_in_subpattern_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'Z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>1,1=>0,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((a|)+)+Z',
                     'tests'=>array($test1));
    }

    function data_for_test_subpatterns_in_alternative() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)b|(a)c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_subpattern_in_atomic_group_and_subpattern_in_alternative() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(a))b|(a)c',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_subpattern_in_positive_lookahead_assertion_and_subpattern_in_alternative() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=(a))ab|(a)c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_subpattern_in_atomic_group_and_subpattern_in_alternative_in_subpattern() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,3=>0),
                        'length'=>array(0=>2,1=>2,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>(a))b|(a)c)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_atomic_group_and_subpattern_in_alternative_in_subpattern_in_possessive_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,3=>0),
                        'length'=>array(0=>2,1=>2,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>(a))b|(a)c)++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_atomic_group_and_subpattern_in_alternative_in_unnumbered_subpattern_in_possessive_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?>(a))b|(a)c)++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_atomic_group_and_subpattern_in_alternative_in_positive_lookahead_assertion_and_any_characters_in_subpattern() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0,3=>0),
                        'length'=>array(0=>2,2=>1,3=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=(?>(a))b|(a)c)(..)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_atomic_group_and_subpattern_in_alternative_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(?>(a))b|(a)c)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_subpattern_in_atomic_group_in_quantifier_and_subpattern_in_subpattern() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaabaaabaabab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>5,3=>9),
                        'length'=>array(0=>14,1=>14,2=>3,3=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>(a+)b)+(aabab))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_alternative_in_atomic_group_in_lazy_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a+|ab)+?c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_alternative_in_atomic_group_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a+|ab)+c',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_quantifier_in_alternative_in_unnumbered_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:a+|ab)+c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_subpattern_in_positive_lookahead_assertion_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=(a))a)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_positive_lookahead_assertion_in_conditional_subpattern_and_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=(a))a)(b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_alternative_in_unnumbered_subpattern_in_possessive_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a|ab)++c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_alternative_in_atomic_group_in_possessive_quantifier() { //однократные подмаски и захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'c',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?>a|ab)++c',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_alternative_in_unnumbered_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aaaabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a|ab)+c',
                     'tests'=>array($test1));
    }

    function data_for_test_positive_lookahead_assertion_in_null_quantifier() {
        $test1 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=abc){0}xyz',
                     'tests'=>array($test1));
    }

    function data_for_test_positive_lookahead_assertion_in_quantifier() {
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xyz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=abc){1}xyz',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_subpattern_in_positive_lookahead_assertion_in_quantifier_and_any_character() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=(a))?.',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_subpattern_in_positive_lookahead_assertion_in_lazy_quantifier_and_any_character() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=(a))??.',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_subroutine_reference_in_positive_lookahead_assertion_in_quantifier_and_character_class_and_character_class_in_subpattern() {
        $test1 = array( 'str'=>'abd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'zcdxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=(?1))?[az]([abc])d',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_negative_lookahead_assertion_in_null_quantifier_and_word_character_in_quantifier() {
        $test1 = array( 'str'=>'aaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?!a){0}\w+',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_positive_lookbehind_assertion_in_quantifier() {
        $test1 = array( 'str'=>'abcxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'pqrxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(abc))?xyz',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_character_class_in_quantifier_8() {
        $test1 = array( 'str'=>'ggg<<<aaa>>>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>12),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\g<a>]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'\\ga',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[\g<a>]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\g<a>]+',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_character_class_in_quantifier_9() {
        $test1 = array( 'str'=>'gggagagaxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[\ga]+',
                     'tests'=>array($test1));
    }

/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaa444:::Z',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[:a[:digit:]]+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaa444:::bbbZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>13),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^[:a[:digit:]:b]+',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_character_classes_2() {
        $test1 = array( 'str'=>' :xxx:',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'[:a]xxx[b:]',
                     'tests'=>array($test1));
    }

    function data_for_test_quantifier_in_positive_lookbehind_assertion_modifier_i() {
        $test1 = array( 'str'=>'xaabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'[aA]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xabc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'[aA]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a{2})b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_quantifier_in_negative_lookbehind_assertion_modifier_i() {
        $test1 = array( 'str'=>'xabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xaabc',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'[bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<!a{2})b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_horizontal_space_character_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'xa c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a\h)c',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_in_quantifier_and_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'axxbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aAAbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xaabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=[^a]{2})b',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_negative_character_class_in_quantifier_and_positive_lookbehind_assertion_modifier_i() {
        $test1 = array( 'str'=>'axxbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aAAbc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xaabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>0),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=[^a]{2})b',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_not_horizontal_space_character_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a\H)c',
                     'tests'=>array($test1));
    }

    function data_for_test_not_vertical_space_character_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a\V)c',
                     'tests'=>array($test1));
    }

    function data_for_test_vertical_space_character_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>"a\nc",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a\v)c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_positive_lookahead_assertion_in_conditional_subpattern_in_possessive_quantifier() { //условные подмаски и захватывающие квантификаторы не поддерживаются
        $test1 = array( 'str'=>'XcccddYX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=c)c|d)++Y',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_positive_lookahead_assertion_in_conditional_subpattern_in_possessive_quantifier_2() { //условные подмаски и захватывающие квантификаторы не поддерживаются
        $test1 = array( 'str'=>'XcccddYX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=c)c|d)*+Y',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_in_subpattern_in_possessive_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>7,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(6),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>6,1=>3),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>9,1=>3),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a{2,3}){2,}+a',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_quantifier_in_subpattern_in_possessive_quantifier_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>6,1=>3),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a{2,3})++a',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_quantifier_in_subpattern_in_possessive_quantifier_3() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>6,1=>3),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a{2,3})*+a',
                     'tests'=>array($test1, $test2));
    }
*/
    function data_for_test_one_data_unit_utf() {
        $test1 = array( 'str'=>'abXde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab\Cde',
                     'tests'=>array($test1));
    }

    function data_for_test_one_data_unit_utf_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'abZdeX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=ab\Cde)X',
                     'tests'=>array($test1));
    }

    function data_for_test_one_data_unit_utf_in_character_class() {
        $test1 = array( 'str'=>'aCb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aDb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[\CD]b',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_one_data_unit_utf_in_character_class_2() {
        $test1 = array( 'str'=>'aJb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a[\C-X]b',
                     'tests'=>array($test1));
    }
/* на следующих 2 тестовых функциях выбрасывается исключение: Uninitialized string offset: 1
    function data_for_test_horizontal_and_vertical_space_characters() {
        $test1 = array( 'str'=>"X X\x0a",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"X\x09X\x0b",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'\v',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"\xa0 X\x0a",
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(4),
                        'next'=>'\H',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\H\h\V\v',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }

    function data_for_test_horizontal_and_vertical_space_characters_in_quantifiers() {
        $test1 = array( 'str'=>"\x09\x20\xa0X\x0a\x0b\x0c\x0d\x0a",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"\x09\x20\xa0\x0a\x0b\x0c\x0d\x0a",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>"\x09\x20\xa0\x0a\x0b\x0c",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(3),
                        'next'=>'\v',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>"\x09\x20\xa0\x0a\x0b",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>'\v',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\H*\h+\V?\v{3,4}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
    function data_for_test_not_horizontal_space_character_in_quantifier() {
        $test1 = array( 'str'=>'XY  ABCDE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'XY  PQR ST',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\H{3,4}',
                     'tests'=>array($test1, $test2));
    }

    function data_for_test_any_characters_and_horizontal_space_character_in_quantifier() {
        $test1 = array( 'str'=>'XY  AB    PQRS',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.\h{3,4}.',
                     'tests'=>array($test1));
    }

    function data_for_test_horizontal_and_not_horizontal_space_characters_in_quantifiers() {
        $test1 = array( 'str'=>'>XNNNYZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'>  X NYQZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'left'=>array(4),
                        'next'=>'X',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'>XYZ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'left'=>array(2),
                        'next'=>'Y',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'>  X NY Z',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>6),
                        'left'=>array(1),
                        'next'=>'Z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\h*X\h?\H+Y\H?Z',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_vertical_and_not_vertical_space_characters_in_quantifiers() {
        $test1 = array( 'str'=>">XY\x0aZ\x0aA\x0bNN\x0c",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>">\x0a\x0dX\x0aY\x0a\x0bZZZ\x0aAAA\x0bNNN\x0c",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>19),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\v*X\v?Y\v+Z\V*\x0a\V+\x0b\V{2,3}\x0c',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_subpattern_and_reset_start() { //\K не поддерживается
        $test1 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(foo)\Kbar',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_and_reset_start_in_alternative_in_subpattern() { //\K не поддерживается
        $test1 = array( 'str'=>'foobar',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>0,2=>3),
                        'length'=>array(0=>3,1=>3,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'foobaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3),
                        'length'=>array(0=>6,1=>3,2=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(foo)(\Kbar|baz)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_reset_start_in_subpattern() { //\K не поддерживается
        $test1 = array( 'str'=>'foobarbaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>0),
                        'length'=>array(0=>6,1=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(foo\Kbar)baz',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_many_absolute_and_relative_back_references() { //\g не поддерживается
        $test1 = array( 'str'=>'ababababbbabZXXXX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>13,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a(b))\1\g1\g{1}\g-1\g{-1}\g{-02}Z',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_alternative_in_named_subpattern_and_named_back_reference() { //\g не поддерживается
        $test1 = array( 'str'=>'tom-tom',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'bon-bon',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<A>tom|bon)-\g{A}',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_relative_back_reference_in_alternative_in_nested_subpatterns() { //\g не поддерживается
        $test1 = array( 'str'=>'bacxxx',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(^(a|b\g{-1}))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_not_duplicate_subpattern_and_back_reference() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xyzxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(5),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcxyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'xyzabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?|(abc)|(xyz))\1',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_not_duplicate_subpattern_and_subroutine_reference() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xyzabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(5),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xyzxyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?|(abc)|(xyz))(?1)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_not_duplicate_subpattern_and_subroutine_reference_2() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'XYabcdY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2,2=>3,3=>4,4=>5,5=>6),
                        'length'=>array(0=>7,1=>1,2=>1,3=>1,4=>1,5=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^X(?5)(a)(?|(b)|(q))(c)(d)(Y)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpatterns_in_not_duplicate_subpattern_and_subroutine_reference() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'XYabcdY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2,2=>3,5=>4,6=>5,7=>6),
                        'length'=>array(0=>7,1=>1,2=>1,5=>1,6=>1,7=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^X(?7)(a)(?|(b|(r)(s))|(q))(c)(d)(Y)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_not_duplicate_subpattern_in_not_duplicate_subpattern_and_subroutine_reference() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'XYabcdY',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2,2=>3,5=>4,6=>5,7=>6),
                        'length'=>array(0=>7,1=>1,2=>1,5=>1,6=>1,7=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^X(?7)(a)(?|(b|(?|(r)|(t))(s))|(q))(c)(d)(Y)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_word_character_in_quantifier_in_named_subpattern_and_named_back_reference_in_quantifier() { //\k не поддерживается
        $test1 = array( 'str'=>'a:aaxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab:ababxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3,1=>3),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(3),
                        'next'=>':',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a:axyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ab:abxyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?\'abc\'\w+):\k<abc>{2}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_word_character_in_quantifier_in_named_subpattern_and_named_back_reference_in_quantifier_2() { //\g не поддерживается
        $test1 = array( 'str'=>'a:aaxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab:ababxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3,1=>3),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(3),
                        'next'=>':',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'a:axyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'left'=>array(1),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'ab:abxyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>2),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?\'abc\'\w+):\g{abc}{2}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_named_and_conditional_subpatterns_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'abd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ce',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?<ab>a)? (?(<ab>)b|c) (?('ab')d|e)',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_in_subpattern_and_relative_reference() { //\g не поддерживается
        $test1 = array( 'str'=>'aXaXZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a.)\g-1Z',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_subpattern_and_relative_reference_2() { //\g не поддерживается
        $test1 = array( 'str'=>'aXaXZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a.)\g{-1}Z',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?(DEFINE) (?<A> a) (?<B> b) )  (?&A) (?&B) ',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'metcalfe 33',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>9),
                        'length'=>array(0=>11,1=>8,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<NAME>(?&NAME_PAT))\s+(?<ADDR>(?&ADDRESS_PAT))\n(?(DEFINE)\n(?<NAME_PAT>[a-z]+)\n(?<ADDRESS_PAT>\d+)\n)',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'1.2.3.4',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>5),
                        'length'=>array(0=>7,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'131.111.10.206',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>10),
                        'length'=>array(0=>14,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'10.0.0.0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>6),
                        'length'=>array(0=>8,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,2=>6),
                        'length'=>array(0=>8,2=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'10.6',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,2=>6),
                        'length'=>array(0=>8,2=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'455.3.4.5',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,2=>6),
                        'length'=>array(0=>8,2=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(DEFINE)(?<byte>2[0-4]\d|25[0-5]|1\d\d|[1-9]?\d))\b(?&byte)(\.(?&byte)){3}',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'1.2.3.4',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>7,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'131.111.10.206',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>10),
                        'length'=>array(0=>14,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'10.0.0.0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>8,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>8,1=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'10.6',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>8,1=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'455.3.4.5',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>6),
                        'length'=>array(0=>8,1=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\b(?&byte)(\.(?&byte)){3}(?(DEFINE)(?<byte>2[0-4]\d|25[0-5]|1\d\d|[1-9]?\d))',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_word_and_space_characters_in_possessive_quantifiers_in_alternative_in_subpattern_in_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'now is the time for all good men to come to the aid of the party',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>59),
                        'length'=>array(0=>64,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'this is not a line with only words and spaces!',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>45),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\w++|\s++)*$',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_digit_in_possessive_quantifier_and_word_character_in_quantifiers() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'12345a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>5),
                        'length'=>array(0=>6,1=>5,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'12345+',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\d++)(\w)',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_possessive_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a++b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_possessive_quantifier_in_subpattern() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a++b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_possessive_quantifier_in_subpattern_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a++)b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_square_brackets_in_negative_character_class_in_possessive_quantifier_and_square_brackets_in_negative_character_class_in_quantifier_in_alternative_in_subpattern_in_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'((abc(ade)ufh()()x',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>17),
                        'length'=>array(0=>16,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'([^()]++|\([^()]*\))+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_square_brackets_in_negative_character_class_in_possessive_quantifier_and_square_brackets_in_negative_character_class_in_quantifier_in_alternative_in_subpattern_in_quantifier_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'(abc)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>5,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'(abc(def)xyz)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>9),
                        'length'=>array(0=>13,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'\(',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'((()aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(2),
                        'next'=>'[^()]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\(([^()]++|\([^()]+\))+\)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_square_brackets_in_negative_character_class_and_relative_reference_in_quantifier_in_alternative_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a(b)c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'a(b(c))d',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>8,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers)',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>11),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'a(b(c)d',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(1),
                        'next'=>')',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^([^()]|\((?1)*\))*$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_square_brackets_in_negative_character_class_and_relative_reference_in_quantifier_in_alternative_in_subpattern_in_quantifier_2() {
        $test1 = array( 'str'=>'>abc>123<xyz<',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>7),
                        'length'=>array(0=>13,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'>abc>1(2)3<xyz<',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>9),
                        'length'=>array(0=>15,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'>abc>(1(2)3)<xyz<',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>5),
                        'length'=>array(0=>17,1=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^>abc>([^()]|\((?1)*\))*<xyz<$',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_any_characters_in_subpatterns_and_relative_references_and_back_references_in_subpatterns_in_alternative_in_unnumbered_subpattern_modifier_i() {
        $test1 = array( 'str'=>'1221',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>4,1=>4,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Satanoscillatemymetallicsonatas',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>31,3=>31,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'AmanaplanacanalPanama',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>21,3=>21,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'AblewasIereIsawElba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>19,3=>19,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'Thequickbrownfox',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:((.)(?1)\2|)|((.)(?3)\4|.))$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_digit_in_quantifier_and_character_class_in_subpattern_and_relative_references_in_alternatives_in_subpattern() {
        $test1 = array( 'str'=>'12',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'(((2+2)*-3)-7)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>11),
                        'length'=>array(0=>14,1=>14,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'-12',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'((2+2)*-3)-7)',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'left'=>array(0),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(\d+|\((?1)([+*-])(?1)\)|-(?1))$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

    function data_for_test_relative_reference_in_quantifier_in_alternative_in_nested_subpatterns() {
        $test1 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>3,1=>3,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'xxyzxyzz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>8,1=>8,2=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'xxyzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(4),
                        'next'=>'x',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'xxyzxyzxyzz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(1),
                        'next'=>'z',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(x(y|(?1){2})z)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }

/*  function data_for_test_digit_and_character_class_in_possessive_quantifiers_in_alternative_in_conditional_subpattern_and_relative_reference_in_alternative_in_unnumbered_subpattern_in_quantifier_in_nested_subpatterns_modifier_x() { //модификатор x и захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'<>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>2,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'<abcd>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>6,1=>6,2=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'<abc <123> hij>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>15,1=>15,2=>15),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'<abc <def> hij>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5,1=>5,2=>5),
                        'length'=>array(0=>5,1=>5,2=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'<abc<>def>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>10,1=>10,2=>10),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'<abc<>',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4,1=>4,2=>4),
                        'length'=>array(0=>2,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'*** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'<',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'<abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>4,1=>4,2=>4),
                        'left'=>array(1),
                        'next'=>'>',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a+(*FAIL)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaabccc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b?c+(*FAIL)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaabccc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b?(*PRUNE)c+(*FAIL)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaabccc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b?(*COMMIT)c+(*FAIL)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaabcccaaabccc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b?(*SKIP)c+(*FAIL)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaxxxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaa++++++',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbbxxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bbb+++++',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'cccxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'ccc++++',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'dddddddd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:aaa(*THEN)\w{6}|bbb(*THEN)\w{5}|ccc(*THEN)\w{4}|\w{3})',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaxxxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>9,1=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaa++++++',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bbbxxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>8,1=>8),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'bbb+++++',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'cccxxxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>7,1=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'ccc++++',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'dddddddd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(aaa(*THEN)\w{6}|bbb(*THEN)\w{5}|ccc(*THEN)\w{4}|\w{3})',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaabccc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5,1=>5),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a+b?(*THEN)c+(*FAIL)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ABX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'AADE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3),
                        'length'=>array(0=>4,1=>3,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ACDE',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3),
                        'length'=>array(0=>4,1=>3,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3),
                        'length'=>array(0=>4,1=>3,2=>1,3=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'AD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>3),
                        'length'=>array(0=>4,1=>3,2=>1,3=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(A (A|B(*ACCEPT)|C) D)(E)',
                     'modifiers'=>'x',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
    function data_for_test_not_word_characters_in_possessive_quantifiers_and_back_and_relative_references_in_subpatterns_in_alternatives_in_unnumbered_subpattern_modifier_i() {
        $test1 = array( 'str'=>'1221',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>4,1=>4,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'Satan, oscillate my metallic sonatas!',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>37,3=>36,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'A man, a plan, a canal: Panama!',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>31,3=>30,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'Able was I ere I saw Elba.',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>26,3=>25,4=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'*** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,3=>0,4=>4),
                        'length'=>array(0=>11,3=>11,4=>1),
                        'left'=>array(1),
                        'next'=>'[fF]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'The quick brown fox',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,3=>0,4=>0),
                        'length'=>array(0=>19,3=>19,4=>1),
                        'left'=>array(1),
                        'next'=>'[tT]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\W*+(?:((.)\W*+(?1)\W*+\2|)|((.)\W*+(?3)\W*+\4|\W*+.\W*+))\W*+$',
                     'modifiers'=>'i',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }

    function data_for_test_any_character_in_subpattern_and_back_and_relative_references_and_any_character_in_alternative_in_subpattern() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>3,1=>3,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aabaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>5,1=>5,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcdcba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>7,1=>7,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'pqaabaaqp',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>9,1=>9,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'ablewasiereisawelba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>19,1=>19,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'rhubarb',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'left'=>array(1),
                        'next'=>'r',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'the quick brown fox',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'left'=>array(1),
                        'next'=>'t',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^((.)(?1)\2|.)$',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }

    function data_for_test_subpattern_and_relative_reference_in_positive_lookbehind_assertion() {
        $test1 = array( 'str'=>'baz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'caz',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)(?<=b(?1))',
                     'tests'=>array($test1, $test2, $test3));
    }

    function data_for_test_relative_reference_in_positive_lookbehind_assertion_and_subpattern() {
        $test1 = array( 'str'=>'zbaaz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>3),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=b(?1))(a)',
                     'tests'=>array($test1, $test2, $test3));
    }

/*  function data_for_test_named_subpattern_and_named_back_reference_in_positive_lookbehind_assertion() { //(?&) не поддерживается
        $test1 = array( 'str'=>'baz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<X>a)(?<=b(?&X))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_not_duplicate_subpattern_and_back_reference_2() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'defdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'[ad]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'defabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'left'=>array(3),
                        'next'=>'d',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?|(abc)|(def))\1',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_not_duplicate_subpattern_and_relative_reference() { //не дублирующиеся подмаски не поддерживаются
        $test1 = array( 'str'=>'abcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'defabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>6,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(6),
                        'next'=>'[ad]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'defdef',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?|(abc)|(def))(?1)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"a\"aaaaa",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>-1,1=>2,3=>2),
                        'length'=>array(0=>7,1=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>"b\"aaaaa",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>-1,4=>2,6=>2),
                        'length'=>array(0=>7,4=>1,6=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>-1,4=>2,6=>2),
                        'length'=>array(0=>7,4=>1,6=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>"b\"11111",
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>-1,4=>2,6=>2),
                        'length'=>array(0=>7,4=>1,6=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:a(?<quote> (?<apostrophe>')|(?<realquote>")) |b(?<quote> (?<apostrophe>')|(?<realquote>")) ) (?('quote')[a-z]+|[0-9]+)',
                     'modifiers'=>'xJ',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>2),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'CCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'CAD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?1)|B)(A(*F)|C)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'CCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'BCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ABCD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'CAD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'BAD',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:(?1)|B)(A(*F)|C)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ACD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'BAD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'BCD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'BAX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'ACX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test8 = array( 'str'=>'ABC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?1)|B)(A(*ACCEPT)XX|C)D',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7, $test8));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'BAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(DEFINE)(A))B(?1)C',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'BAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(DEFINE)((A)\2))B(?1)C',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_square_brackets_in_negative_character_class_in_possessive_quantifier_and_named_relative_reference_in_alternative_in_subpattern_in_quantifier_in_named_subpattern_modifier_x() { //модификатор x и захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'(ab(cd)ef)',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>7),
                        'length'=>array(0=>10,1=>10,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<pn> \( ( [^()]++ | (?&pn) )* \) )',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?!a(*SKIP)b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=a(*SKIP)b|ac)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=a(*THEN)b|ac)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=a(*PRUNE)b)',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=a(*ACCEPT)b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?(?!a(*SKIP)b))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_reset_start_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a\Kb)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_reset_start_in_atomic_group_in_subpattern() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>0),
                        'length'=>array(0=>1,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>a\Kb))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_reset_start_in_subpattern_2() { //\K не поддерживается
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,1=>0),
                        'length'=>array(0=>1,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a\Kb)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_reset_start_in_alternative() { //\K не поддерживается
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a\Kcz|ac',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_reset_start_in_alternative_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a\Kbz|ab)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?&t)(?(DEFINE)(?<t>a\Kb))$',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_square_brackets_in_negative_character_class_and_relative_reference_in_quantifier_in_alternative_in_subpattern_in_quantifier_3() {
        $test1 = array( 'str'=>'a(b)c',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'a(b(c)d)e',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>8),
                        'length'=>array(0=>9,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^([^()]|\((?1)*\))*$',
                     'tests'=>array($test1, $test2));
    }

/*  function data_for_test_named_subpattern_and_relative_references_in_alternative_in_named_subpattern() { //(?P>) не поддерживается
        $test1 = array( 'str'=>'0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'00',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'0000',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>4,1=>4,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?P<L1>(?P<L2>0)(?P>L1)|(?P>L2))',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_named_subpattern_and_relative_references_in_alternative_in_named_subpattern_2() { //(?P>) не поддерживается
        $test1 = array( 'str'=>'0',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>1,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'00',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>1,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'0000',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>0),
                        'length'=>array(0=>1,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?P<L1>(?P<L2>0)|(?P>L2)(?P>L1))',
                     'tests'=>array($test1, $test2, $test3));
    }
*//* This one does fail, as expected, in Perl. It needs the complex item at the
     end of the pattern. A single letter instead of (B|D) makes it not fail,
     which I think is a Perl bug. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ACABX',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*COMMIT)(B|D)',
                     'tests'=>array($test1));
    }
*//* Check the use of names for failure */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'AC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'CB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(A(*PRUNE:A)B|C(*PRUNE:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*//* Force no study, otherwise mark is not seen. The studied version is in
     test 2 because it isn't Perl-compatible. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*MARK:A)(*SKIP:B)(C|X)',
                     'modifiers'=>'KSS',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'CB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(A(*THEN:A)B|C(*THEN:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'CB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:A(*THEN:A)B|C(*THEN:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'CB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?>A(*THEN:A)B|C(*THEN:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*//* This should succeed, as the skip causes bump to offset 1 (the mark). Note
that we have to have something complicated such as (B|Z) at the end because,
for Perl, a simple character somehow causes an unwanted optimization to mess
with the handling of backtracking verbs. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*SKIP:A)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* Test skipping over a non-matching mark. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*MARK:B)(*SKIP:A)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* Check shorthand for MARK */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*:A)A+(*SKIP:A)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* Don't loop! Force no study, otherwise mark is not seen. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(),
                        'length'=>array(),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*:A)A+(*SKIP:A)(B|Z)',
                     'modifiers'=>'KSS',
                     'tests'=>array($test1));
    }
*//* This should succeed, as a non-existent skip name disables the skip */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*SKIP:B)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,MK=>-1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*SKIP:B)(B|Z) | AC(*:B)',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* COMMIT at the start of a pattern should act like an anchor. Again, 
however, we need the complication for Perl. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ABCDEFG',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2),
                        'length'=>array(0=>3,1=>1,2=>1,3=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2),
                        'length'=>array(0=>3,1=>1,2=>1,3=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'DEFGABC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,2=>1,3=>2),
                        'length'=>array(0=>3,1=>1,2=>1,3=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*COMMIT)(A|P)(B|P)(C|P)',
                     'tests'=>array($test1, $test2, $test3));
    }
*//* COMMIT inside an atomic group can't stop backtracking over the group. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abbb',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\w+)(?>b(*COMMIT))\w{2}',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abbb',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(\w+)b(*COMMIT)\w{2}',
                     'tests'=>array($test1));
    }
*//* Check opening parens in comment when seeking forward reference. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'bac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?&t)(?#()(?(DEFINE)(?<t>a))',
                     'tests'=>array($test1));
    }
*//* COMMIT should override THEN */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'yes',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(*COMMIT)(?>yes|no)(*THEN)(*F))?',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'yes',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(*COMMIT)(yes|no)(*THEN)(*F))?',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'bc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'b?(*SKIP)c',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*SKIP)bc',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*SKIP)b',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_named_back_reference_in_alternative_without_second_branch_in_named_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'xxx',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>3,1=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?P<abn>(?P=abn)xxx|)+',
                     'tests'=>array($test1));
    }

    function data_for_test_negative_character_class_in_subpattern_in_unnumbered_subpattern_modifier_i_and_relative_reference() {
        $test1 = array( 'str'=>'aa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aA',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'[^bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'aB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'[^bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'Ba',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'[^bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1,1=>1),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(1),
                        'next'=>'[^bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?i:([^b]))(?1)',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }

/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(1),
                        'next'=>'[^bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'left'=>array(1),
                        'next'=>'[^bB]',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?&t)*+(?(DEFINE)(?<t>a))\w$',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aaaaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>7),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'aaaaaa',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>6),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?&t)*(?(DEFINE)(?<t>a))\w$',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_subpattern_in_possessive_quantifier_and_word_character_in_subpattern() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'YZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>4,1=>1),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)*+(\w)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_unnumbered_subpattern_in_possessive_quantifier_and_word_character_in_subpattern() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'YZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a)*+(\w)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_subpattern_in_possessive_quantifier_and_word_character_in_subpattern_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>3),
                        'length'=>array(0=>4,1=>1),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'YZ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)++(\w)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_unnumbered_subpattern_in_possessive_quantifier_and_word_character_in_subpattern_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'YZ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(2),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a)++(\w)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_subpattern_in_possessive_quantifier_and_word_character_in_subpattern_3() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'YZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>0),
                        'length'=>array(0=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)?+(\w)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_unnumbered_subpattern_in_possessive_quantifier_and_word_character_in_subpattern_3() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'YZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a)?+(\w)',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_subpattern_in_possessive_quantifier_and_word_character_in_subpattern_4() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>3,2=>4),
                        'length'=>array(0=>5,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>1),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'YZ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a){2,}+(\w)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_unnumbered_subpattern_in_possessive_quantifier_and_word_character_in_subpattern_4() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaaX',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>4),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aaa',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(1),
                        'next'=>'\w',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'YZ',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(3),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?:a){2,}+(\w)',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
    function data_for_test_alternative_in_subpattern_in_quantifier_and_relative_reference() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a|)*(?1)b',
                     'tests'=>array($test1, $test2, $test3));
    }

/*  function data_for_test_subpattern_in_possessive_quantifier_and_relative_reference() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)++(?1)b',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_subpattern_in_possessive_quantifier_and_relative_reference_2() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>4,1=>4),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)*+(?1)b',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_relative_reference_and_subpattern_in_unnumbered_subpattern_in_null_quantifier() {
        $test1 = array( 'str'=>'b',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?1)(?:(b)){0}',
                     'tests'=>array($test1));
    }

/*  function data_for_test_square_brackets_in_negative_character_class_in_quantifier_in_atomic_group_and_relative_reference_in_alternative_in_unnumbered_subpattern_in_quantifier_in_nested_subpatterns_modifier_x() { //модификатор x не поддерживается
        $test1 = array( 'str'=>'foo(bar(baz)+baz(bop))',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3,3=>4),
                        'length'=>array(0=>22,1=>22,2=>19,3=>17),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(foo ( \( ((?:(?> [^()]+ )|(?2))*) \) ) )',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>1),
                        'length'=>array(0=>2,1=>2,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(A (A|B(*ACCEPT)|C) D)(E)',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(?:a|b(*THEN)c)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_string_and_any_character_in_lazy_quantifier_and_alternative_in_unnumbered_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(?:a|bc)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(a|b(*THEN)c)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_string_and_any_character_in_lazy_quantifier_and_alternative_in_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(a|bc)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(?:a|b(*THEN)c)++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_string_and_any_character_in_lazy_quantifier_and_alternative_in_unnumbered_subpattern_in_possessive_quantifier() { //ленивые и захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(?:a|bc)++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(a|b(*THEN)c)++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_string_and_any_character_in_lazy_quantifier_and_alternative_in_subpattern_in_possessive_quantifier() { //ленивые и захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>2,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(a|bc)++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(?:a|b(*THEN)c|d)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_string_and_any_character_in_lazy_quantifier_and_alternatives_in_unnumbered_subpattern() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\A.*?(?:a|bc|d)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_subpattern_in_unnumbered_subpattern_in_possessive_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'beetle',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(b))++',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(?=(a(*ACCEPT)z))a)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_subpattern_and_relative_reference_in_quantifier() {
        $test1 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)(?1)+ab',
                     'tests'=>array($test1));
    }

/*  function data_for_test_subpattern_and_relative_reference_in_possessive_quantifier() { //захватывающие квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aaaab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>4,1=>1),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(a)(?1)++ab',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aZbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?=a(*:M))aZ',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aZbc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?!(*:M)b)aZ',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'backgammon',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(DEFINE)(a))?b(?1)',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_not_newline_character_in_quantifier() {
        $test1 = array( 'str'=>"abc\ndef",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\N+',
                     'tests'=>array($test1));
    }

    function data_for_test_not_newline_character_in_quantifier_2() {
        $test1 = array( 'str'=>"abc\ndef ",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^\N{1,}',
                     'tests'=>array($test1));
    }

/*  function data_for_test_quantifier_and_recursion_in_alternative_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaabcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(R)a+|(?R)b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_and_recursion_in_subpattern_in_alternative_in_conditional_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaabcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?(R)a+|((?R))b)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_and_relative_reference_in_alternative_in_conditional_subpattern_in_subpattern() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaabcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?(R)a+|(?1)b))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_quantifier_and_relative_reference_in_alternative_in_conditional_subpattern_in_subpattern_2() { //условные подмаски не поддерживаются
        $test1 = array( 'str'=>'aaaabcde',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>5,1=>5),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?(R1)a+|(?1)b))',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>1,MK=>9),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(*:any \nname)',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'bba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(?&t)c|(?&t))(?(DEFINE)(?<t>a|b(*PRUNE)c))',
                     'tests'=>array($test1, $test2, $test3));
    }
*//* Checking revised (*THEN) handling *//* Capture */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (a(*THEN)b) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (a(*THEN)b|(*F)) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>1),
                        'length'=>array(0=>4,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? ( (a(*THEN)b) | (*F) ) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1,2=>1),
                        'length'=>array(0=>4,1=>2,2=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? ( (a(*THEN)b) ) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*//* Non-capture */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?:a(*THEN)b) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?:a(*THEN)b|(*F)) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?: (?:a(*THEN)b) | (*F) ) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?: (?:a(*THEN)b) ) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*//* Atomic */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?>a(*THEN)b) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?>a(*THEN)b|(*F)) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?> (?>a(*THEN)b) | (*F) ) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?> (?>a(*THEN)b) ) c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*//* Possessive capture */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (a(*THEN)b)++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1),
                        'length'=>array(0=>4,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (a(*THEN)b|(*F))++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>1,2=>1),
                        'length'=>array(0=>4,1=>2,2=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? ( (a(*THEN)b)++ | (*F) )++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>1,2=>1),
                        'length'=>array(0=>4,1=>2,2=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? ( (a(*THEN)b)++ )++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*//* Possessive non-capture */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?:a(*THEN)b)++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?:a(*THEN)b|(*F))++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?: (?:a(*THEN)b)++ | (*F) )++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*? (?: (?:a(*THEN)b)++ )++ c',
                     'modifiers'=>'x',
                     'tests'=>array($test1));
    }
*//* Condition assertion */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(?(?=a(*THEN)b)ab|ac)',
                     'tests'=>array($test1));
    }
*//* Condition */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*?(?(?=a)a|b(*THEN)c)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*?(?:(?(?=a)a|b(*THEN)c)|d)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*?(?(?=a)a(*THEN)b|c)',
                     'tests'=>array($test1));
    }
*//* Assertion */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^.*(?=a(*THEN)b)',
                     'modifiers'=>' ',
                     'tests'=>array($test1));
    }
*//*------------------------*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>a(*:m))',
                     'modifiers'=>'imsxSK ',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'a',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>(a)(*:m))',
                     'modifiers'=>'imsxSK ',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xacd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a(*ACCEPT)b)c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xacd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,1=>1),
                        'length'=>array(0=>1,1=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(a(*ACCEPT)b))c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,1=>1),
                        'length'=>array(0=>1,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3,1=>1),
                        'length'=>array(0=>1,1=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xacd',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>3,1=>1),
                        'length'=>array(0=>1,1=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=(a(*COMMIT)b))c',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'acd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<!a(*FAIL)b)c',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3,MK=>-1),
                        'length'=>array(0=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a(*:N)b)c',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a(*PRUNE)b)c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a(*SKIP)b)c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'xabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=a(*THEN)b)c',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_subpattern_and_relative_reference_in_quantifier_and_any_character_in_subpattern() {
        $test1 = array( 'str'=>'abcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,2=>3),
                        'length'=>array(0=>4,1=>1,2=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a)(?2){2}(.)',
                     'tests'=>array($test1));
    }

/*  function data_for_test_() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*MARK:A)(*PRUNE:B)(C|X)',
                     'modifiers'=>'KS',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*MARK:A)(*PRUNE:B)(C|X)',
                     'modifiers'=>'KSS',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*MARK:A)(*THEN:B)(C|X)',
                     'modifiers'=>'KS',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*MARK:A)(*THEN:B)(C|X)',
                     'modifiers'=>'KSY',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'C',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'D',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>1,1=>1,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(*MARK:A)(*THEN:B)(C|X)',
                     'modifiers'=>'KSS',
                     'tests'=>array($test1, $test2));
    }
*//* This should fail, as the skip causes a bump to offset 3 (the skip) */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(),
                        'length'=>array(),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*SKIP)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* Same */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(),
                        'length'=>array(),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*MARK:B)(*SKIP:B)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(),
                        'length'=>array(),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*:A)A+(*SKIP)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* This should fail, as a null name is the same as no name */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(),
                        'length'=>array(),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*SKIP:)(B|Z) | AC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*//* A check on what happens after hitting a mark and them bumping along to
something that does not even start. Perl reports tags after the failures here, 
though it does not when the individual letters are made into something 
more complicated. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,MK=>1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'XXYZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'XAQQ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'XAQQXZZ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test6 = array( 'str'=>'AXQQQ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test7 = array( 'str'=>'AXXQQQ',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*:A)B|XX(*:B)Y',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5, $test6, $test7));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>0),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'CD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'AC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'CB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(A(*THEN:A)B|C(*THEN:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>0),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'CD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'AC',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test5 = array( 'str'=>'CB',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(A(*PRUNE:A)B|C(*PRUNE:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3, $test4, $test5));
    }
*//* An empty name does not pass back an empty string. It is the same as if no
name were given. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0),
                        'length'=>array(0=>2,1=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'CD',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>0,MK=>-1),
                        'length'=>array(0=>2,1=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^(A(*PRUNE:)B|C(*PRUNE:B)D)',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2));
    }
*//* PRUNE goes to next bumpalong; COMMIT does not. */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ACAB',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2,MK=>2),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*PRUNE:A)B',
                     'modifiers'=>'K',
                     'tests'=>array($test1));
    }
*//* Mark names can be duplicated */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AABC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,MK=>1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'XXYZ',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1,MK=>-1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*:A)B|X(*:A)Y',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aw',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>2,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'b(*:m)f|a(*:n)w',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abaw',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test4 = array( 'str'=>'abax',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'b(*:m)f|aw',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3, $test4));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'AAAC',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'A(*MARK:A)A+(*SKIP:B)(B|Z) | AAC',
                     'modifiers'=>'xK',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'axy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(*PRUNE:X)bc|qq',
                     'modifiers'=>'KY',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'axy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a(*THEN:X)bc|qq',
                     'modifiers'=>'KY',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abxy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abpq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(*MARK:A)b)..x',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abxy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abpq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(*MARK:A)b)..(*:Y)x',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abxy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abpq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(*PRUNE:A)b)..x',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abxy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abpq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(*PRUNE:A)b)..(*:Y)x',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abxy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abpq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(*THEN:A)b)..x',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abxy',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'abpq',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0,MK=>-1),
                        'length'=>array(0=>3,MK=>1),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=a(*THEN:A)b)..(*:Y)x',
                     'modifiers'=>'K',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
    function data_for_test_subpattern_in_quantifier_and_back_reference_in_quantifier_in_subpattern() {
        $test1 = array( 'str'=>'hello world test',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>12,2=>12),
                        'length'=>array(0=>4,2=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(another)?(\1?)test',
                     'tests'=>array($test1));
    }

    function data_for_test_subpattern_in_quantifier_and_back_reference_in_quantifier_in_subpattern_2() {
        $test1 = array( 'str'=>'hello world test',
                        'is_match'=>false,
                        'full'=>false,
                        'index_first'=>array(),
                        'length'=>array(),
                        'left'=>array(18),
                        'next'=>'a',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(another)?(\1+)test',
                     'tests'=>array($test1));
    }

/*  function data_for_test_() {
        $test1 = array( 'str'=>'aac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(a(*COMMIT)b){0}a(?1)|aac',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?!a(*COMMIT)b)ac|cd',
                     'tests'=>array($test1));
    }
*/
    function data_for_test_quantifier_in_unnumbered_subpattern_in_quantifier_in_subpattern_in_quantifier() {
        $test1 = array( 'str'=>'aac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?:a?)*)*c',
                     'tests'=>array($test1));
    }

/*  function data_for_test_quantifier_in_atomic_group_in_quantifier_in_subpattern_in_quantifier() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'aac',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>2),
                        'length'=>array(0=>3,1=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'((?>a?)*)*c',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_in_atomic_group_and_positive_lookbehind_assertion() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*?a)(?<=ba)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_in_unnumbered_subpattern_and_positive_lookbehind_assertion() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aba',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:.*?a)(?<=ba)',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*?a(*PRUNE)b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*?a(*PRUNE)b',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^a(*PRUNE)b',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*?a(*SKIP)b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_in_atomic_group_modifier_s() { //модификатор s и ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*?a)b',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_in_atomic_group() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>2),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*?a)b',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_begin_of_line_in_atomic_group_modifier_s() { //модификатор s не поддерживается
        $test1 = array( 'str'=>'aab',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>1),
                        'left'=>array(1),
                        'next'=>'b',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>^a)b',
                     'modifiers'=>'s',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_in_atomic_group_and_subpatterns_in_alternative_in_positive_lookbehind_assertion() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'alphabetabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>8,1=>8),
                        'length'=>array(0=>0,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'endingwxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6,2=>6),
                        'length'=>array(0=>0,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*?)(?<=(abcd)|(wxyz))',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_atomic_group_and_subpatterns_in_alternative_in_positive_lookbehind_assertion() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'alphabetabcd',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,1=>8),
                        'length'=>array(0=>12,1=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'endingwxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0,2=>6),
                        'length'=>array(0=>10,2=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*)(?<=(abcd)|(wxyz))',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_any_character_in_quantifier_in_atomic_group() { //однократные подмаски не поддерживаются
        $test1 = array( 'str'=>'abcdfooxyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>10),
                        'left'=>array(qtype_preg_matching_results::UNKNOWN_CHARACTERS_LEFT),
                        'next'=>qtype_preg_matching_results::UNKNOWN_NEXT_CHARACTER,
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*)foo',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_any_character_in_lazy_quantifier_in_atomic_group() { //ленивые квантификаторы не поддерживаются, на жадных будет другой результат
        $test1 = array( 'str'=>'abcdfooxyz',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>.*?)foo',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 2870 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'.*?',
                     'modifiers'=>'g+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\b',
                     'modifiers'=>'g+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\b',
                     'modifiers'=>'+g',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'',
                     'modifiers'=>'g',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 5966 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abbab',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'a*',
                     'modifiers'=>'g',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 6085 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc1abc2xyzabc3',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Gabc.',
                     'modifiers'=>'g',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abc1abc2xyzabc3',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>4),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc.',
                     'modifiers'=>'g',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 6307 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>"a\nb\nc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^',
                     'modifiers'=>'mg',
                     'tests'=>array($test1, $test2));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>"A\nC\nC\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>9),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?<=C\n)^',
                     'modifiers'=>'mg',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 6547 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>"abc\n",
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>4),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'\Z',
                     'modifiers'=>'g',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 6806 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'=ba=',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?:(?>([ab])))+a=',
                     'modifiers'=>'+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'=ba=',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>1),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?>([ab]))+a=',
                     'modifiers'=>'+',
                     'tests'=>array($test1));
    }
*//* следующие тесты со строки 6860 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=abc){3}abc',
                     'modifiers'=>'+',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>0),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=abc)+abc',
                     'modifiers'=>'+',
                     'tests'=>array($test1, $test2, $test3));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abcabcabc',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'xyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>6),
                        'length'=>array(0=>3),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=abc)++abc',
                     'modifiers'=>'+',
                     'tests'=>array($test1, $test2, $test3));
    }
*//* следующие тесты со строки 7128 в testoutput1 */
/*  function data_for_test_() {
        $test1 = array( 'str'=>'Xabcdefghi',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>9),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'abc\K|def\K',
                     'modifiers'=>'g+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'Xabcdefghi',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>3),
                        'length'=>array(0=>1),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'ab\Kc|de\Kf',
                     'modifiers'=>'g+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'ABCDECBA',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>2),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'(?=C)',
                     'modifiers'=>'g+',
                     'tests'=>array($test1));
    }
*/
/*  function data_for_test_() {
        $test1 = array( 'str'=>'abcdef',
                        'is_match'=>true,
                        'full'=>true,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>0),
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test2 = array( 'str'=>'** Failers',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        $test3 = array( 'str'=>'defabcxyz',
                        'is_match'=>true,
                        'full'=>false,
                        'index_first'=>array(0=>5),
                        'length'=>array(0=>0),
                        'left'=>array(),
                        'next'=>'',
                        'tags'=>array(qtype_preg_cross_tester::TAG_FROM_PCRE));

        return array('regex'=>'^abc\K',
                     'modifiers'=>'+',
                     'tests'=>array($test1, $test2, $test3));
    }
*/}
?>